/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */

/**
 * @author Neeraj Joshi <jneeraj@us.ibm.com>
 *
 * This class is used to invoke the client side CIM-SPL Provider
 *
 */
package org.apache.imperius.cimspl.client;
import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.apache.imperius.spl.parser.exceptions.SPLException;




public class TestClient 
{
	
	public static final int INVALID_OPERATION = -1;
	public static final int LIST_POLICY = 0;
	public static final int EXECUTE_POLICY = 1;
	public static final int CREATE_POLICY = 2;
	public static final int DELETE_POLICY = 3;
	public static final int MODIFY_POLICY = 4;
	public static final int LISTALLNAMES_POLICY = 5;
	public static final int LISTALL_POLICY = 6;
	private static  String USER = "";
	private static  String PWD = "";
	private static  int PORT = 5988;
	private static  String SERVER = "localhost";
	private static  String NAMESPACE = "root/cimv2";
	private static String POLICY_NAME;
	private static String POLICY_PATH;
	private CIM_SPLPolicyRuleClientProvider _cimsplProvider = null;
	
	public TestClient(String user, String pwd, 
			int port, String server, String ns, 
			int operation) throws SPLException
	{
		
		_establishConnection();
	}

	public TestClient() throws SPLException
    {
        _establishConnection();
    }

    /**
	 * @param args
	 */
	public static void main(String[] args) 
	{
		int operation = _getInputs(args);
		TestClient ptc = null;
		try
	    {  
			try
			{
			
				ptc = new TestClient(USER,PWD,PORT,SERVER,NAMESPACE,operation);
				
			}
			catch (Exception e)
			{
				e.printStackTrace();
				System.err.println("Error: " + e.getMessage());
			    System.exit(0);  
			        
			} 
	        switch (operation)
	        {
	            case CREATE_POLICY :
	                {
	                                      
	                    System.out.println("create policy" + POLICY_NAME 
	                        + " path " + POLICY_PATH);
	                    ptc.createPolicy(POLICY_PATH, POLICY_NAME); 
	                }
	                break;
	            case LIST_POLICY :
	                {
	                    
	                    System.out.println("list policy" + POLICY_NAME);
	                    ptc.listPolicy(POLICY_NAME);
	                }
	                break;
	            case MODIFY_POLICY :
	                {
	                   
	                   System.out.println("modify policy" + POLICY_NAME 
	                        + " path " + POLICY_PATH);
	                    ptc.modifyPolicy(POLICY_PATH, POLICY_NAME);
	                }
	                break;
	            case LISTALL_POLICY :
	                {
	                    ptc.listAllPolicies();
	                    System.out.println("list all policies" );
	                }
	                break;
	            case LISTALLNAMES_POLICY :
	                {
	                    ptc.listAllPolicyNames();
	                    System.out.println("list all names");
	                }
	                break;
	            case DELETE_POLICY :
	                {
	                	System.out.println("delete policy" + POLICY_NAME);
	                    ptc.deletePolicy(POLICY_NAME);
	                    
	                }
	                break;
	            case EXECUTE_POLICY :
	                {
	                	System.out.println("exec policy" + POLICY_NAME);
	                    ptc.executePolicy(POLICY_NAME);
	                    
	                }
	        
	                break;
	            case INVALID_OPERATION :
	                	_throwUsage();
	                break;

	        }
	        ptc.shutdown();
	    }    
		catch (Exception e)
		{
		    System.err.println("Error: " + e.getMessage());
		      
		        
		} 
		/*finally 
		{
			ptc.shutdown();
		}*/
		
	}
	
	public void shutdown()
	{
		try {
			_cimsplProvider.cleanup();
		} catch (SPLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
	}
	
	public void executePolicy(String policyName) throws SPLException 
	{
		Map objMap = new HashMap();
		Object result = _cimsplProvider.executePolicy(policyName, objMap);
		System.out.println("Result of execution" + result);
	    
		
	}

	public void deletePolicy(String policyName) throws SPLException 
	{
		_cimsplProvider.deletePolicy(policyName);
         
	
		
	}

	public void listAllPolicyNames() throws SPLException 
	{
		List policyNames = _cimsplProvider.enumeratePolicyNames();
		Iterator it = policyNames.iterator();
		while (it.hasNext ())
        {
			String policyName = (String)it.next ();
           
  		   System.out.println("----------------------------");
  		   System.out.println("Policy : " + policyName);
  		   System.out.println("----------------------------");
        }
	
		
	}

	public void listAllPolicies() throws SPLException 
	{
		Map policyMap = _cimsplProvider.enumeratePolicies();
		Iterator it  = policyMap.keySet().iterator();
		
		while (it.hasNext())
        {
           String policyName = (String)it.next();
           String policyString = (String)policyMap.get(policyName);
          
		  
  		   System.out.println("----------------------------");
  		   System.out.println("Policy Name: " + policyName);
  		   System.out.println("Policy String: " + policyString);
  		   System.out.println("----------------------------");
        }

		
		
	}

	public void modifyPolicy(String policyPath, String policyName) throws SPLException 
	{
		String policyString = _readPolicyString(policyPath,policyName);
		
        _cimsplProvider.modifyPolicy(policyName, policyString);
    	
		
		
	}

	public String listPolicy(String policyName) throws SPLException 
	{
		String policyString = _cimsplProvider.getPolicy(policyName);
		
		System.out.println("Policy String: " + policyString);
        return policyString;

		
	}

	public void createPolicy(String policyPath, String policyName) throws SPLException 
	{
		 String policyString = _readPolicyString(policyPath,policyName);
		 
		
		 _cimsplProvider.createPolicy(policyName, policyString);
		 
		
		
	}

	static int _getInputs(String[] args)
	{
	    int operation = INVALID_OPERATION;
	    if (args.length < 1)
	    {
	        _throwUsage();
	    }
	    for (int i=0; i < args.length; i++)
	    {
	        if ( args[i].equalsIgnoreCase("-u")) 
	        {
	        	
                if ( i + 1 < args.length )
                {
                	USER=args[i+1];
                    i++;
                }
                else
                {
                	System.err.println("A user name is required");
                	_throwUsage();
                }

	            
            }
	        else if (args[i].equalsIgnoreCase("-pwd"))
	        {
	            if (i+1 < args.length)
                {
	            	 PWD = args[i+1];
	            	 i++;
                }
                else
                {
                	System.err.println("A pwd is required");
                	_throwUsage();
                }
	         }
	         else if (args[i].equalsIgnoreCase("-n"))
	         {
	        	
	        	 if (i +1 < args.length)
                 {
	        		 NAMESPACE = args[i+1];
	        		 i++;
                 }
                 else
                 {
                	 System.err.println("A namespace is required");
                	 _throwUsage();
                 }
	         }
	         else if (args[i].equalsIgnoreCase("-s"))
	         {
	        	
	        	 if (i +1 <  args.length)
                 {
	        		 SERVER = args[i+1];
	        		 i++;
                 }
                 else
                 {
                	 System.err.println("A servername is required");
                	 _throwUsage();
                 }
	         }
	         else if (args[i].equalsIgnoreCase("-p"))
	         {
	        	 
	        	 if (i+1 < args.length)
                 {
	        		 PORT = Integer.parseInt(args[i+1]); 
	        		 i++;
                 }
                 else
                 {
                	 System.err.println("A servername is required");
                     _throwUsage();
                 }
	         }
	         else if (args[i].equalsIgnoreCase("-l"))
             {
	        	 
                 if (i+1 < args.length)
                 {
                	 POLICY_NAME = args[i+1];
                     operation = LIST_POLICY;
                     break; 
                    
                 }
                 else
                 {
                	 
                     _throwUsage();
                 }
             }
	         else if (args[i].equalsIgnoreCase("-x"))
	         {
	        	 if (i + 1 < args.length)
                 {
                	 POLICY_NAME = args[i+1];
	                 operation = EXECUTE_POLICY;
	                 break;
                	 
                    
                 }
                 else
                 {
                	 _throwUsage();
	             }
	             
	         }
	         else if (args[i].equalsIgnoreCase("-m"))
	         {
	             if ((i+2) < args.length)
	             {
	                    POLICY_PATH = args[i+1];
	                    POLICY_NAME = args[i+2];
	                    operation = MODIFY_POLICY;
	                    break;
	             }
	             else
	             {
	                    _throwUsage();
	             }

	          }
	          else if (args[i].equalsIgnoreCase("-d"))
	          {
	        	  
	              if (i+1 < args.length)
	              {
	                  POLICY_NAME = args[i+1];
	                  operation = DELETE_POLICY;
	                  break;
	              }
	              else
	              {
	                  _throwUsage();
	              }
	          }
	          else if (args[i].equalsIgnoreCase("-i"))
	          {
	              operation = LISTALL_POLICY;
	              break;
	          }
	          else if (args[i].equalsIgnoreCase("-a"))
	          {
	              operation = LISTALLNAMES_POLICY;
	              break;
	              
	          }
	          else if (args[i].equalsIgnoreCase("-c"))
	          {
	              if ((i+2) <args.length)
	              {
	                  POLICY_PATH = args[i+1];
	                  POLICY_NAME = args[i+2];
	                  operation = CREATE_POLICY;
	                  break;
	                    
	              }
	              else
	              {
	                  _throwUsage();
	              }
	          }
	          else
	          {
	              _throwUsage();
	          }
	     }
	     if(operation == INVALID_OPERATION)
	     {
	        _throwUsage();
	     }
	    
	     return operation;
	}

	private static void _throwUsage() 
	{
		System.out.println(" Usage: PolicyClient ");
		System.out.println( "   [-s <server name>] [-p <port>] [-u <user>] [-p <pwd>] "); 
		System.out.println( "                 -l <policy name> OR " );
		System.out.println( "                 -c <policy path> <policy name> OR " );
		System.out.println( "                 -m <policy path> <policy name> OR" );
		System.out.println( "                 -d <policy name> OR" );
		System.out.println( "                 -x <policy name> OR" );
		System.out.println( "                 -i  OR" );
		System.out.println( "                 -a  " );
	    System.exit(0);

				
	}
	
	private String _readPolicyString(String path, String policyName)
	{
		StringBuffer policy = new StringBuffer();
		path = path + "/" + policyName;
		//path.concat(policyName);	  
		if(!path.endsWith(".spl"))
		{
			path = path  + ".spl";
			
		}	
		try
		{
			System.out.println("path="+path);
			FileReader input = new FileReader(path);
			  
			BufferedReader bufRead = new BufferedReader(input);
			String line;    // String that holds current file line
			int count = 0;  // Line number of count 
			line = bufRead.readLine();
			policy.append(line);
			policy.append("\n");
			count++;
			while (line != null)
			{
				line = bufRead.readLine();
				policy.append(line);
				policy.append("\n");
			    count++;
			}
			            
			bufRead.close();
		            
		}
		catch (FileNotFoundException e) 
		{
			System.err.println("Error Reading file");
			e.printStackTrace();
		} 
		catch (IOException e) 
		{
			System.err.println("Error Reading file");
			e.printStackTrace();
		}
		return policy.toString();
		

	}
	void _establishConnection() throws SPLException
    
	{
		
		
		String url = "http://" + SERVER + ":" + PORT;
		_cimsplProvider =	new CIM_SPLPolicyRuleClientProvider(url, 
				NAMESPACE, USER, PWD);
		
		
		System.out.println("Established Connection");
	}



}
