/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */

/**
 * @author Prashant Baliga <prabalig@in.ibm.com>
 *
 */

package org.apache.imperius.javaspl;

import java.util.List;
import java.util.Map;

import org.apache.imperius.spl.core.Actuator;
import org.apache.imperius.spl.core.DataCollector;
import org.apache.imperius.spl.core.InternalClient;
import org.apache.imperius.spl.manager.PolicyManager;
import org.apache.imperius.spl.manager.impl.PolicyManagerImpl;
import org.apache.imperius.spl.parser.exceptions.SPLException;
import org.apache.imperius.spl.parser.util.InternalClientFactory;

public class Java_SPLPolicyRuleProvider {

    private PolicyManager _policyManager = null;
    
    private static Java_SPLPolicyRuleProvider javaSPLPolicyRuleProvider = null;
    
    private Java_SPLPolicyRuleProvider() throws SPLException {
        initialize();
    }
    
    public static synchronized Java_SPLPolicyRuleProvider getInstance() throws SPLException {
        
        if(javaSPLPolicyRuleProvider == null) {
            
            javaSPLPolicyRuleProvider = new Java_SPLPolicyRuleProvider();
        }
        
        return javaSPLPolicyRuleProvider;
    }

//  public Java_SPLPolicyRuleProvider() throws SPLException {
//      initialize();
//  }

    public void cleanup() throws SPLException {

    }

    public void initialize() throws SPLException {

        DataCollector dc = new JavaDataCollectorImpl();
        Actuator ac = new JavaActuatorImpl();
        _policyManager = new PolicyManagerImpl(dc, ac);

        // registering internal client
        InternalClient oInternalClientObj = new InternalClient(_policyManager);
        InternalClientFactory.registerInternalClient(oInternalClientObj);

    }

    public boolean createPolicy(String name, String policyString)
            throws SPLException {

        boolean result = _policyManager.createPolicy(name, policyString);

        return result;
    }

    public boolean validatePolicyString(String policyString)
            throws SPLException {

        boolean result = _policyManager.validatePolicyString(policyString);

        return result;
    }

    public void deletePolicy(String name) throws SPLException {

        _policyManager.deletePolicy(name);

        return;
    }

    public List enumeratePolicyNames() throws SPLException {

        List policyNames = _policyManager.getAllPolicyNames();

        return policyNames;
    }

    public Map enumeratePolicies() throws SPLException {

        Map policyInstances = _policyManager.getAllPolicies();

        return policyInstances;

    }

    public String getPolicy(String policyName) throws SPLException {

        String policy = _policyManager.getPolicy(policyName);

        return policy;

    }

    public void modifyPolicy(String name, String policyString)
            throws SPLException {

        _policyManager.updatePolicy(name, policyString);

    }

    public Object executePolicy(String policyName, Map instanceObjectMap)
            throws SPLException {

        Object retVal = _policyManager.evaluatePolicy(policyName,
                instanceObjectMap);

        return retVal;
    }
    
    
    public boolean isPolicyExists(String policyName) {
        boolean returnFlag = false;
        try
        {
            List policyNamesList = _policyManager.getAllPolicyNames();
            if(policyNamesList != null && policyNamesList.size() > 0)
            {
                if(policyNamesList.contains(policyName)) {
                    returnFlag = true;
                }
            }
        } catch(SPLException splex) {
            
        }
        return returnFlag;
    }
    
	public void setLoggingEnabled(boolean enableOrDisable)
	{
		_policyManager.enableLogging(enableOrDisable);
	}

	public static void main(String[] args) {

    }

}
