/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */

// @author Neeraj Joshi <jneeraj@us.ibm.com>
 
header 
{

/*
 * 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @author Neeraj Joshi <jneeraj@us.ibm.com>
 *
 */
	package org.apache.imperius.spl.parser.compiler;
	
	import java.util.List;
	import java.util.ArrayList;

}

options 
{
    language=Java;
}

class SPLParser extends Parser;

options 
{
    exportVocab=cimspl;
    genHashLines = true;        // include line number information
    ASTLabelType="org.apache.imperius.spl.parser.compiler.ASTWithLineNumber";
    buildAST = true;     
    k = 1;          // uses CommonAST by default
}

tokens 
{
	STRATEGY_DEF; IMPORT_DEF; DECL_DEF; CONSTANT_DEFN; MACRO_DEFN; POLICY_DEF; CONDITION_DEF;
	POLICY_GRP_DEF; DECISION_DEF; TYPE; ARGUMENTS; ARG_DEF;	EXPR; ELIST; METHOD_CALL; 
	UNARY_MINUS; QUALIFIER; PARAMETERS; PARAMETER_DEF; UNARY_PLUS; ARRAY_DECLARATOR; INDEX_OP; 
	COLLECT_OP ;ASSOCIATION; ACTIONBLOCK; COLLECT_CALL ; UNARY_NOT;BASICCOLLECTION_OP;
}

{ 
	private List errorList = null; 
	
	public List getErrorList()
	{
		return errorList;
	}
}

splpolicy :
        {
        	if(errorList == null) 
        		errorList = new ArrayList();
        }
        (importstatement)+ strategystatement declarationstatement (policystatement)+
;  
exception catch [RecognitionException ex]
{
	reportError(ex);
	errorList.add(ex.getMessage());
}

importstatement : 
  //      i:"Import"^ {#i.setType(IMPORT_DEF);} IDENT (DOT! IDENT)* (COMMA! IDENT (DOT! IDENT)*)* (expression)? SEMI! 
      i:"Import"^ {#i.setType(IMPORT_DEF);}(qualifier)? "Class"!  IDENT (DOT! IDENT)*  (identList)? (expression)?  SEMI!
;
exception catch [RecognitionException ex]
{
	reportError(ex);
	errorList.add(ex.getMessage());
} 

qualifier :
	q:"Qualifier"^  {#q.setType(QUALIFIER);} IDENT
;
exception catch [RecognitionException ex]
{
	reportError(ex);
	errorList.add(ex.getMessage());
}

identList :
	c:COLON^ IDENT (COMMA! IDENT)*
;
exception catch [RecognitionException ex]
{
	reportError(ex);
	errorList.add(ex.getMessage());
}

strategystatement :
	s:"Strategy"^ {#s.setType(STRATEGY_DEF);}("Execute_All_Applicable" | "Execute_First_Applicable")  SEMI!
;  
exception catch [RecognitionException ex]
{
	reportError(ex);
	errorList.add(ex.getMessage());
}

declarationstatement :     
	(d:"Declaration"^ {#d.setType(DECL_DEF);} LCURLY! ((constantDefinitions) | (macroDeclarations))* RCURLY! )?
;
exception catch [RecognitionException ex]
{
	reportError(ex);
	errorList.add(ex.getMessage());
} 

constantDefinitions ! :
	id:IDENT v:varInitializer SEMI!
	{#constantDefinitions = #(#[CONSTANT_DEFN,"CONSTANT_DEFN"],id,v);}
;
exception catch [RecognitionException ex]
{
	reportError(ex);
	errorList.add(ex.getMessage());
} 

varInitializer :  
	( ASSIGN^ expression )?
;
exception catch [RecognitionException ex]
{
	reportError(ex);
	errorList.add(ex.getMessage());
} 

  
macroDeclarations !:
	"Macro"! LCURLY! m:macrodef RCURLY!
	{#macroDeclarations = #(#[MACRO_DEFN,"MACRO"],m);}
;
exception catch [RecognitionException ex]
{
	reportError(ex);
	errorList.add(ex.getMessage());
} 

macrodef :
	"Name"^ ASSIGN! id:IDENT SEMI!
	"type"! ASSIGN! splType SEMI! 
	 argumentList 
	"procedure"! ASSIGN! expression
;
exception catch [RecognitionException ex]
{
	reportError(ex);
	errorList.add(ex.getMessage());
} 

argumentList  :
	("argument"! ASSIGN! argument (COMMA! argument)* SEMI!)?
	{#argumentList = #(#[ARGUMENTS,"ARGUMENTS"], #argumentList);}
;
exception catch [RecognitionException ex]
{
	reportError(ex);
	errorList.add(ex.getMessage());
} 

argument ! :
	id:IDENT COLON! c:splType  
	{#argument = #(#[ARG_DEF,"ARG_DEF"],  #([TYPE,"TYPE"],c), id);}      
;
exception catch [RecognitionException ex]
{
	reportError(ex);
	errorList.add(ex.getMessage());
} 

policystatement :
	(policydef | policyGroup)   SEMI!
;
exception catch [RecognitionException ex]
{
	reportError(ex);
	errorList.add(ex.getMessage());
} 

policydef :  
	p:"Policy"^ {#p.setType(POLICY_DEF);} LCURLY! declarationstatement conditionstatement decision RCURLY! COLON! priority
;
exception catch [RecognitionException ex]
{
	reportError(ex);
	errorList.add(ex.getMessage());
} 

conditionstatement :
	(c:"Condition"^ {#c.setType(CONDITION_DEF);} LCURLY! expression RCURLY!)?
;
exception catch [RecognitionException ex]
{
	reportError(ex);
	errorList.add(ex.getMessage());
} 

decision :
	d:"Decision"^ {#d.setType(DECISION_DEF);}  LCURLY! actionBlock RCURLY!
;
exception catch [RecognitionException ex]
{
	reportError(ex);
	errorList.add(ex.getMessage());
} 

priority :
	NUM_INT
; 
exception catch [RecognitionException ex]
{
	reportError(ex);
	errorList.add(ex.getMessage());
} 

expression :
	assignmentExpression
	{#expression = #(#[EXPR,"EXPR"],#expression);}
;
exception catch [RecognitionException ex]
{
	reportError(ex);
	errorList.add(ex.getMessage());
} 

exprList :  
	expression (COMMA! expression)*
	{#exprList = #(#[ELIST,"ELIST"], exprList);}
;
exception catch [RecognitionException ex]
{
	reportError(ex);
	errorList.add(ex.getMessage());
} 

assignmentExpression : 
	conditionalExpression
	(
		ASSIGN^ assignmentExpression
	)?    
;
exception catch [RecognitionException ex]
{
	reportError(ex);
	errorList.add(ex.getMessage());
} 

conditionalExpression :
	logicalOrexpression
;
exception catch [RecognitionException ex]
{
	reportError(ex);
	errorList.add(ex.getMessage());
} 

logicalOrexpression : 
	logicalAndExpression (LOR^ logicalAndExpression)*
;
exception catch [RecognitionException ex]
{
	reportError(ex);
	errorList.add(ex.getMessage());
} 

logicalAndExpression  :  
	exclusiveOrExpression (LAND^ exclusiveOrExpression)*
;
exception catch [RecognitionException ex]
{
	reportError(ex);
	errorList.add(ex.getMessage());
} 

exclusiveOrExpression : 
	equalityExpression (BXOR^ equalityExpression)*
;
exception catch [RecognitionException ex]
{
	reportError(ex);
	errorList.add(ex.getMessage());
} 

equalityExpression : 
	relationalExpression ((NOT_EQUAL^ | EQUAL^) relationalExpression)*
;
exception catch [RecognitionException ex]
{
	reportError(ex);
	errorList.add(ex.getMessage());
} 

relationalExpression :
	additiveExpression (( LT^ | GT^ | LE^ | GE^ ) additiveExpression)*
;
exception catch [RecognitionException ex]
{
	reportError(ex);
	errorList.add(ex.getMessage());
} 

additiveExpression : 
	multiplicativeExpression ((PLUS^ | MINUS^) multiplicativeExpression)*
;
exception catch [RecognitionException ex]
{
	reportError(ex);
	errorList.add(ex.getMessage());
} 

// multiplication/division (level 2)
multiplicativeExpression :
	unaryExpression ((STAR^ | DIV^ ) unaryExpression)*
;
exception catch [RecognitionException ex]
{
	reportError(ex);
	errorList.add(ex.getMessage());
} 

unaryExpression :
	(MINUS^ {#MINUS.setType(UNARY_MINUS);} unaryExpression )| (PLUS^ {#PLUS.setType(UNARY_PLUS);} unaryExpression) | (unaryExpressionNotPlusMinus)
;
exception catch [RecognitionException ex]
{
	reportError(ex);
	errorList.add(ex.getMessage());
} 

unaryExpressionNotPlusMinus : 
	(LNOT^ {#LNOT.setType(UNARY_NOT);} unaryExpression )| (primaryExpression)
;
exception catch [RecognitionException ex]
{
	reportError(ex);
	errorList.add(ex.getMessage());
} 

primaryExpression :   
	identPrimary | constant | "true"| "false" | "Self"  | (LPAREN! assignmentExpression RPAREN!)
;
exception catch [RecognitionException ex]
{
	reportError(ex);
	errorList.add(ex.getMessage());
} 

identPrimary :
	IDENT^ ( (DOT! IDENT (indexOp | methodCall)?)* | methodCall)
	| collectOperation
	| basicCollectionOp  
;
exception catch [RecognitionException ex]
{
	reportError(ex);
	errorList.add(ex.getMessage());
} 
                  
collectOperation :
	c:"collect"^ {#c.setType(COLLECT_OP);} collectmethodcall (indexOp (methodOrPropertyOrArrayForCollect)? )?  
;
exception catch [RecognitionException ex]
{
	reportError(ex);
	errorList.add(ex.getMessage());
} 
                 
collectmethodcall :    
	LPAREN^ {#LPAREN.setType(COLLECT_CALL); }  paramList RPAREN!
;
exception catch [RecognitionException ex]
{
	reportError(ex);
	errorList.add(ex.getMessage());
} 
          
methodOrPropertyOrArrayForCollect : 
    DOT! IDENT ( indexOp | methodCall)?
;
exception catch [RecognitionException ex]
{
	reportError(ex);
	errorList.add(ex.getMessage());
} 
         
indexOp : 
	lb:LBRACK^ {#lb.setType(INDEX_OP);} expression RBRACK!
;
exception catch [RecognitionException ex]
{
	reportError(ex);
	errorList.add(ex.getMessage());
} 

basicCollectionOp : 
	lb:LBRACK^ {#lb.setType(BASICCOLLECTION_OP);} exprList RBRACK!
;
exception catch [RecognitionException ex]
{
	reportError(ex);
	errorList.add(ex.getMessage());
} 
                       
methodCall :
   lp:LPAREN^ {#lp.setType(METHOD_CALL);} paramList RPAREN!
;            
exception catch [RecognitionException ex]
{
	reportError(ex);
	errorList.add(ex.getMessage());
} 

constant : 
		// DATE_TIME
		//|   
		NUM_INT
        |   CHAR_LITERAL
        |   STRING_LITERAL
        |   NUM_FLOAT
        |   NUM_LONG
        |   NUM_DOUBLE
;
exception catch [RecognitionException ex]
{
	reportError(ex);
	errorList.add(ex.getMessage());
} 

//dateTime : NUM_INT DOT NUM_INT ( (PLUS | MINUS) NUM_INT | COLON NUM_INT)
       //   ;

paramList : 
	(exprList)? 
;
exception catch [RecognitionException ex]
{
	reportError(ex);
	errorList.add(ex.getMessage());
} 

actionBlock :
	actionBlockExpr ( (ARROW^ | LAND^ | LOR^ | BOR^) actionBlockExpr)*
;
exception catch [RecognitionException ex]
{
	reportError(ex);
	errorList.add(ex.getMessage());
} 

actionBlockExpr :
	basicActionBlock | LPAREN! actionBlock RPAREN!
;
exception catch [RecognitionException ex]
{
	reportError(ex);
	errorList.add(ex.getMessage());
} 

basicActionBlock :
	// (IDENT | "Set" | "InvokePolicy" ) (DOT! IDENT)? (methodCall) ((EQUAL | NOT_EQUAL | GE | GT | LT | LE) constant)?
	(
		// ( IDENT (DOT! IDENT (methodCall))* )
		IDENT^ ( (DOT! IDENT (indexOp | methodCall)?)* | methodCall)
		|
		( ("Set" | "InvokePolicy") (DOT! IDENT)? (methodCall) )
	)
	(
		(EQUAL | NOT_EQUAL | GE | GT | LT | LE) constant
	)?
	{#basicActionBlock = #(#[ACTIONBLOCK,"ACTIONBLK"],basicActionBlock );}
;  
exception catch [RecognitionException ex]
{
	reportError(ex);
	errorList.add(ex.getMessage());
} 

policyGroup :
	p:"PolicyGroup"^ {#p.setType(POLICY_GRP_DEF);} assocName LCURLY! splpolicy RCURLY! COLON! priority
;
exception catch [RecognitionException ex]
{
	reportError(ex);
	errorList.add(ex.getMessage());
} 

assocName :
	COLON! IDENT^ LPAREN! IDENT COMMA! IDENT RPAREN!
	{#assocName = #(#[ASSOCIATION,"ASSOC"], assocName);}
;
exception catch [RecognitionException ex]
{
	reportError(ex);
	errorList.add(ex.getMessage());
} 

splType :
	basicSplTypes (LBRACK RBRACK)?
;
exception catch [RecognitionException ex]
{
	reportError(ex);
	errorList.add(ex.getMessage());
} 

        
basicSplTypes :        
	"Sint8" | "Sint16" | "Uint8" | "Uint16" | 
 	"Sint32" | "Sint64" | "Uint32" | "Uint64" |
    "Real32" | "Real64" | "Boolean" | "Char16"|
    "String" | "DateTime" | "Reference" 
;     
exception catch [RecognitionException ex]
{
	reportError(ex);
	errorList.add(ex.getMessage());
} 
 

class SPLLexer extends Lexer;

options {
    exportVocab=cimspl;      // call the vocabulary "Java"
    testLiterals=false;    // don't automatically test for literals
    k=4;                   // four characters of lookahead
    // can't deal with 16 bit chars

    charVocabulary='\u0003'..'\u00FF';

}

// OPERATORS

LPAREN      :   '('     ;
RPAREN      :   ')'     ;
LBRACK      :   '['     ;
RBRACK      :   ']'     ;
LCURLY      :   '{'     ;
RCURLY      :   '}'     ;
COLON       :   ':'     ;
COMMA       :   ','     ;
DOT		    :	'.'		;
ASSIGN      :   '='     ;
EQUAL       :   "=="    ;
LNOT        :   '!'     ;
NOT_EQUAL   :   "!="    ;
DIV     :   '/'     ;
PLUS        :   '+'     ;
MINUS       :   '-'     ;
//DATE_TIME   : (('0'..'9'|'*') ('0'..'9'|'*')('0'..'9'|'*')('0'..'9'|'*')('0'..'1'|'*')('0'..'9'|'*')('0'..'3'|'*')('0'..'9'|'*')('0'..'6'|'*')('0'..'9'|'*')('0'..'6'|'*')('0'..'9'|'*')('0'..'6'|'*')('0'..'9'|'*')('.')('0'..'9'|'*')('0'..'9'|'*')('0'..'9'|'*')('0'..'9'|'*')('0'..'9'|'*')('0'..'9'|'*')(PLUS|MINUS|':')('0'..'9'|'*')('0'..'9'|'*')('0'..'9'|'*')){_ttype = DATE_TIME;}
			  
// ;
STAR        :   '*'     ;
GE      :   ">="    ;
GT      :   ">"     ;
LE      :   "<="    ;
LT     :   '<'     ;
BXOR        :   '^'     ;
BOR     :   '|'  
   ;
LOR     :   "||"    ;
LAND        :   "&&"    ;
SEMI        :   ';'     ;
ARROW   : "->";
          // Whitespace -- ignored
WS	:	(	' '
		|	'\t'
		|	'\f'
			// handle newlines
		|	(	options {generateAmbigWarnings=false;}
			:	"\r\n"  // Evil DOS
			|	'\r'    // Macintosh
			|	'\n'    // Unix (the right way)
			)
			{ newline(); }
		)+
		{ _ttype = Token.SKIP; }
	;

// Single-line comments
SL_COMMENT
	:	"//"
		(~('\n'|'\r'))* ('\n'|'\r'('\n')?)?
		{$setType(Token.SKIP); newline();}
	;

// multiple-line comments
ML_COMMENT
	:	"/*"
		(	/*	'\r' '\n' can be matched in one alternative or by matching
				'\r' in one iteration and '\n' in another.  I am trying to
				handle any flavor of newline that comes in, but the language
				that allows both "\r\n" and "\r" and "\n" to all be valid
				newline is ambiguous.  Consequently, the resulting grammar
				must be ambiguous.  I'm shutting this warning off.
			 */
			options {
				generateAmbigWarnings=false;
			}
		:
			{ LA(2)!='/' }? '*'
		|	'\r' '\n'		{newline();}
		|	'\r'			{newline();}
		|	'\n'			{newline();}
		|	~('*'|'\n'|'\r')
		)*
		"*/"
		{$setType(Token.SKIP);}
	;


// character literals
CHAR_LITERAL
	:	'\'' ( ESC | ~('\''|'\n'|'\r'|'\\') ) '\''
	;

// string literals
STRING_LITERAL
	:	'"' (ESC|~('"'|'\\'|'\n'|'\r'))* '"'
	;



// escape sequence -- note that this is protected; it can only be called
//   from another lexer rule -- it will not ever directly return a token to
//   the parser
// There are various ambiguities hushed in this rule.  The optional
// '0'...'9' digit matches should be matched here rather than letting
// them go back to STRING_LITERAL to be matched.  ANTLR does the
// right thing by matching immediately; hence, it's ok to shut off
// the FOLLOW ambig warnings.
protected
ESC
	:	'\\'
		(	'n'
		|	'r'
		|	't'
		|	'b'
		|	'f'
		|	'"'
		|	'\''
		|	'\\'
		|	('u')+ HEX_DIGIT HEX_DIGIT HEX_DIGIT HEX_DIGIT
		|	'0'..'3'
			(
				options {
					warnWhenFollowAmbig = false;
				}
			:	'0'..'7'
				(
					options {
						warnWhenFollowAmbig = false;
					}
				:	'0'..'7'
				)?
			)?
		|	'4'..'7'
			(
				options {
					warnWhenFollowAmbig = false;
				}
			:	'0'..'7'
			)?
		)
	;


// hexadecimal digit (again, note it's protected!)
protected
HEX_DIGIT
	:	('0'..'9'|'A'..'F'|'a'..'f')
	;


// an identifier.  Note that testLiterals is set to true!  This means
// that after we match the rule, we look in the literals table to see
// if it's a literal or really an identifer
IDENT
	options {testLiterals=true;}
	:	('a'..'z'|'A'..'Z'|'_'|'$') ('a'..'z'|'A'..'Z'|'_'|'0'..'9'|'$'|'/')*
	;
                     
//DATE_TIME
 //  : ('0'..'9'|STAR) ('0'..'9'|STAR)('0'..'9'|STAR)('0'..'9'|STAR)('0'..'1'|STAR)('0'..'9'|STAR)('0'..'3'|STAR)('0'..'9'|STAR)('0'..'6'|STAR)('0'..'9'|STAR)('0'..'6'|STAR)('0'..'9'|STAR)('0'..'6'|STAR)('0'..'9'|STAR)(DOT)('0'..'9'|STAR)('0'..'9'|STAR)('0'..'9'|STAR)('0'..'9'|STAR)('0'..'9'|STAR)('0'..'9'|STAR)(PLUS|MINUS)('0'..'9'|STAR)('0'..'9'|STAR)('0'..'9'|STAR){_ttype = DATE_TIME;}
 //   
//	 ;
// a numeric literal
NUM_INT
	{boolean isDecimal=false; Token t=null;}
    :   '.' {_ttype = DOT;}
            (	('0'..'9')+ (EXPONENT)? (f1:FLOAT_SUFFIX {t=f1;})?
                {
				if (t != null && t.getText().toUpperCase().indexOf('F')>=0) {
                	_ttype = NUM_FLOAT;
				}
				else {
                	_ttype = NUM_DOUBLE; // assume double
				}
				}
            )?

	|	(	'0' {isDecimal = true;} // special case for just '0'
			(	('x'|'X')
				(											// hex
					// the 'e'|'E' and float suffix stuff look
					// like hex digits, hence the (...)+ doesn't
					// know when to stop: ambig.  ANTLR resolves
					// it correctly by matching immediately.  It
					// is therefor ok to hush warning.
					options {
						warnWhenFollowAmbig=false;
					}
				:	HEX_DIGIT
				)+

			|	//float or double with leading zero
				(('0'..'9')+ ('.'|EXPONENT|FLOAT_SUFFIX)) => ('0'..'9')+

			|	('0'..'7')+									// octal
			)?
		|	('1'..'9') ('0'..'9')*  {isDecimal=true;}		// non-zero decimal
		)
		(	('l'|'L') { _ttype = NUM_LONG; }

		// only check to see if it's a float if looks like decimal so far
		|	{isDecimal}?
            (   '.' ('0'..'9')* (EXPONENT)? (f2:FLOAT_SUFFIX {t=f2;})?
            |   EXPONENT (f3:FLOAT_SUFFIX {t=f3;})?
            |   f4:FLOAT_SUFFIX {t=f4;}
            )
            {
			if (t != null && t.getText().toUpperCase() .indexOf('F') >= 0) {
                _ttype = NUM_FLOAT;
			}
            else {
	           	_ttype = NUM_DOUBLE; // assume double
			}
			}
        )?
	;


// a couple protected methods to assist in matching floating point numbers
protected
EXPONENT
	:	('e'|'E') ('+'|'-')? ('0'..'9')+
	;


protected
FLOAT_SUFFIX
	:	'f'|'F'|'d'|'D'
	;


