/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */


/**
 * @author Prashant Baliga <prabalig@in.ibm.com>
 * This interface needs to be implemented by 
 * the binding provider.
 * The impl is used to perform actions or call
 * methods
 */



package org.apache.imperius.spl.core;



import java.util.List;
import java.util.Map;

import org.apache.imperius.spl.parser.exceptions.SPLException;




public interface Actuator 
{
    /**
     * Invokes the method on the given object with given argument list
     * argumentList is a list of Argument objects and  
     * @param className
     * @param qualifier
     * @param instance
     * @param methodName
     * @param argTypeList - list of 
     * @param argumentList
     * @return
     * @throws SPLException
     */
	public Object invokeMethod(String className, 
			String qualifier,
			Object instance, 
			String methodName, 
			List argumentList) throws SPLException ;
	/**
	 * Modifies the given instance with Map of new property values
	 * @param className
	 * @param qualifier
	 * @param instance
	 * @param memberMap
	 * @throws SPLException
	 */
	public void modifyInstance(String className,
			String qualifier,
			Object instance,
			Map memberMap) throws SPLException ;
	
	/**
	 * Invokes a policy with the given name and list of input objects
	 * @param policyName
	 * @param objects
	 * @return
	 * @throws SPLException
	 */

}
