/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */


/**
 * @author Neeraj Joshi <jneeraj@us.ibm.com>
 * All SPL Expressions need to implement this interface
 */

package org.apache.imperius.spl.core;

import org.apache.imperius.spl.parser.exceptions.SPLException;




public interface Expression
{
    /**
     * Evaluates the expression and return a result
     * @return
     * @throws SPLException
     */
    Object evaluate() throws SPLException;
    
    /**
     * Get the return type of the expression
     * @return
     * @throws SPLException
     */
    TypeInfo getType() throws SPLException;
    
    /**
     * Validate the expression for semantic correctness
     * @return
     * @throws SPLException
     */
    boolean validate() throws SPLException;
    
    /**
     * Provides a string representation of the expression
     * @return
     */
    String toString();
    
    /**
     * is true if the expression evaluates to an array
     * @return
     */
    boolean isArray();
    
    /**
     * Applicable only in the case of complex return types
     * provides the class name of the complex type
     * @return
     * @throws SPLException
     */
    
    String getReferenceTypeName() throws SPLException;
    
    
}
