/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */


/**
 * @author Neeraj Joshi <jneeraj@us.ibm.com>
 *
 * Constants for types supported by SPL
 */
package org.apache.imperius.spl.core;

import org.apache.imperius.spl.parser.exceptions.SPLException;

public class TypeConstants
{
    
    public static final int INVALID = -1;
    public static final int booleanType = 1;
    
    public static final int stringType = 2;
    
    public static final int numericType = 3;
    
    public static final int dateTime = 4;
    
    public static final int byteType = 5;
    
    public static final int charType = 6;
    
    public static final int shortType = 7;
    
    public static final int intType = 8;
    
    public static final int longType = 9;
    
    public static final int floatType = 10;
    
    public static final int doubleType = 11;
    
    //public static final int dateTime = 12;
    
    public static final int referenceType = 13;
    
   
    
    public static int getActualType(String type) throws SPLException
    {
        
        
        int retTp = -1;
        
        if (type.equals("Uint8") || type.equals("Sint8"))
        {
            retTp = TypeConstants.byteType;
        }
        else if (type.equals("Uint16") || type.equals("Sint16"))
        {
            retTp = TypeConstants.shortType;
        }
        else if (type.equals("Uint32") || type.equals("Sint32"))
        {
            retTp = TypeConstants.intType;
        }
        else if (type.equals("Uint64") || type.equals("Sint64"))
        {
            retTp = TypeConstants.longType;
        }
        else if (type.equals("Real32"))
        {
            retTp = TypeConstants.floatType;
        }
        else if (type.equals("Real64"))
        {
            retTp = TypeConstants.doubleType;
        }
        else if (type.equals("Boolean"))
        {
            retTp = TypeConstants.booleanType;
        }
        else if (type.equals("Char16"))
        {
            retTp = TypeConstants.charType;
        }
        else if (type.equals("String"))
        {
            retTp = TypeConstants.stringType;
        } // CHANGED from CIMDateTime to SPLDateTime
        else if (type.equals("DateTime"))
        {
            retTp = TypeConstants.dateTime;
        }
        else if (type.equals("Reference") || type.equals("Reference[]"))
        {
            retTp = TypeConstants.referenceType;
        }
        
        if(retTp ==-1)
        {
           
            throw new SPLException("invalid type");
            
        }
        
      
        return retTp;
    }
    
   
    
}
