/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */

/**
 * @author Prashant Baliga <prabalig@in.ibm.com>
 *
 */

package org.apache.imperius.spl.datastore.impl;

import java.util.Hashtable;
import java.util.Map;

import org.apache.imperius.spl.parser.exceptions.SPLException;
import org.apache.imperius.spl.parser.statements.impl.SPLPolicy;
import org.apache.imperius.util.Messages;

class PolicyCache 
{
	private Map _policyCache = new Hashtable();
	private static PolicyCache singletonObj = null;

	private PolicyCache() 
	{

	}

	public static synchronized PolicyCache getInstance() 
	{
		if (singletonObj == null) 
		{
			singletonObj = new PolicyCache();
		}
		return singletonObj;
	}

	public boolean policyExists(String policyName)
	{
		if (!_policyCache.containsKey(policyName)) 
		{
			return false;
		}
		return true;
	}

	public synchronized boolean insertPolicy(String policyName,
			SPLPolicy policyObject) 
	{
		if (!policyExists(policyName)) 
		{
			_policyCache.put(policyName, policyObject);
			return true;
		} 
		else 
		{
			return false;
		}
	}

	public boolean updatePolicy(String policyName, SPLPolicy policyObject)
			throws SPLException 
	{
		if (policyExists(policyName)) 
		{
			_policyCache.remove(policyName);
			_policyCache.put(policyName, policyObject);
			return true;
		} 
		else 
		{
			throw new SPLException(
					Messages
							.getString("SPL_POLICY_DOES_NOT_EXIST_IN_CACHE_EXCEPTION_MSG"));
		}
	}

	public synchronized boolean deletePolicy(String policyName)
			throws SPLException 
	{
		if (policyExists(policyName)) 
		{
			_policyCache.remove(policyName);
			return true;
		} 
		else 
		{
			return false;
		}
	}

	public synchronized SPLPolicy getPolicy(String policyName)
			throws SPLException 
	{
		if (policyExists(policyName)) 
		{
			SPLPolicy pol = (SPLPolicy) _policyCache.get(policyName);
			return pol;
		} 
		else 
		{
			throw new SPLException(
					Messages
							.getString("SPL_POLICY_DOES_NOT_EXIST_IN_CACHE_EXCEPTION_MSG"));
		}
	}
}
