/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */


/**
 * @author Prashant Baliga <prabalig@in.ibm.com>
 *
 */

package org.apache.imperius.spl.datastore.impl;

import org.apache.imperius.spl.parser.exceptions.SPLException;
import org.apache.imperius.spl.parser.statements.impl.SPLPolicy;


public interface PolicyParser
{
    
    public SPLPolicy createInternalPolicyObject(String PolicyRuleName,
            String PolicyString) throws SPLException;
    
    public boolean validate(String PolicyRuleName, String PolicyString);
    
    public void init() throws SPLException;
    
    public void shutDown();
    
    /**
     * Determine if the given policy string is properly formatted and parsable.
     * @param policyString the SPL policy.
     * @return true if the the string is parsable, false otherwise.
     * @throws SPLException // TODO: why do we need to throw an exception here?
     */
    public boolean validatePolicyString(String policyString) throws SPLException;
    
}
