/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */


/**
 * @author Prashant Baliga <prabalig@in.ibm.com>
 *
 */

package org.apache.imperius.spl.datastore.impl;



import java.util.List;
import java.util.Map;

import org.apache.imperius.spl.parser.exceptions.SPLException;

interface PolicyRepository
{
	public static final String REPOSITORY_FILE_NAME = "PolicyStorage.DAT"; //$NON-NLS-1$
	
    public String getPolicyString(String PolicyRuleName) throws SPLException;
            
    
    public boolean insertPolicy(String PolicyRuleName, String PolicyString)
            throws SPLException;
    
    public void updatePolicy(String PolicyRuleName, String PolicyString)
            throws SPLException;
    
    public void deletePolicy(String PolicyRuleName) throws SPLException;
    
    /**
     * Get a map of that maps from policy names to the policy strings.
     * @return
     * @throws SPLException
     */
    public Map getAllPolicies() throws SPLException;
    
    public List getAllPolicyNames() throws SPLException;
    
    public boolean policyExists(String PolicyRuleName) throws SPLException;
    
    public void init() throws SPLException;
    
    public void shutdown();
    
    
    
}
