/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */
package org.apache.imperius.spl.parser.compiler.symboltable;

import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.logging.Logger;

import org.apache.imperius.spl.core.InstanceInfo;
import org.apache.imperius.spl.parser.exceptions.SPLException;
import org.apache.imperius.util.SPLLogger;

public class AnchorData 
{
	private static Logger logger = SPLLogger.getSPLLogger().getLogger();
	private static final String sourceClass = "AnchorData";
	private Map _classNameToQualifierMap;
	private Map _classNameToInstanceInfoListMap;
	
	public AnchorData()
	{
		logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "AnchorData");
		
		_classNameToQualifierMap = new Hashtable();
		_classNameToInstanceInfoListMap = new Hashtable();
		
		logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "AnchorData");
	}
	
	public List getAnchorClassList()
	{
		logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "getAnchorClassList");

		List anchorClassList = new ArrayList();
		Iterator anchorKeySetIt = _classNameToInstanceInfoListMap.keySet().iterator();
		while(anchorKeySetIt.hasNext())
		{
			String className = (String)anchorKeySetIt.next();
			anchorClassList.add(className);
		}
		
		logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "getAnchorClassList");
		return anchorClassList;
	}
	
	public Object getInstance(String instanceName) 
	{
		logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "getInstance");

		Object instance = null;
		Iterator anchorKeySetIt = _classNameToInstanceInfoListMap.keySet().iterator();
		while(anchorKeySetIt.hasNext())
		{	
			String className = (String)anchorKeySetIt.next();
			List instanceInfoList = (List)_classNameToInstanceInfoListMap.get(className);
			Iterator instanceInfoListIt = instanceInfoList.iterator();
			while(instanceInfoListIt.hasNext())
			{
				InstanceInfo ii = (InstanceInfo)instanceInfoListIt.next();
				if(ii.getInstanceName().equals(instanceName))
				{
					instance = ii.getInstance();
					return instance;
				}
			}
		}	
		
		logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "getInstance");
		return instance;
	}
	
	public void insertInstanceObject(String className, 
			String instanceName, 
			Object instance) throws SPLException 
	{
		
		List instanceInfoList = (List)_classNameToInstanceInfoListMap.get(className);
		boolean instanceFound = false;
		
		Iterator instanceInfoListIt = instanceInfoList.iterator();
		while(instanceInfoListIt.hasNext())
		{
			InstanceInfo instanceInfo = (InstanceInfo) instanceInfoListIt.next();
			if(instanceInfo.getInstanceName() != null)
			{	
				if(instanceInfo.getInstanceName().equalsIgnoreCase(instanceName))
				{
					instanceInfo.setInstance(instance);
					instanceFound = true;
					break;
				}
			}
			else 
			{
			}
		}
		if(!instanceFound)
		{
			throw new SPLException("Invalid instance Name " + instanceName);
		}
	}
	
	public String getQualifierForClass(String className)
	{
		String qualifier = (String)_classNameToQualifierMap.get(className);
		return qualifier;
	}
	
	public List getInstanceInfoList(String className)
	{
		List instanceInfoList = (List)_classNameToInstanceInfoListMap.get(className);
		return instanceInfoList;
	}
	
	public void addInstanceInfoListForClass(String className, List instanceInfoList)
	{
		_classNameToInstanceInfoListMap.put(className, instanceInfoList);
	}
	
	public void addQualifierForClass(String className, String qualifier)
	{
		if(qualifier != null)
		{	
			_classNameToQualifierMap.put(className, qualifier);
		}	
	}
	
}
