/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */

/**
 * @author Prashant Baliga <prabalig@in.ibm.com>
 *
 */

package org.apache.imperius.spl.parser.compiler.symboltable;

import java.util.List;
import java.util.logging.Logger;

import org.apache.imperius.spl.core.TypeInfo;
import org.apache.imperius.spl.parser.exceptions.SPLException;
import org.apache.imperius.spl.parser.statements.impl.MacroDefinition;
import org.apache.imperius.spl.parser.util.TypeInfoImpl;
import org.apache.imperius.util.SPLLogger;

public class MacroSymbol implements Symbol 
{
	private String _macroName;
	private TypeInfoImpl _returnType = new TypeInfoImpl();
	private List _argTypeList;
	private MacroDefinition _macroDefn;
	private static Logger logger = SPLLogger.getSPLLogger().getLogger();
	private static final String sourceClass = "MacroSymbol";
	private boolean _isArray = false;

	public MacroSymbol(String macroName, TypeInfo retType, List argTypes,
			MacroDefinition md) throws SPLException 
	{
		logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "MacroSymbol");

		this._macroName = macroName;
		this._returnType.copy(retType);
		_returnType.setReferenceTypeName(md.getReferenceTypeName());
		this._argTypeList = argTypes;
		this._macroDefn = md;
		if (md.isArray()) 
		{
			this._isArray = true;
		}

		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "MacroSymbol");
	}

	public TypeInfo getType() 
	{
		logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "getType");

		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "getType");

		return _returnType;
	}

	public Object getValue() 
	{
		logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "getValue");

		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "getValue");

		return null;
	}

	public List getArgTypeList() 
	{
		logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "getArgTypeList");
	
		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "getArgTypeList");

		return _argTypeList;
	}

	public void setArgTypeList(List argTypeList) 
	{
		logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "setArgTypeList");

		_argTypeList = argTypeList;
	
		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "setArgTypeList");
	}

	public MacroDefinition getMacroDefn() 
	{
		logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "getMacroDefn");
	
		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "getMacroDefn");

		return _macroDefn;
	}

	public void setMacroDefn(MacroDefinition macroDefn) 
	{
		logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "setMacroDefn");

		_macroDefn = macroDefn;
	
		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "setMacroDefn");
	}

	public String getMacroName() 
	{
		logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "getMacroName");

		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "getMacroName");

		return _macroName;
	}

	public void setMacroName(String macroName) 
	{
		logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "setMacroName");
	
		_macroName = macroName;

		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "setMacroName");
	}

	public void setReturnType(TypeInfo returnType) 
	{
		logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "setReturnType");

		_returnType.copy(returnType);
		
		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "setReturnType");
	}

	public boolean isArray() 
	{
		logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "isArray");

		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "isArray");

		return _returnType.getIsArray();
	}

	public boolean isKey() 
	{
		logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "isKey");

		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "isKey");

		return false;
	}

	public boolean isClassSymbol() 
	{
		logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "isClassSymbol");

		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "isClassSymbol");

		return false;
	}

	public boolean isArray(boolean isArray) 
	{
		return _returnType.getIsArray();
	}

	public void setArray(boolean isArray) 
	{
		_returnType.setIsArray(isArray);
	}

	public String getReferenceTypeName() 
	{
		return _returnType.getReferenceTypeName();
	}
}
