/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */


/**
 * @author Prashant Baliga <prabalig@in.ibm.com>
 *
 */



package org.apache.imperius.spl.parser.compiler.symboltable;

import java.util.List;

import org.apache.imperius.spl.core.TypeInfo;
import org.apache.imperius.spl.parser.exceptions.SPLException;
import org.apache.imperius.spl.parser.util.TypeInfoImpl;




public class MethodSymbol implements Symbol 
{
	private String _methodName;
	private TypeInfoImpl _returnType = new TypeInfoImpl();
	private List _argList;
	private SPLSymbolTable _methodSymbolTable=null;
	
	public MethodSymbol(String nName, 
			int retType, 
			boolean isReturnArray, 
			String returnReferenceType, 
			List argList,
			SPLSymbolTable metSym) throws SPLException
	{
		
		
		_methodName = nName;
		_returnType.setIsArray(isReturnArray);
		_returnType.setReferenceTypeName(returnReferenceType);
		_returnType.setType(retType);
		//_returnType = new TypeInfoImpl(retType,returnReferenceType,isReturnArray);
		_argList=argList;
		_methodSymbolTable = metSym;
		
	}
	
	public MethodSymbol(String nName, 
			TypeInfo retType, 
			List argTypeList,
			SPLSymbolTable argSymbols)
	{
		
		
		_methodName = nName;
		_returnType.copy(retType);
		_argList=argTypeList;
		_methodSymbolTable=argSymbols;
		
	}
	
	public String getMethodName() 
	{
		
		return _methodName;
	}


	public void setMethodName(String methodName) 
	{
			
		_methodName = methodName;
	
		
	}


	public TypeInfo getType() 
	{
			
		return _returnType;
	}

	public Object getValue() 
	{
	
		return null;
	}


	public boolean isArray() 
	{
	
		return false;
	}


	public boolean isKey() 
	{
	
		return false;
	}


	public boolean isClassSymbol() 
	{
	
		return true;
	}


	public List getArgumentList() 
	{
	
		
		return _argList;
	}


	public void setArgumentList(List argList) 
	{
	
		
		_argList = argList;
	
		
	}


	public String getReferenceTypeName() 
	{
			return _returnType.getReferenceTypeName();
	}

	public boolean isReturnArray() 
	{
		return _returnType.getIsArray();
	}

	

	public TypeInfo getReturnType() {
		return _returnType;
	}

	public SPLSymbolTable getMethodSymbolTable() {
		return _methodSymbolTable;
	}

	public void getMethodSymbolTable(SPLSymbolTable symbolTable) {
		_methodSymbolTable = symbolTable;
	}

}
