/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */


/**
 * @author Prashant Baliga <prabalig@in.ibm.com>
 *
 */



package org.apache.imperius.spl.parser.compiler.symboltable;

import org.apache.imperius.spl.core.TypeInfo;
import org.apache.imperius.spl.parser.exceptions.SPLException;
import org.apache.imperius.spl.parser.util.TypeInfoImpl;





public class PropertySymbol implements Symbol 
{
	
	private TypeInfoImpl _dataType = new TypeInfoImpl();
	private String _name;
	private Object _value;
	//private boolean _isArray;
	private boolean _isKey;
	//private String _referenceTypeName;
	private boolean _isPropertyOfClass;
	
	public PropertySymbol(String n, 
			int type, 
			String referenceType, 
			boolean isArray, 
			boolean isKey, 
			boolean isProp) throws SPLException
	{
		_dataType = new TypeInfoImpl(type,referenceType,isArray);
		_name = n;
		_isKey = isKey;
		_isPropertyOfClass = isProp;
	}
	
	public PropertySymbol(
			String n, 
			TypeInfo tp ,
			boolean isKey, 
			boolean isProp)
	{
		
		this._name = n;
		this._dataType.copy(tp);
		
		this._isKey = isKey;
		this._isPropertyOfClass = isProp;
		
		
	}

	public TypeInfo getType() 
	{
		
		return _dataType;
	}
	
	public Object getValue() 
	{
		
		return _value;
	}
	
	public void setValue(Object o) 
	{
		
		_value = o;
	}


	public String getName() 
	{
			
		return _name;
	}


	public boolean isArray() 
	{
		
		return _dataType.getIsArray();
	}


	public boolean isKey() 
	{
			
		return _isKey;
	}

	public boolean isClassSymbol() 
	{
	
		return _isPropertyOfClass;
	}

	public String getReferenceTypeName() 
	{
		
		return _dataType.getReferenceTypeName();
	}

}
