/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */
//

/**
 * @author Neeraj Joshi <jneeraj@us.ibm.com>
 *
 */

package org.apache.imperius.spl.parser.expression.primary;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.logging.Logger;

import org.apache.imperius.spl.core.Expression;
import org.apache.imperius.spl.core.TypeConstants;
import org.apache.imperius.spl.core.TypeInfo;
import org.apache.imperius.spl.parser.compiler.symboltable.MethodSymbol;
import org.apache.imperius.spl.parser.compiler.symboltable.PropertySymbol;
import org.apache.imperius.spl.parser.compiler.symboltable.SPLSymbolTable;
import org.apache.imperius.spl.parser.compiler.symboltable.Symbol;
import org.apache.imperius.spl.parser.exceptions.IllegalExpressionTypeException;
import org.apache.imperius.spl.parser.exceptions.SPLException;
import org.apache.imperius.spl.parser.util.TypeInfoImpl;
import org.apache.imperius.util.Messages;
import org.apache.imperius.util.SPLLogger;


public class PostCollectExpression implements Expression
{
	private Expression arrExp1;
	private String methodOrParam;
	private boolean methodOrParamExists = false;
	private boolean array2Exists = false;
	private Expression arrExp2;
	private ArrayList pList;
	private SPLSymbolTable symTab;
	private static Logger logger = SPLLogger.getSPLLogger().getLogger();
	private static final String sourceClass = "PostCollectExpression";
	private TypeInfoImpl _dataType = new TypeInfoImpl();
	public boolean isIdentifierMethod;
	public int instanceNumer = -1;
	private String targetName = "";

	public boolean isArray()
	{
		logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "isArray");

		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "isArray");

		return this._dataType.getIsArray();
	}

	public void setArray(boolean isArray)
	{
		logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "setArray");
		this._dataType.setIsArray(isArray);

		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "setArray");
	}

	public PostCollectExpression(Expression a1, String m, Expression a2,
			ArrayList pList, SPLSymbolTable sTab) throws SPLException
	{
		logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "PostCollectExpression");

		this.arrExp1 = a1;
		this.arrExp2 = a2;
		this.methodOrParam = m;
		this.pList = pList;
		this.symTab = sTab;
	}

	// (arrExp1 = arrayIndex[cSymTab] ( id:IDENT ( #(INDEX_OP
	// arrExp2=expression[cSymTab]) | #(METHOD_CALL (pList = exprList[cSymTab])?
	// ) )? )? )
	// [exp].ID1[exp]
	// [exp].ID1(pList)
	// [exp].ID1
	public Object evaluate(List filteredTargetInstances) throws SPLException
	{
		logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");

		Object result = null;

		// evaluate the array expression to get the integer index
		Integer arrayIndex = (Integer) arrExp1.evaluate();
		int sizeOfFilteredTargetInstances = filteredTargetInstances.size();
		this.instanceNumer=arrayIndex.intValue();
		
		// use the index to extract the corresponding object from the filtered
		// instances
		if ((sizeOfFilteredTargetInstances > arrayIndex.intValue())
				&& (sizeOfFilteredTargetInstances != 0))
		{
			Object selectedInstance = filteredTargetInstances.get(arrayIndex
					.intValue());
			if (!methodOrParam.equals("")) // of the form [exp].ID1 OR
				// [exp].ID1(pList) OR
				// [exp].ID1[exp]
			{
				if (!isIdentifierMethod) // not a method call i.e of type
					// [exp].ID1 OR [exp].ID1[exp]
				{
					// populate the symbol table with the map corresponding to
					// the current instance
					//TBD add Map of ClassToInstance
					symTab.populateInstanceValuesForInstance(this.targetName, this.targetName, selectedInstance, new HashMap());

//					PropertySymbol prop = (PropertySymbol) symTab.getSymbol(this.targetName + "." + methodOrParam); // extract
					String qualifiedPropertyNm=methodOrParam;                                                                // the symbol
					if(!targetName.equalsIgnoreCase(""))    
					{
						qualifiedPropertyNm=targetName+"."+methodOrParam;
					}
					PropertySymbol prop = (PropertySymbol) symTab.getSymbol( qualifiedPropertyNm); // extract
					result = prop.getValue(); // get its value
					if (prop.isArray())
					{
						this._dataType.setIsArray(true);
					}
					else
					{
						this._dataType.setIsArray(false);
					}
				} // its a method invoke
				else
				{
					logger.severe("Cannot invoke methods in the Declaration or Condition sections");

					throw new SPLException(
							Messages
									.getString(
											"SPL_INVALID_METHOD_INVOCATION_SECTION_EXCEPTION_MSG",
											new Object[] { "Declaration or Condition" }));
				}
			}
			else
				// of the type [exp]
			{
				// symTab.populateSymbolTableWithInstanceValues((Map)selectedInstance);
				result = selectedInstance;
				this._dataType.setIsArray(false);
			}

			logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");

			return result;
		}
		else
		{
			logger.severe(Thread.currentThread().getName()+" "+"array index is out of range");
			
			throw new SPLException(Messages
					.getString("SPL_ARRAY_INDEX_OUT_OF_RANGE_EXCEPTION_MSG"));
		}
	}

	public TypeInfo getType() throws SPLException
	{
		logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "getType");

		if (_dataType != null)
			return _dataType;
		else
		{
			validate();

			logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "getType");

			return _dataType;
		}
	}

	public boolean validate(String targetClassName) throws SPLException
	{
		logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "validate");

		// make sure [exp] is numeric
		if (arrExp1 != null && (arrExp1.getType().getType() == TypeConstants.intType)) 
		{
			targetName = targetClassName;

			// String targetClassName =
			// (String)this.symTab.getAnchorClassList().get(0);
			if (!methodOrParam.equals("")) 
			{
				methodOrParamExists = true;
				// System.out.println("!methodOrParam.equals() "+methodOrParam);

				String propName = /* targetClassName + "." + */methodOrParam;
				boolean recurse = false; // do not go up the symbol table
				// hierarchy
				String qualifiedPropertyName = targetClassName + "." + propName;
				// System.out.println("!methodOrParam.equals()
				// "+qualifiedPropertyName);

				// make  sure ID1 exists in the symbol table
				if (symTab.symbolExists(qualifiedPropertyName, recurse)) 
				{
					// System.out.println("symTab.symbolExists(propName,
					// recurse)");

					//TODO xiping 06/21/09
//					Collection c = symTab.getSymbol(qualifiedPropertyName);
					Symbol sym = null;
//					
					
					_dataType.copy(sym.getType());

					if (sym instanceof MethodSymbol) 
					{
						// System.out.println("sym instanceof MethodSymbol");

						// ArrayList formalParams = (ArrayList)
						// ((MethodSymbol)sym).getArgTypeList();
						// TypeResolver.validateActualParameterTypes(formalParams,pList);
						// dataType = sym.getType();
						// isIdentifierMethod = true;
						logger
								.severe(Thread.currentThread().getName()
										+ " "
										+ "Cannot invoke methods in the Declaration or Condition sections");

						throw new SPLException(
								Messages
										.getString(
												"SPL_INVALID_METHOD_INVOCATION_SECTION_EXCEPTION_MSG",
												new Object[] { "Declaration or Condition" }));
					} 
					else if (sym instanceof PropertySymbol)
					// sym instanceof PropertySymbol
					{
						// System.out.println("sym instanceof PropertySymbol");
						// //System.out.println("propType "+propType.getType());

						PropertySymbol pSym = (PropertySymbol) sym;
						if (pSym.isArray()) 
						{
							array2Exists = true;
							// System.out.println("pSym.isArray()");
							_dataType.setIsArray(false);

							if (arrExp2 != null) 
							{
								if (arrExp2.getType().getType() != TypeConstants.intType) 
								{
									logger
											.severe("arr index should be integer and not "
													+ arrExp2.getType());

									throw new IllegalExpressionTypeException(
											Messages
													.getString(
															"SPL_POSTCOLLECT_EXP_EXCEPTION1_MSG",
															new Object[] { arrExp2
																	.getType() }));
								}
							}
						}
						isIdentifierMethod = false;
					} 
					else 
					{
						logger.severe(Thread.currentThread().getName() + " "
								+ "unknown symbol type "
								+ qualifiedPropertyName);
						
						throw new SPLException(Messages.getString(
								"SPL_SYMBOL_DOES_NOT_EXIST_EXCEPTION_MSG",
								new Object[] { qualifiedPropertyName }));
					}
				} 
				else 
				{
					logger.severe(Thread.currentThread().getName() + " "
							+ "symbol does not exist " + qualifiedPropertyName);
					
					throw new SPLException(Messages.getString(
							"SPL_SYMBOL_DOES_NOT_EXIST_EXCEPTION_MSG",
							new Object[] { qualifiedPropertyName }));
				}
			}
			else
			{
				//System.out.println("method or param name is null");
				// expression of type [exp]
				_dataType.setType(TypeConstants.referenceType);
				_dataType.setIsArray(false);
				isIdentifierMethod = false;
				// ; // data type is same as the
				// target class name in the
				// collect
			}
		}
		else
		{
			logger.severe(
					"arr index should be integer and not " + arrExp1.getType());

			throw new IllegalExpressionTypeException(Messages.getString(
					"SPL_POSTCOLLECT_EXP_EXCEPTION1_MSG",
					new Object[] { arrExp1.getType() }));

		}
		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "validate");

		return true;
	}

	public Expression getArrExp1()
	{
		logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "getArrExp1");

		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "getArrExp1");

		return arrExp1;
	}

	public void setArrExp1(Expression arrExp1)
	{
		logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "setArrExp1");
		
		this.arrExp1 = arrExp1;

		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "setArrExp1");
	}

	public Expression getArrExp2()
	{
		logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "getArrExp2");

		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "getArrExp2");

		return arrExp2;
	}

	public void setArrExp2(Expression arrExp2)
	{
		logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "setArrExp2");
		
		this.arrExp2 = arrExp2;

		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "setArrExp2");
	}

	public String getMethodOrParam()
	{
		logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "getMethodOrParam");

		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "getMethodOrParam");

		return methodOrParam;
	}

	public void setMethodOrParam(String methodOrParam)
	{
		logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "setMethodOrParam");
		this.methodOrParam = methodOrParam;

		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "setMethodOrParam");
	}

	public ArrayList getPList()
	{
		logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "getPList");

		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "getPList");

		return pList;
	}

	public void setPList(ArrayList list)
	{
		logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "setPList");

		pList = list;

		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "setPList");
	}

	public SPLSymbolTable getSymTab()
	{
		logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "getSymTab");

		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "getSymTab");

		return symTab;
	}

	public void setSymTab(SPLSymbolTable symTab)
	{
		logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "setSymTab");

		this.symTab = symTab;

		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "setSymTab");
	}

	public Object evaluate() throws SPLException
	{
		logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");

		logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");
		
		throw new SPLException(Messages.getString(
				"SPL_EVALUATE_METHOD_NOT_SUPPORTED_EXCEPTION_MSG",
				new Object[] { sourceClass }));

	}

	public String toString()
	{
		logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "toString");
		
		String str="";
		if(this.arrExp1!=null)
		{
			str+=this.arrExp1.toString();
		}
		if(this.methodOrParam!=null)
		{
			str+=this.methodOrParam.toString();
		}
		
		if(this.arrExp2!=null)
		{
			str+=" "+this.arrExp1.toString();
		}

		logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "toString");

		return str;
	}

	public String getReferenceTypeName() throws SPLException 
	{
		// TODO Auto-generated method stub
		return null;
	}

	public int getInstanceNumer() 
	{
		return instanceNumer;
	}

	public void setInstanceNumer(int instanceNumer) 
	{
		this.instanceNumer = instanceNumer;
	}

	public boolean validate() throws SPLException 
	{
		// TODO Auto-generated method stub
		throw new SPLException(Messages.getString(
				"SPL_VALIDATE_METHOD_NOT_SUPPORTED_EXCEPTION_MSG",
				new Object[] { sourceClass }));
	}

	public boolean isMethodOrParamExists() 
	{
		return methodOrParamExists;
	}

	
	public boolean Array2Exists() 
	{
		return array2Exists;
	}
}
