/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */
//

/**
 * @author Neeraj Joshi <jneeraj@us.ibm.com>
 *
 */

package org.apache.imperius.spl.parser.expression.primary;

import java.util.Iterator;
import java.util.List;
import java.util.logging.Logger;

import org.apache.imperius.spl.core.Expression;
import org.apache.imperius.spl.core.TypeConstants;
import org.apache.imperius.spl.core.TypeInfo;
import org.apache.imperius.spl.parser.compiler.symboltable.SPLSymbolTable;
import org.apache.imperius.spl.parser.exceptions.SPLException;
import org.apache.imperius.spl.parser.expressions.ReferenceExpression;
import org.apache.imperius.spl.parser.util.TypeInfoImpl;
import org.apache.imperius.util.Messages;
import org.apache.imperius.util.SPLLogger;

public class SelfExpression implements Expression, ReferenceExpression
{
    
    private SPLSymbolTable sTab;

    private TypeInfo _dataType = null;

	private static Logger logger = SPLLogger.getSPLLogger().getLogger();

	private static final String sourceClass = "SelfExpression";
    
    
    public SelfExpression(SPLSymbolTable symTab)throws SPLException
    {
        logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "SelfExpression");

		sTab = symTab;

		if (!validate()) 
		{
			logger.severe(Thread.currentThread().getName() + " "
					+ "validation error: " + sourceClass
					+ " has wrong data type passed in.");

			throw new SPLException(Messages.getString(
					"SPL_VALIDATION_ERROR_MSG", new Object[] { sourceClass }));
		}
		
		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ sourceClass);
    }
    
    public String getReferenceTypeName()
    {
    	 logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");
         //System.out.println("evaluating Self expression");
    	 String referenceTypeName = null;
         
          SPLSymbolTable symbtab=sTab;
          while(sTab.getParentSymbolTable() != null && sTab.getParentSymbolTable().getSymbolTableType() == SPLSymbolTable.COLLECT){
          	symbtab=sTab.getParentSymbolTable();
          }
          
          
          if (symbtab.getParentSymbolTable() != null && symbtab.getParentSymbolTable().getSymbolTableType() != SPLSymbolTable.COLLECT )
          {	
              SPLSymbolTable parentSymTab = symbtab.getParentSymbolTable();
              List anchorClassList = parentSymTab.getAnchorClassList();
              Iterator anchorClassListIt = anchorClassList.iterator();
             
             	referenceTypeName = (String)anchorClassListIt.next();
             
             
          }
          else{
//          	throw new SPLException("non collect Parent SymbolTable not found in Self Expression");
        	  logger.severe("non collect Parent SymbolTable not found in Self Expression");
        	  
          }
          logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");
          return referenceTypeName;
    }
    
    public Object evaluate() throws SPLException
    {
        logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "evaluate");

		Object ins = null;
		SPLSymbolTable symbtab = sTab;
		while (sTab.getParentSymbolTable() != null
				&& sTab.getParentSymbolTable().getSymbolTableType() == SPLSymbolTable.COLLECT) 
		{
			symbtab = sTab.getParentSymbolTable();
		}

		if (symbtab.getParentSymbolTable() != null
				&& symbtab.getParentSymbolTable().getSymbolTableType() != SPLSymbolTable.COLLECT) 
		{
			SPLSymbolTable parentSymTab = symbtab.getParentSymbolTable();
			String referenceTypeName = null;
			List anchorClassList = parentSymTab.getAnchorClassList();
			Iterator anchorClassListIt = anchorClassList.iterator();

			referenceTypeName = (String) anchorClassListIt.next();

			ins = parentSymTab.getInstance(referenceTypeName);
		} 
		else 
		{
			throw new SPLException(Messages.getString("SPL_SELF_EXPRESSION_EXCEPTION_MSG"));
		}
		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "evaluate");

		return ins;
    }
    
    public TypeInfo getType()
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "getType");
        
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "getType");
      
        return _dataType;
    }
    
    public boolean validate() throws SPLException
    {
        logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "validate");
		
        _dataType = new TypeInfoImpl(TypeConstants.referenceType, null, false);

		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "validate");
		
		return true;
    }
    
    public boolean isArray()
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "isArray");
        
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "isArray");
        
        return false;
    }
    
    public String toString()
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "toString");
        
        String str=" self ";
                         
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "toString");
       
        return str;
    }
    
}
