/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */
//

/**
 * @author Prashant Baliga <prabalig@in.ibm.com>
 *
 */

package org.apache.imperius.spl.parser.expressions;

import java.util.Iterator;
import java.util.List;
import java.util.Vector;
import java.util.logging.Logger;

import org.apache.imperius.spl.core.Expression;
import org.apache.imperius.spl.core.TypeInfo;
import org.apache.imperius.spl.parser.exceptions.SPLException;
import org.apache.imperius.spl.parser.util.TypeInfoImpl;
import org.apache.imperius.util.Messages;
import org.apache.imperius.util.SPLLogger;

public abstract class DoubleArgumentExpression implements Expression
{
    
    protected Expression _lhsExp;

	protected Expression _rhsExp;

	private static Logger logger = SPLLogger.getSPLLogger().getLogger();

	private final String sourceClass = "DoubleArgumentExpression";

	// protected boolean _isArray = false;

	protected TypeInfoImpl _dataType = new TypeInfoImpl();
    
    public DoubleArgumentExpression(List exprList)
    {
        logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "DoubleArgumentExpression");

		if (exprList != null) 
		{
			if (exprList.size() == 2) 
			{
				Expression l = (Expression) exprList.get(0);
				Expression r = (Expression) exprList.get(1);

				if (l == null) 
				{
					logger
							.severe("left hand side expression passed in is null.");
					throw new IllegalArgumentException(Messages.getString(
							"SPL_PASSED_EXPRESSION_ERROR_MSG",
							new Object[] { "left" }));
				} 
				else 
				{
					// //System.out.println("DoubleArgumentExpression::LHS
					// "+l.getClass()+l.toString());
				}

				if (r == null) 
				{
					logger
							.severe("right hand side expression passed in is null.");
					throw new IllegalArgumentException(Messages.getString(
							"SPL_PASSED_EXPRESSION_ERROR_MSG",
							new Object[] { "right" }));
				} 
				else 
				{
					// //System.out.println("DoubleArgumentExpression::RHS
					// "+r.getClass()+r.toString());
				}
				this._lhsExp = l;
				this._rhsExp = r;
			} 
			else 
			{
				logger.severe("No of arguments should be 2, but is "
						+ exprList.size());

				throw new IllegalArgumentException(Messages.getString(
						"SPL_NO_OF_ARGUMENTS_PASSED_ERROR_MSG", new Object[] {
								"2", exprList.size() + "" }));
			}
		} 
		else 
		{
			logger
					.severe("No of arguments passed required number of arguments is 2");

			throw new IllegalArgumentException(Messages.getString(
					"SPL_NO_OF_ARGUMENTS_PASSED_ERROR_MSG", new Object[] {
							"2", "blank" }));
		}
		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "DoubleArgumentExpression");
    }
    
    public abstract Object evaluate() throws SPLException;
    
    public TypeInfo getType()
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "getType");

        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "getType");
       
        return _dataType;
    }
    
    public boolean isArray()
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "isArray");

        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "isArray");
       
        return _dataType.getIsArray();
    }
    
    public String toString()
    {
        logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "toString");

        String className = this.getClass().getName();

		String str = className.substring(className.lastIndexOf(".") + 1,
				className.length())
				+ "( "
				+ this._lhsExp.toString()
				+ " , "
				+ this._rhsExp.toString() + " )";

		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "toString");

		return str;
    }
    
    public abstract boolean validate() throws SPLException;
    
    public String getReferenceTypeName() 
    {
    	return null;
    }

    //Xiping 05/25/08
    public Iterator getSubexpressions() {
    	Vector subexpressions = new Vector();
    	subexpressions.add(_lhsExp);
    	subexpressions.add(_rhsExp);
    	
    	return subexpressions.iterator();
    }
    //

    //Xiping 10/13/08
    public Expression getLHS() {
    	return _lhsExp;
    }

    public Expression getRHS() {
    	return _rhsExp;
    }
    
    public void replaceLHS(Expression lhs) {
    	_lhsExp = lhs;
    }
    
    public void replaceRHS(Expression rhs) {
    	_rhsExp = rhs;
    }
    //

}
