/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */
//

/**
 * @author Prashant Baliga <prabalig@in.ibm.com>
 *
 */

package org.apache.imperius.spl.parser.expressions;

import java.util.ArrayList;
import java.util.logging.Logger;

import org.apache.imperius.spl.core.Expression;
import org.apache.imperius.spl.core.TypeInfo;
import org.apache.imperius.spl.parser.compiler.symboltable.MethodSymbol;
import org.apache.imperius.spl.parser.compiler.symboltable.SPLSymbolTable;
import org.apache.imperius.spl.parser.exceptions.SPLException;
import org.apache.imperius.spl.parser.util.TypeInfoImpl;
import org.apache.imperius.util.SPLLogger;


public class MethodExpression implements Expression
{
    
    private TypeInfo returnType = new TypeInfoImpl();

	private ArrayList passedParams;

	// private SPLSymbolTable symTab;

	private static Logger logger = SPLLogger.getSPLLogger().getLogger();

	private final String sourceClass = "MethodExpression";
    
    public MethodExpression(MethodSymbol ms, ArrayList pList,
            SPLSymbolTable symTab)
    {
        logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "MethodExpression");

		returnType = ms.getType();
		passedParams = pList;
		// this.symTab = symTab;

		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "MethodExpression");
    }
    
    public Object evaluate()
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");

        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");
       
        return null;
    }
    
    public TypeInfo getType()
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "getType");

        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "getType");
       
        return returnType;
    }
    
    public boolean validate() throws SPLException
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "validate");

        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "validate");
       
        return true;
    }
    
    public boolean isArray()
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "isArray");

        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "isArray");
       
        return false;
    }
    
    public String toString()
    {
        logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "toString");

		String className = this.getClass().getName();

		String str = className.substring(className.lastIndexOf(".") + 1,
				className.length())
				+ "[ " + this.passedParams.toString() + " ]";

		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "toString");

		return str;
    }

	public String getReferenceTypeName() throws SPLException 
	{
		return null;
	}
    
}
