/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */
//

/**
 * @author Prashant Baliga <prabalig@in.ibm.com>
 *
 */

package org.apache.imperius.spl.parser.expressions;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.logging.Logger;

import org.apache.imperius.spl.core.Expression;
import org.apache.imperius.spl.core.TypeInfo;
import org.apache.imperius.spl.parser.exceptions.SPLException;
import org.apache.imperius.spl.parser.util.TypeInfoImpl;
import org.apache.imperius.util.Messages;
import org.apache.imperius.util.SPLLogger;


public abstract class MultipleArgumentExpression implements Expression
{
    
    protected List _expressions;

	// protected Expression rhsExp;

	private static Logger logger = SPLLogger.getSPLLogger().getLogger();

	private final String sourceClass = "MultipleArgumentExpression";

	protected TypeInfo _dataType = new TypeInfoImpl();
    
    public MultipleArgumentExpression(List exprList)
    {
        logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "MultipleArgumentExpression");

		if (exprList != null) 
		{
			if (exprList.size() != 0) 
			{
				this._expressions = new ArrayList();
				for (int i = 0; i < exprList.size(); i++) 
				{
					Expression l = (Expression) exprList.get(i);
					if (l == null) 
					{
						logger.severe("expression passed in is null.");

						throw new IllegalArgumentException(Messages.getString(
								"SPL_BLANK_EXPRESSION_PASSED_ERROR_MSG",
								new Object[] { "" }));
					} 
					else 
					{
						this._expressions.add(l);
					}
				}
			} 
			else 
			{
				logger.severe(Thread.currentThread().getName() + " "
						+ "No of arguments should be > 0, but is "
						+ exprList.size());
				
				throw new IllegalArgumentException(Messages.getString(
						"SPL_NO_ARGUMENTS_PASSED_ERROR_MSG", new Object[] {
								">0", "" + exprList.size() }));
			}
		} 
		else 
		{
			logger
					.severe(Thread.currentThread().getName()
							+ " "
							+ "No of arguments passed required number of arguments is 2");

			throw new IllegalArgumentException(Messages.getString(
					"SPL_NO_ARGUMENTS_PASSED_ERROR_MSG", new Object[] {
							">0", "blank"}));
		}
		
		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "MultipleArgumentExpression");
        
    }
    
    public abstract Object evaluate() throws SPLException;
    
    public TypeInfo getType()
    {
        logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "getType");

		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "getType");

		return _dataType;
    }
    
    public boolean isArray()
    {
        logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "isArray");

		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "isArray");

		return this._dataType.getIsArray();
    }
    
    public String toString()
    {
        logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "toString");

		String className = this.getClass().getName();

		String str = className.substring(className.lastIndexOf(".") + 1,
				className.length())
				+ "( ";

		if (this._expressions.size() != 0) 
		{
			for (int i = 0; i < this._expressions.size(); i++) 
			{
				Expression l = (Expression) this._expressions.get(i);
				if (l == null) 
				{
					logger.severe("expression passed in is null.");

					throw new IllegalArgumentException(Messages.getString(
							"SPL_BLANK_EXPRESSION_PASSED_ERROR_MSG",
							new Object[] { "" }));
				} 
				else 
				{
					str += l.toString();
				}
				if (i != this._expressions.size() - 1) 
				{
					str += " , ";
				}
			}
			str += " )";
		}

		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "toString");

		return str;
    }
    
    public abstract boolean validate() throws SPLException;
    
    public String getReferenceTypeName() throws SPLException
    {
    	return null;
    }
    
    //xiping 06/13/08
    public Iterator getSubexpressions() {
    	return _expressions.iterator();
    }
    
    public List getExpressions() {
    	return _expressions;
    }
    
    public void replaceExpressions(List list) {
    	_expressions = list;
    }
}
