/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */
//

/**
 * @author Xiping Wang
 *
 */
package org.apache.imperius.spl.parser.expressions;

import java.util.Iterator;
import java.util.List;
import java.util.Vector;
import java.util.logging.Logger;

import org.apache.imperius.spl.core.Expression;
import org.apache.imperius.spl.core.TypeInfo;
import org.apache.imperius.spl.parser.exceptions.SPLException;
import org.apache.imperius.spl.parser.util.TypeInfoImpl;
import org.apache.imperius.util.Messages;
import org.apache.imperius.util.SPLLogger;

public abstract class QuadrupleArgumentExpression implements Expression
{
    protected Expression _exp1;

	protected Expression _exp2;

	protected Expression _exp3;

	protected Expression _exp4;

	protected TypeInfo _dataType = new TypeInfoImpl();

	private static Logger logger = SPLLogger.getSPLLogger().getLogger();

	private static final String sourceClass = "class";
    
    public boolean isArray()
    {
        logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "isArray");

		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "isArray");

		return _dataType.getIsArray();
    }
    
    public QuadrupleArgumentExpression(List exprList)
    {
        logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "QuadrupleArgumentExpression");

		if (exprList != null) 
		{
			if (exprList.size() == 4) 
			{
				Expression e1 = (Expression) exprList.get(0);
				Expression e2 = (Expression) exprList.get(1);
				Expression e3 = (Expression) exprList.get(2);
				Expression e4 = (Expression) exprList.get(3);
				
				if (e1 == null) 
				{
					logger.severe("first expression passed in is null.");

					throw new IllegalArgumentException(Messages.getString(
							"SPL_BLANK_EXPRESSION_PASSED_ERROR_MSG",
							new Object[] { "first" }));
				}

				if (e2 == null) 
				{
					logger.severe("second expression passed in is null.");

					throw new IllegalArgumentException(Messages.getString(
							"SPL_BLANK_EXPRESSION_PASSED_ERROR_MSG",
							new Object[] { "second" }));
				}

				if (e3 == null) 
				{
					logger.severe("third expression passed in is null.");

					throw new IllegalArgumentException(Messages.getString(
							"SPL_BLANK_EXPRESSION_PASSED_ERROR_MSG",
							new Object[] { "third" }));
				}

				if (e4 == null) 
				{
					logger.severe("fourth expression passed in is null.");

					throw new IllegalArgumentException(Messages.getString(
							"SPL_BLANK_EXPRESSION_PASSED_ERROR_MSG",
							new Object[] { "fourth" }));
				}

				this._exp1 = e1;
				this._exp2 = e2;
				this._exp3 = e3;
				this._exp4 = e4;

			} 
			else 
			{
				logger.severe(Thread.currentThread().getName() + " "
						+ "number of parameters not 4");

				throw new IllegalArgumentException(Messages.getString(
						"SPL_NO_OF_ARGUMENTS_PASSED_ERROR_MSG", new Object[] {
								"4", "" + exprList.size() }));
			}
		}
		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "QuadrupleArgumentExpression");
        
    }
    
    public abstract Object evaluate() throws SPLException;
    
    public TypeInfo getType()
    {
        logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "getType");

		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "getType");

		return _dataType;
    }
    
    public abstract boolean validate() throws SPLException;
    
    public String getReferenceTypeName() throws SPLException
    {
    	return null;
    	//throw new SPLException("ReferenceTypeName not supported");
    }
    
    //Xiping 05/29/2008
    public Iterator getSubexpressions() {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "getSubexpressions");

        Vector subexpressions = new Vector();
    	subexpressions.add(_exp1);
    	subexpressions.add(_exp2);
    	subexpressions.add(_exp3);
    	subexpressions.add(_exp4);
    	
    	logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "getSubexpressions");
    	
    	return subexpressions.iterator();
    }
}
