/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */
//

/**
 * @author Prashant Baliga <prabalig@in.ibm.com>
 *
 */

package org.apache.imperius.spl.parser.expressions;

import java.util.ArrayList;
import java.util.List;
import java.util.logging.Logger;

import org.apache.imperius.spl.core.Expression;
import org.apache.imperius.spl.core.TypeConstants;
import org.apache.imperius.spl.core.TypeInfo;
import org.apache.imperius.spl.parser.compiler.symboltable.SPLSymbolTable;
import org.apache.imperius.spl.parser.exceptions.SPLException;
import org.apache.imperius.spl.parser.expressions.impl.StringConstant;
import org.apache.imperius.spl.parser.util.TypeInfoImpl;
import org.apache.imperius.spl.parser.util.TypeResolver;
import org.apache.imperius.util.Messages;
import org.apache.imperius.util.SPLLogger;


public class TimeZoneExpression extends SingleArgumentExpression implements
        StringExpression
{
    
    public static final String className = TimeZoneExpression.class.getName();

	private static Logger logger = SPLLogger.getSPLLogger().getLogger();

	private static final String sourceClass = "TimeZoneExpression";
    
    
    public boolean isArray()
    {
        logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "isArray");

		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "isArray");

		return _dataType.getIsArray();
    }
    
    public TimeZoneExpression(List exprList, boolean validateExpression)
            throws SPLException
    {
        super(exprList);

		logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "TimeZoneExpression");

		if (validateExpression) 
		{
			if (!validate()) 
			{
				logger.severe(Thread.currentThread().getName() + " "
						+ "validation error: " + className
						+ " has wrong data type passed in.");

				throw new SPLException(Messages.getString(
						"SPL_VALIDATION_ERROR_MSG", new Object[] { className }));
			}
		}
		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "TimeZoneExpression");
        
    }
    
    public Object evaluate() throws SPLException
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");
       
        try
        {
            logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");
           
            return this._exp.evaluate();
        }
        catch (Exception e)
        {
            logger.severe(Thread.currentThread().getName()+" "+"evaluation errro: " + e.toString());

            throw new SPLException(Messages.getString(
					"SPL_EVALUATION_ERROR_MSG", new Object[] { className + " : " + e.getLocalizedMessage() }));
        }

    }
    
    public static TimeZoneExpression getInstance(String tzid,
            SPLSymbolTable sTab) throws SPLException
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "getType");

        Expression e = new StringConstant(tzid);
        List aList = new ArrayList(1);	// DAW
        aList.add(e);
        boolean validateExp = true;
        if (sTab.getSymbolTableType() == SPLSymbolTable.COLLECT)
        {
            validateExp = false;
        }

        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "getType");
       
        return new TimeZoneExpression(aList, validateExp);
    }
    
    public boolean validate() throws SPLException
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "validate");

        TypeInfo eType = _exp.getType();
        _dataType = new TypeInfoImpl(TypeConstants.stringType,null,false);
        if (TypeResolver.isString(eType))
        {
            logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "validate");
           
            return true;
        }
        
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "validate");
       
        return false;
    }
    
    public String toString()
    {
        logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "toString");

		String className = this.getClass().getName();

		String str = className.substring(className.lastIndexOf(".") + 1,
				className.length())
				+ "( " + this._exp.toString() + " )";

		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "toString");

		return str;
    }

	public String getReferenceTypeName() throws SPLException 
	{
		return null;
	}
}
