/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */
//

/**
 * @author Prashant Baliga <prabalig@in.ibm.com>
 *
 */

package org.apache.imperius.spl.parser.expressions;

import java.util.List;
import java.util.logging.Logger;

import org.apache.imperius.spl.core.Expression;
import org.apache.imperius.spl.core.TypeInfo;
import org.apache.imperius.spl.parser.exceptions.SPLException;
import org.apache.imperius.spl.parser.util.TypeInfoImpl;
import org.apache.imperius.util.SPLLogger;


public abstract class ZeroArgumentExpression implements Expression
{
    private static Logger logger = SPLLogger.getSPLLogger().getLogger();

	private final String sourceClass = "ZeroArgumentExpression";

	protected TypeInfo _dataType = new TypeInfoImpl();
    
    public boolean isArray()
    {
        return this._dataType.getIsArray();
    }
    
    public ZeroArgumentExpression(List exprList)
    {
        logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "ZeroArgumentExpression");

		/*
		 * if (exprList != null) { throw new IllegalArgumentException(
		 * "ZeroArgumentExpression expression should not be passed arguments");
		 *  }
		 */

		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "ZeroArgumentExpression");
       
    }
    
    public abstract Object evaluate() throws SPLException;
    
    public TypeInfo getType()
    {
        logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "getType");

		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "getType");

		return _dataType;
    }
    
    public String toString()
    {
        logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "toString");

		String className = this.getClass().getName();

		String str = className.substring(className.lastIndexOf(".") + 1,
				className.length())
				+ "()";

		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "toString");

		return str;
    }
    
    public abstract boolean validate() throws SPLException;
    
    public String getReferenceTypeName() throws SPLException
    {
    	return null;
    }
    
}
