/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */
//
/**
 * 
 * @author Xiping Change Log: 3/9/07: Neeraj Joshi: Changed signature to take a
 *         list of expressions and boolean This will help in reflection
 * 
 */

package org.apache.imperius.spl.parser.expressions.impl;

import java.util.List;
import java.util.logging.Logger;

import org.apache.imperius.spl.core.TypeConstants;
import org.apache.imperius.spl.core.TypeInfo;
import org.apache.imperius.spl.parser.exceptions.SPLException;
import org.apache.imperius.spl.parser.expressions.NumericExpression;
import org.apache.imperius.spl.parser.expressions.SingleArgumentExpression;
import org.apache.imperius.spl.parser.util.TypeResolver;
import org.apache.imperius.util.Messages;
import org.apache.imperius.util.SPLLogger;

public class Abs extends SingleArgumentExpression implements NumericExpression 
{

	public static final String className = Abs.class.getName();

	private static Logger logger = SPLLogger.getSPLLogger().getLogger();

	private static final String sourceClass = "Abs";

	public Abs(List exprList, boolean validateExpression) throws SPLException 
	{
		super(exprList);
		logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "Abs");

		if (validateExpression) 
		{
			if (!validate()) 
			{
				logger.severe(Thread.currentThread().getName() + " "
						+ "validation error: " + className
						+ " has wrong data type passed in.");
				throw new SPLException(Messages.getString(
						"SPL_VALIDATION_ERROR_MSG", new Object[] { className }));
			}
		}
		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "Abs");

	}

	public Object evaluate() throws SPLException 
	{
		logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "evaluate");
		Object returnVal = abs((Number) _exp.evaluate());
		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "evaluate" + returnVal);

		return returnVal;
	}

	public static Number abs(Number o1) throws SPLException 
	{
		logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "Number");

		try 
		{
			Number[] o = new Number[1];
			o[0] = o1;

			int resolvedType = TypeResolver.resolveType(o);

			if (resolvedType == TypeConstants.doubleType) 
			{
				Double dVal = new Double(Math.abs(o1.doubleValue()));
				logger.exiting(sourceClass, Thread.currentThread().getName()
						+ " " + "Number" + dVal);

				return dVal;
			} 
			else if (resolvedType == TypeConstants.floatType) 
			{
				Float fVal = new Float(Math.abs(o1.floatValue()));
				logger.exiting(sourceClass, Thread.currentThread().getName()
						+ " " + "Number" + fVal);

				return fVal;
			} 
			else if (resolvedType == TypeConstants.longType) 
			{
				Long lVal = new Long(Math.abs(o1.longValue()));
				logger.exiting(sourceClass, Thread.currentThread().getName()
						+ " " + "Number" + lVal);

				return lVal;
			} 
			else 
			{
				Integer iVal = new Integer(Math.abs(o1.intValue()));
				logger.exiting(sourceClass, Thread.currentThread().getName()
						+ " " + "Number" + iVal);

				return iVal;
			}
		} 
		catch (Exception e) 
		{
			logger.severe(Thread.currentThread().getName() + " "
					+ "evaluation error: " + e.toString());

			throw new SPLException(Messages.getString(
					"SPL_EVALUATION_ERROR_MSG", new Object[] { e.toString() }));
		}
	}

	public boolean validate() throws SPLException 
	{
		logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "validate");

		TypeInfo eType = _exp.getType();

		// dataType = TypeConstants.numericType;
		if (TypeResolver.isNumeric(eType) && !eType.getIsArray()) 
		{
			_dataType.copy(eType);
			logger.exiting(sourceClass, Thread.currentThread().getName() + " "
					+ "validate");

			return true;
		}

		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "validate");

		return false;
	}

	public String getReferenceTypeName() throws SPLException 
	{
		// TODO Auto-generated method stub
		return null;
	}

	public String toString() 
	{
		logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "toString");

		String str = "Abs(" + this._exp.toString() + ")";

		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "toString");

		return str;
	}

}
