/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */
//

/**
 * @author Prashant Baliga <prabalig@in.ibm.com>
 *
 */

package org.apache.imperius.spl.parser.expressions.impl;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.logging.Logger;

import org.apache.imperius.spl.core.Expression;
import org.apache.imperius.spl.core.TypeConstants;
import org.apache.imperius.spl.core.TypeInfo;
import org.apache.imperius.spl.parser.exceptions.IllegalParameterTypeException;
import org.apache.imperius.spl.parser.exceptions.SPLException;
import org.apache.imperius.spl.parser.expressions.TripleArgumentExpression;
import org.apache.imperius.spl.parser.util.ExpressionUtility;
import org.apache.imperius.spl.parser.util.TypeResolver;
import org.apache.imperius.util.Messages;
import org.apache.imperius.util.SPLLogger;


public class ApplyToCollection extends TripleArgumentExpression implements
        Expression
{
    
    public String className = InCollection.class.toString();
    
    public List _expressionList = null;
    
    public String _operationString = null;
    
    private static Logger logger = SPLLogger.getSPLLogger().getLogger();
    private static final String sourceClass="ApplyToCollection";
    
    
    
    public static final String PLUS = "PLUS";
    
    public static final String MINUS = "MINUS";
    
    public static final String STAR = "MULTIPLY";
    
    public static final String DIV = "DIVIDE";
    
    public ApplyToCollection(List exprList, boolean evaluateExpression)
            throws SPLException
    {
        super(exprList);
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "ApplyToCollection");

        _dataType.setType(TypeConstants.booleanType);
        this._dataType.setIsArray(false);
        //System.out.println("ApplyToCollection");
        if (evaluateExpression)
        {
            //System.out.println("evaluateExpression " + evaluateExpression);
            if (!validate())
            {

                logger.severe
               ("validation error: wrong data type passed in"+ this._dataType);
                throw new SPLException(Messages.getString(
						"SPL_VALIDATION_ERROR_MSG", new Object[] { className }));
            }
        }
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "ApplyToCollection");
        
        // this.dataType=TypeConstants.numericType;
    }
    
    public Object evaluate() throws SPLException
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");

        //System.out.println("ApplyToCollection:evaluate");
        Expression lhsExpression = (Expression) this._lhsExp;
        Expression rhsExpression = (Expression) this._rhsExp;
        
        if (rhsExpression.isArray())
        {
            //System.out.println("rhs is of type Basic Collection");
            Object rhsResult = rhsExpression.evaluate();
            //System.out.println("rhsResult , class " + rhsResult + " "+ rhsResult.getClass());
            if (!(rhsResult instanceof java.util.List))
            {
                //System.out.println("rhsResult is not of type List");
                throw new SPLException(Messages.getString(
    					"SPL_PASSED_EXPRESSION_TYPE_ERROR_MSG",
    					new Object[] { "RHS","is not", "List" }));

            }
            List resultList = new ArrayList();
            ArrayList rhsResultArray = (ArrayList) rhsResult;
            //System.out.println("resultArray size " + rhsResultArray.size()+ " to string " + rhsResultArray.toString());
            //System.out.println("LHS expression is of type "+ lhsExpression.getType());
            if ((rhsResultArray != null) && (!rhsResultArray.isEmpty()))
            {
                Object lhsResult = (Object) lhsExpression.evaluate();
                //System.out.println("lhsResult " + lhsResult.toString());
                Iterator rhsResultIt = rhsResultArray.iterator();
                while (rhsResultIt.hasNext())
                {
                    Object value = (Object) rhsResultIt.next();
                    //System.out.println("rhsResult element " + value.toString());
                    
                    if (_operationString.equalsIgnoreCase(DIV))
                    {
                        Object Value1 = lhsResult;
                        Object Value2 = value;
                        Object res = ExpressionUtility.division(
                                (Number) Value2, (Number) Value1);
                        //System.out.println("result" + res.toString());
                        resultList.add(res);
                    }
                    if (_operationString.equalsIgnoreCase(PLUS))
                    {
                        Object Value1 = lhsResult;
                        Object Value2 = value;
                        Object res = ExpressionUtility.plus((Number) Value1,
                                (Number) Value2);
                        resultList.add(res);
                    }
                    if (_operationString.equalsIgnoreCase(MINUS))
                    {
                        Object Value1 = lhsResult;
                        Object Value2 = value;
                        Object res = ExpressionUtility.minus((Number) Value2,
                                (Number) Value1);
                        resultList.add(res);
                    }
                    if (_operationString.equalsIgnoreCase(STAR))
                    {
                        Object Value1 = lhsResult;
                        Object Value2 = value;
                        Object res = ExpressionUtility.multiplication(
                                (Number) Value2, (Number) Value1);
                        resultList.add(res);
                    }
                    
                }

                logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");
             
                return resultList;
            }
            else
            {
                logger.severe(
                "the RHS of ApplyToCollection expression is null BasicCollection");
                
                throw new IllegalParameterTypeException(Messages.getString(
    					"SPL_PASSED_EXPRESSION_TYPE_ERROR_MSG", new Object[] {
    							"Right", "is null", "" }));
            }
        }
        else
        {
            logger.severe(
            "the RHS of ApplyToCollection expression should be of BasicCollection type");
            
            
            throw new IllegalParameterTypeException(Messages.getString(
					"SPL_PASSED_EXPRESSION_TYPE_ERROR_MSG", new Object[] {
							"Right", "should be", "BasicCollection" }));
        }
        
    }
    
    public boolean validate() throws SPLException
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "validate");

        //System.out.println("ApplyToCollection : validate ");
        Expression expression1 = (Expression) this._lhsExp;
        Expression expression2 = (Expression) this._midExp;
        Expression expression3 = (Expression) this._rhsExp;
        TypeInfo leftType = expression1.getType();
        TypeInfo middleType = expression2.getType();
        TypeInfo rightType = expression3.getType();
        
        if (!expression3.isArray())
        {
            logger.severe(Thread.currentThread().getName()+" "+"Last Expression should be a collection");
            
            throw new SPLException("Last Expression should be a collection");
        }
        
        if (expression1.isArray())
        {
            logger.severe(Thread.currentThread().getName()+" "+"First Expression cannot be a collection");
            
            throw new SPLException(Messages.getString(
					"SPL_PASSED_EXPRESSION_TYPE_ERROR_MSG", new Object[] {
							"right", "should not", "collection" }));
        }
        
        if (!TypeResolver.isString(middleType))
        {
            logger.severe(
            "Middle Expression should be a string describing the Operation");
            
            throw new SPLException(Messages.getString(
					"SPL_PASSED_EXPRESSION_TYPE_ERROR_MSG", new Object[] {
							"middle", "should", "string" }));
        }
        this._operationString = (String) expression2.evaluate();
        if (this._operationString == null)
        {
            logger.severe(Thread.currentThread().getName()+" "+"Operation string is null");
            
            throw new SPLException(Messages.getString(
					"SPL_PASSED_EXPRESSION_ERROR_MSG", new Object[] {
							"middle" }));

        }
//        if (this._operationString == null)
//        {
//            logger.severe(Thread.currentThread().getName()+" "+"Operation type is "
//                    + this._operationString);
//            
//            throw new SPLException("Operation type is "
//                    + this._operationString);
//        }
        
        if (_operationString.equalsIgnoreCase(DIV)
                || _operationString.equalsIgnoreCase(PLUS)
                || _operationString.equalsIgnoreCase(MINUS)
                || _operationString.equalsIgnoreCase(STAR))
        {
            
            if (TypeResolver.isNumeric(leftType))
            {
                if (TypeResolver.isNumeric(rightType))
                {
                    this._dataType = TypeResolver
                            .binaryNumericPromotionResolver(leftType, rightType);
                   _dataType.setIsArray(true);
                    logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "validate");
                    
                    return true;
                }
                else
                {
                    logger.severe(
                    "RHS is not numeric, and hence not applicable to ApplyToCollection");
                    
                    
                    throw new SPLException(Messages.getString(
							"SPL_TYPE_NOT_COMPATIBLE_EXCEPTION_MSG",
							new Object[] { leftType, rightType }));
                }
            }
            else
            {
                logger.severe(
                "LHS is not numeric, and hence not applicable to ApplyToCollection");
                
                
                throw new SPLException(Messages.getString(
						"SPL_TYPE_NOT_COMPATIBLE_EXCEPTION_MSG",
						new Object[] { leftType, rightType }));
            }
        }
        else
        {
            logger.severe(
            "operationString is not supported by ApplyToCollection");
            throw new SPLException(Messages.getString("SPL_OPERATION_NOT_SUPPORTED_EXCEPTION_MSG"));        
        }
    }

	public String getReferenceTypeName() throws SPLException {
		// TODO Auto-generated method stub
		return null;
	}
	
    public String toString()
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "toString");

        String str = "ApplyToCollection("+ this._lhsExp.toString() + "," + this._midExp.toString() + "," + this._rhsExp.toString() + ")";
        	
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "toString");
       
        return str;
    }
    
}
