/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */
//

/**
 * @author Prashant Baliga <prabalig@in.ibm.com>
 *
 */
package org.apache.imperius.spl.parser.expressions.impl;

import java.util.logging.Logger;

import org.apache.imperius.spl.core.Expression;
import org.apache.imperius.spl.core.TypeConstants;
import org.apache.imperius.spl.core.TypeInfo;
import org.apache.imperius.spl.parser.exceptions.SPLException;
import org.apache.imperius.spl.parser.util.TypeInfoImpl;
import org.apache.imperius.util.SPLLogger;



public class CharConstant implements Expression
{
    
    private char _charValue;
    
    private TypeInfo _dataType =new TypeInfoImpl();
    
    private static Logger logger = SPLLogger.getSPLLogger().getLogger();
    private static final String sourceClass="CharConstant";
    
    
    
    public boolean isArray()
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "isArray");

        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "isArray");
    
        
        return _dataType.getIsArray();
    }
    
    public CharConstant(char c)
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "CharConstant");

        this._charValue = c;
        _dataType.setType(TypeConstants.charType);
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "CharConstant");
        
    }
    
    // @Override
    /*
     * public Object evaluate() { // 
     * Character(charValue); }
     * 
     * @Override public int getType() { 
     * return TypeConstants.charType; } public boolean validate() throws
     * CIMSPLException {
     */

    public Object evaluate() throws SPLException
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");

        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");
    
       
        return new Character(_charValue);
    }
    
    public TypeInfo getType()
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "getType");

        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "getType");
    
       
        return _dataType;
    }
    
    public boolean validate() throws SPLException
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "validate");

        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "validate");
    
       
        return Character.isLetter(_charValue);
    }
    
    public String toString()
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "toString");
        
        String str=Character.toString(this._charValue);
        
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "toString");
       
        return str;
    }

	public String getReferenceTypeName() throws SPLException {
		
		return null;
	}
    
}
