/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */
//

/**
 * @author Prashant Baliga <prabalig@in.ibm.com>
 *
 */
package org.apache.imperius.spl.parser.expressions.impl;

import java.util.ArrayList;
import java.util.List;
import java.util.logging.Logger;

import org.apache.imperius.spl.core.Expression;
import org.apache.imperius.spl.core.TypeConstants;
import org.apache.imperius.spl.parser.exceptions.SPLException;
import org.apache.imperius.spl.parser.expressions.SingleArgumentExpression;
import org.apache.imperius.util.SPLLogger;


public class CollectionSize extends SingleArgumentExpression implements
        Expression
{
    
    public String className = CollectionSize.class.toString();
    
    private static Logger logger = SPLLogger.getSPLLogger().getLogger();
    private static final String sourceClass="CollectionSize";
    
    
    
    public CollectionSize(List exprList, boolean evaluateExpression)
            throws SPLException
    {
        super(exprList);   
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "CollectionSize");

        //System.out.println("CollectionSize");
        if (evaluateExpression)
        {
            //System.out.println("evaluateExpression " + evaluateExpression);
            if (!validate())
            {
                logger.severe(
                        "validation error: wrong data type passed in "
                        + this._dataType);
                throw new SPLException(
                        "validation error: wrong data type passed in "
                                + this._dataType);
            }
        }
        
        this._dataType.setType(TypeConstants.intType);

        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "CollectionSize");
    
    }
    
    public Object evaluate() throws SPLException
    {  
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");

        //System.out.println("CollectionSize:evaluate");
        //System.out.println("this.exp class " + this.exp.getClass() + " type "+ this.exp.getType());
        
        Object expResult = this._exp.evaluate();
        if (!(expResult instanceof java.util.List))
        {
            logger.severe(Thread.currentThread().getName()+" "+"result of expression is not of type List");
            throw new SPLException(
                    "result of expression is not of type List");
        }
        ArrayList resultArray = (ArrayList) expResult;
        //System.out.println("resultArray size " + resultArray.size()+ " to string " + resultArray.toString());
        // //System.out.println("resultArray is of type "+expression.getType());
        if ((resultArray != null) && (!resultArray.isEmpty()))
        {
            Integer size = new Integer(resultArray.size());
            //System.out.println("Size of collection is " + size);
            logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");
            
            return size;
            
        }
        else
        {
            logger.severe(Thread.currentThread().getName()+" "+"result Array is empty");
            throw new SPLException("result Array is empty");
        }
        
    }
    
    /*
     * public int getType() {
     * 
     * return this.dataType; }
     */

    public boolean validate() throws SPLException
    {  
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "validate");

        //System.out.println("CollectionSize : validate ");
        //System.out.println("this.exp.getClass() " + this.exp.getClass()+ " type " + this.exp.getType());
        
        if (!this._exp.isArray())
        {
            //System.out.println("expression is not a valid BasicCollectExpression");
            logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "validate");
            
            return false;
        }
        else
        {
            //System.out.println("expression is a valid BasicCollectExpression");

            logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "validate");
        
            return true;
        }
        
    }

	public String getReferenceTypeName() throws SPLException {
		// TODO Auto-generated method stub
		return null;
	}
	
    public String toString()
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "toString");

        String str = "CollectionSize("+ this._exp.toString() + ")";
        	
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "toString");
       
        return str;
    }
    
}
