/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */
//

/**
 * @author Prashant Baliga <prabalig@in.ibm.com>
 *
 */
package org.apache.imperius.spl.parser.expressions.impl;

import java.util.List;
import java.util.logging.Logger;

import org.apache.imperius.spl.core.TypeConstants;
import org.apache.imperius.spl.core.TypeInfo;
import org.apache.imperius.spl.parser.exceptions.SPLException;
import org.apache.imperius.spl.parser.expressions.BooleanExpression;
import org.apache.imperius.spl.parser.expressions.SingleArgumentExpression;
import org.apache.imperius.spl.parser.util.TypeResolver;
import org.apache.imperius.util.Messages;
import org.apache.imperius.util.SPLLogger;


public class ContainsOnlyDigits extends SingleArgumentExpression implements
        BooleanExpression
{
    
    public static final String className = ContainsOnlyDigits.class.getName();
    
    private static Logger logger = SPLLogger.getSPLLogger().getLogger();
    private static final String sourceClass="ContainsOnlyDigits";
    
    
    
    public ContainsOnlyDigits(List exprList, boolean validateExpression)
            throws SPLException
    {
        super(exprList);
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "ContainsOnlyDigits");

        if (validateExpression)
        {
            if (!validate())
            {
                logger.severe(Thread.currentThread().getName()+" "+"validation error: " + className
                        + " has wrong data type passed in.");
                
                throw new SPLException(Messages.getString(
						"SPL_VALIDATION_ERROR_MSG", new Object[] { className }));
            }
        }
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "ContainsOnlyDigits");
        
    }
    
    public Object evaluate() throws SPLException
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");

        try
        {
            String s0 = (String) _exp.evaluate();
            
            if (s0.length() == 0)
            {
                return Boolean.TRUE;
            }
            
            for (int i = 0; i < s0.length(); i++)
            {
                if (!Character.isDigit(s0.charAt(i)))
                {
                    logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");
                    
                    return Boolean.FALSE;
                }
            }

            logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");
         
            return Boolean.TRUE;
        }
        catch (Exception e)
        {
            logger.severe(Thread.currentThread().getName()+" "+"evaluation error: " + e.toString());
            
            throw new SPLException(Messages.getString(
					"SPL_EVALUATION_ERROR_MSG", new Object[] { e
							.getLocalizedMessage() }));
        }
    }
    
    public boolean validate() throws SPLException
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "validate");

        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "validate");
     
        TypeInfo eType = _exp.getType();
        
        if (TypeResolver.isString(eType))
        {
            _dataType.setType(TypeConstants.booleanType);
            logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "validate");
            
            return true;
        }

        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "validate");
     
        return false;
    }
    
    public String toString()
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "toString");

        String str = "ContainsOnlyDigits( "+ this._exp.toString() + ")";
        	
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "toString");
       
        return str;
    }
}
