/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */
//

/**
 * @author Prashant Baliga <prabalig@in.ibm.com>
 *
 */
package org.apache.imperius.spl.parser.expressions.impl;

import java.util.Calendar;
import java.util.Date;
import java.util.logging.Logger;

import org.apache.imperius.spl.core.TypeConstants;
import org.apache.imperius.spl.core.TypeInfo;
import org.apache.imperius.spl.parser.exceptions.SPLException;
import org.apache.imperius.spl.parser.expressions.CalendarExpression;
import org.apache.imperius.spl.parser.expressions.ConstantExpression;
import org.apache.imperius.spl.parser.util.ExpressionUtility;
import org.apache.imperius.spl.parser.util.TypeInfoImpl;
import org.apache.imperius.util.Messages;
import org.apache.imperius.util.SPLLogger;


public class DateTimeConstant extends ConstantExpression implements
        CalendarExpression
{
    
    /*
     * value of this constant
     */
    Calendar _calendarValue;
    
    DateTime _dateTime;
    
    boolean preferStandardTime;
    
    boolean isLenient;
    
   
    
  
    
    
    private TypeInfo _dataType=new TypeInfoImpl();
    
    private static Logger logger = SPLLogger.getSPLLogger().getLogger();
    private static final String sourceClass="DateTimeConstant";
    
    
    
    public boolean isArray()
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "isArray");

        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "isArray");
     
        
        return _dataType.getIsArray();
    }
    
  
    
        
    public DateTimeConstant(String dateStr) throws SPLException 
    {
    	    	
    	_dateTime = new DateTime(dateStr);
    	_dataType.setType(TypeConstants.dateTime);
	}
    
    public DateTimeConstant(Calendar val) throws SPLException
    {
    	_dataType.setType(TypeConstants.dateTime);
    	_calendarValue = val;
    }

	/**
     * Evaluates and returns the value of this expression using the specified
     * <code>SensorLookup</code> to obtain the values of indeterminates
     * (quantities whose magnitude cannot be determined at the time of authoring
     * the expression) occuring in this expression. Since this
     * <code>Expression</code> has a constant value, the specified
     * <code>SensorLookup</code>is not used.
     * <p>
     * 
     * @param s
     *            the specified <code>SensorLookup</code>
     * @return the value of this expression
     * @throws com.ibm.autonomic.policy.expression.EvaluationException
     */
    public Object evaluate() throws SPLException
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");

        try
        {
            if (_calendarValue == null)
            {
            	_calendarValue = ExpressionUtility.computeCalendarValue(this.isLenient,
                        this.preferStandardTime, this._dateTime); 
                // Need to construct the constant
               
            }
            logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "evaluate" + _calendarValue);
                
            return _calendarValue;
            
        }
        catch (Exception e)
        {
            logger.severe(Thread.currentThread().getName()+" "+"evaluation error: " + e.toString());
            
            throw new SPLException(Messages.getString(
					"SPL_EVALUATION_ERROR_MSG", new Object[] { e
							.getLocalizedMessage() }));

        }
        
    }
    
    public TypeInfo getType()
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "getType");

        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "getType");
     
        return _dataType;
    }
    
    public String toString()
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "toString");
        String str="";
        if(this._calendarValue ==null){
        	try{
        		this.evaluate();
        	}
        	catch(SPLException e){
        		e.printStackTrace();
        		logger.severe(e.getMessage());
        	}
        }
        if(this._calendarValue !=null){
        	
        
        Calendar cal=this._calendarValue;
        Date dt=cal.getTime();
        //dt.
        //str=Integer.toString(dt.getYear())+"/"+Integer.toString(dt.getMonth())+"/"+Integer.toString(dt.getDay())+" "+Integer.toString(dt.getHours())+":"+Integer.toString(dt.getMinutes())+":"+Integer.toString(dt.getSeconds())+":"+Integer.toString(0)+" MinuteOffset:"+Integer.toString(dt.getTimezoneOffset())+" eraAD:"+Boolean.toString(true);
        str=Integer.toString(cal.get(Calendar.DAY_OF_MONTH))+"/"+Integer.toString(cal.get(Calendar.MONTH))+
        	"/"+Integer.toString(cal.get(Calendar.DAY_OF_WEEK))+" "+Integer.toString(cal.get(Calendar.HOUR_OF_DAY))+":"+
        		Integer.toString(cal.get(Calendar.MINUTE))+":"+Integer.toString(cal.get(Calendar.SECOND))+":"+Integer.toString(0)+" MinuteOffset:"+
        			Integer.toString((cal.get(Calendar.ZONE_OFFSET) + cal.get(Calendar.DST_OFFSET))/60 * 1000)+" eraAD:"+Boolean.toString(true);
        }
        else{
        	str="DateTimeConstant";
        }
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "toString");
       
        return str;
    }

	
    
}
