/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */
//

/**
 * @author Prashant Baliga <prabalig@in.ibm.com>
 *
 */
package org.apache.imperius.spl.parser.expressions.impl;

import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.apache.imperius.spl.core.TypeConstants;
import org.apache.imperius.spl.core.TypeInfo;
import org.apache.imperius.spl.parser.exceptions.SPLException;
import org.apache.imperius.spl.parser.expressions.DoubleArgumentExpression;
import org.apache.imperius.spl.parser.expressions.NumericExpression;
import org.apache.imperius.spl.parser.util.TypeResolver;
import org.apache.imperius.util.Messages;
import org.apache.imperius.util.SPLLogger;


/**
 * 
 * @author Xiping Change Log: 1. 3/9/07: Neeraj Joshi: Changed constructor to
 *         take a list to facilitate reflection in the expression factory 2.
 *         3/9/07: Neeraj Joshi: Changed the algorithm which determines return
 *         type to use numeric promotion
 * 
 */
public class Division extends DoubleArgumentExpression implements
	NumericExpression
{
    
    public static final String className = Division.class.getName();
    
    private static Logger logger = SPLLogger.getSPLLogger().getLogger();
    private static final String sourceClass="Division";
    
    
    
    public Division(List exprList, boolean validateExpression)
    throws SPLException
    {
        super(exprList);
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "Division");
        
        if (validateExpression)
        {
            if (!validate())
            {
                logger.severe(Thread.currentThread().getName()+" "+"validation error: " + className
                        + " has wrong data type passed in.");
                
                throw new SPLException(Messages.getString(
    					"SPL_VALIDATION_ERROR_MSG", new Object[] { className}));
            }
        }
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "Division");
        
    }
    
    public Object evaluate() throws SPLException
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");
        
        try
        {
            if(logger.isLoggable(Level.FINE))
                logger.fine(Thread.currentThread().getName()+" LHS/RHS "+_lhsExp.toString()+"/"+_rhsExp.toString());
            
            Number lhs = (Number) _lhsExp.evaluate();
            Number rhs = (Number) _rhsExp.evaluate();
            
            if(logger.isLoggable(Level.FINE))
                logger.fine(Thread.currentThread().getName()+" LHS,RHS::"+lhs+","+rhs);
            
            Number result=division(lhs, rhs);
            if(logger.isLoggable(Level.FINE))
                logger.fine(Thread.currentThread().getName()+" division result ::"+lhs+"/"+rhs+"="+result);
            
            logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");
            
            return result;
        }
        catch (Exception e)
        {
            logger.severe(Thread.currentThread().getName()+" "+"evaluation error: " + e.toString());
            
            throw new SPLException(Messages.getString(
					"SPL_EVALUATION_ERROR_MSG", new Object[] { e
							.getLocalizedMessage() }));

        }
    }
    
    public static Number division(Number o1, Number o2) throws SPLException
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "division");
        
        Number[] o = new Number[2];
        o[0] = o1;
        o[1] = o2;
        
//        int resolvedType = TypeResolver.resolveType(o);
        if((o1!=null)&&(o2!=null))
        {
            if(o2.intValue()!=0)
            {
                //if (resolvedType == TypeConstants.doubleType)
                //{
                Double result=new Double(o1.doubleValue() / o2.doubleValue());
                if(logger.isLoggable(Level.FINE))
                    logger.fine(Thread.currentThread().getName()+" result "+result);
                logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "division");
                
                return result;
                /* }
                else if (resolvedType == TypeConstants.floatType)
                {
                    logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "division");
                    
                    Float result=new Float(o1.floatValue() / o2.floatValue());
                    if(logger.isLoggable(Level.FINE))
                        logger.fine(Thread.currentThread().getName()+" result "+result);
                    
                    return result;
                }
                else if (resolvedType == TypeConstants.longType)
                {
                    logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "division");
                    
                    Long result=new Long(o1.longValue() / o2.longValue());
                    if(logger.isLoggable(Level.FINE))
                        logger.fine(Thread.currentThread().getName()+" result "+result);
                    
                    return result;
                }
                else if (resolvedType == TypeConstants.intType)
                {
                    logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "division");
                    
                    Integer result=new Integer(o1.intValue() / o2.intValue());
                    if(logger.isLoggable(Level.FINE))
                        logger.fine(Thread.currentThread().getName()+" result "+result);
                    
                    return result;
                }
                else if (resolvedType == TypeConstants.shortType)
                {
                    logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "division");
                    
                    Short result=new Short((short) (o1.shortValue() / o2.shortValue()));
                    if(logger.isLoggable(Level.FINE))
                        logger.fine(Thread.currentThread().getName()+" result "+result);
                    
                    return result;
                }
                else if (resolvedType == TypeConstants.byteType)
                {
                    logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "division");
                    
                    Byte result=new Byte((byte) (o1.byteValue() / o2.byteValue()));
                    if(logger.isLoggable(Level.FINE))
                        logger.fine(Thread.currentThread().getName()+" result "+result);
                    
                    return result;
                }
                else
                {
                    logger.severe(Thread.currentThread().getName()+" "+"wrong operand type.");
                    
                    throw new SPLException("wrong operand type.");
                }*/
            }
            else
            {
                logger.severe(Thread.currentThread().getName()+" "+"denominator is zero  LHS="+o1+" RHS="+o2);
                
                throw new SPLException("denominator is zero  LHS="+o1+" RHS="+o2);
            }
            
        }
        else
        {
            logger.severe(Thread.currentThread().getName()+" "+"operand is null  LHS="+o1+" RHS="+o2);
            
            throw new SPLException("operand is null  LHS="+o1+" RHS="+o2);
        }
        
    }
    
    public boolean validate() throws SPLException
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "validate");
        
        TypeInfo lType = _lhsExp.getType();
        TypeInfo rType = _rhsExp.getType();
        
        // dataType = TypeConstants.numericType;
        if (TypeResolver.isNumeric(lType) && TypeResolver.isNumeric(rType) &&
        		!lType.getIsArray() && !rType.getIsArray())
        {
            //dataType = TypeResolver
            //        .binaryNumericPromotionResolver(lType, rType);
            _dataType.setType(TypeConstants.doubleType);
            logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "validate");
            
            return true;
        }
        
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "validate");
        
        return false;
    }
    
    public String toString()
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "toString");

        String str = this._lhsExp.toString() + " / " + this._rhsExp.toString();
        	
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "toString");
       
        return str;
    }
    
}
