/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */
//

/**
 * @author Prashant Baliga <prabalig@in.ibm.com>
 *
 */
package org.apache.imperius.spl.parser.expressions.impl;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.logging.Logger;

import org.apache.imperius.spl.core.Expression;
import org.apache.imperius.spl.core.TypeConstants;
import org.apache.imperius.spl.parser.exceptions.IllegalParameterTypeException;
import org.apache.imperius.spl.parser.exceptions.SPLException;
import org.apache.imperius.spl.parser.expressions.BooleanExpression;
import org.apache.imperius.spl.parser.expressions.DoubleArgumentExpression;
import org.apache.imperius.spl.parser.util.ExpressionUtility;
import org.apache.imperius.util.Messages;
import org.apache.imperius.util.SPLLogger;


//public class InCollection extends DoubleArgumentExpression implements
//        Expression

//Xiping 03/19/2009
public class InCollection extends DoubleArgumentExpression implements
        BooleanExpression
{
    
    public String className = InCollection.class.toString();
    
    private static Logger logger = SPLLogger.getSPLLogger().getLogger();
    private static final String sourceClass="InCollection";
    
    
    
    public InCollection(List exprList, boolean evaluateExpression)
            throws SPLException
    {
        super(exprList);
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "InCollection");

        this._dataType.setType(TypeConstants.booleanType);
        this._dataType.setIsArray(false);
       //System.out.println("InCollection");
        if (evaluateExpression)
        {
           //System.out.println("evaluateExpression " + evaluateExpression);
            if (!validate())
            {

                logger.severe(
                        "validation error: wrong data type passed in "
                        + this._dataType);
             
                throw new SPLException(Messages.getString(
    					"SPL_VALIDATION_ERROR_MSG", new Object[] { className }));
            }
        }
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "InCollection");
        
    }
    
    public Object evaluate() throws SPLException
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");

        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");
     
       //System.out.println("InCollection:evaluate");
        Expression lhsExpression = (Expression) this._lhsExp;
        Expression rhsExpression = (Expression) this._rhsExp;
        if (rhsExpression.isArray())
        {
           //System.out.println("rhs is of type Basic Collection");
            Object rhsResult = rhsExpression.evaluate();
           //System.out.println("rhsResult , class " + rhsResult + " "+ rhsResult.getClass());
            if (!(rhsResult instanceof java.util.List))
            {
                logger.severe(Thread.currentThread().getName()+" "+"rhsResult is not of type List");
                throw new SPLException(Messages.getString(
						"SPL_PASSED_EXPRESSION_TYPE_ERROR_MSG", new Object[] {
								"RHS", "is not", "List" }));
            }
            // BasicCollectionExpression
            // expression=(BasicCollectionExpression)rhsExpression;
            Boolean result = Boolean.FALSE;
            ArrayList rhsResultArray = (ArrayList) rhsResult;
           //System.out.println("resultArray size " + rhsResultArray.size()+ " to string " + rhsResultArray.toString());
            ////System.out.println("resultArray is of type
            // "+expression.getType());
           //System.out.println("LHS expression is of type "+ lhsExpression.getType());
            if ((rhsResultArray != null) && (!rhsResultArray.isEmpty()))
            {
                Object lhsResult = (Object) lhsExpression.evaluate();
                // String lhsResultString=lhsResult.toString();
               //System.out.println("lhsResult " + lhsResult.toString());
                Iterator rhsResultIt = rhsResultArray.iterator();
                while (rhsResultIt.hasNext())
                {
                    Object value = (Object) rhsResultIt.next();
                    // String valueString=value.toString();
                   //System.out.println("rhsResult element " + value.toString());
                    if (ExpressionUtility.compare(lhsResult, value) == 0)
                    {
                       //System.out.println("InCollection:true");
                        result = Boolean.TRUE;
                        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");
                        
                        return result;
                    }
                    
                    /*
                     *System.out.println("rhs Array value "+value.toString());
                     * if(valueString.compareTo(lhsResultString)==0){
                     *System.out.println("InCollection:true");
                     * result=Boolean.TRUE; return result; }
                     */
                   //System.out.println("did not match");
                }
               //System.out.println("no matches: Lhs not inCollection");
                result = Boolean.FALSE;
                logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");
                
                return result;
            }
            else
            { 
                logger.severe(
                "the RHS of InCollection expression is null BasicCollection");
                throw new IllegalParameterTypeException(Messages.getString(
						"SPL_PASSED_EXPRESSION_ERROR_MSG",
						new Object[] { "RHS" }));

            }
        }
        else
        {
            logger.severe(
            "the RHS of InCollection expression should be of BasicCollection type");
            
            throw new SPLException(Messages.getString(
					"SPL_PASSED_EXPRESSION_TYPE_ERROR_MSG", new Object[] {
							"RHS", "should be", "BasicCollection" }));
        }
        
    }
    
    public boolean validate() throws SPLException
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "validate");

       //System.out.println("InCollection : validate ");
        Expression lhsExpression = (Expression) this._lhsExp;
        Expression rhsExpression = (Expression) this._rhsExp;
        
        if (rhsExpression.isArray())
        {
            if (lhsExpression.getType().getType() != rhsExpression.getType().getType())
            {
                logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "validate");
                
                throw new SPLException(Messages.getString(
						"SPL_TYPE_NOT_COMPATIBLE_EXCEPTION_MSG", new Object[] {
								lhsExpression.getType(), rhsExpression.getType() }));
            }
           //System.out.println("rhs is of type BasicCollection ,class=" + rhsExpression.getClass() + "   "+ rhsExpression.toString());
            /*
             * BasicCollectionExpression
             * rhsBasicCollectionExpression=(BasicCollectionExpression)rhsExpression;
             * System.out.println("lhs Type() : rhsBasicCollection
             * CollectionType() "+lhsExpression.getType() +" :
             * "+rhsBasicCollectionExpression.getCollectionType());
             * if(lhsExpression.getType()
             * !=rhsBasicCollectionExpression.getCollectionType()){
             * System.out.println("Validation:false "); return false; } else{
             * System.out.println("validation: true lhs:rhs
             * "+lhsExpression.getType()+":"+rhsBasicCollectionExpression.getCollectionType());
             * 
             * return true; }
             */
            logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "validate");
            
            return true;
        }
        else
        {
           //System.out.println("RHS of InCollection is not of type Basic Collection, returning false(Validation)");

            logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "validate");
            return false;
        }
        
    }
    
    public String toString()
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "toString");

        String str = "InCollection("+this._lhsExp.toString() + "," + this._rhsExp.toString() +")";
        	
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "toString");
       
        return str;
    }
    
}
