/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */
//

/**
 * @author Neeraj Joshi
 *
 */
package org.apache.imperius.spl.parser.expressions.impl;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.logging.Logger;

import org.apache.imperius.spl.core.Expression;
import org.apache.imperius.spl.parser.exceptions.SPLException;
import org.apache.imperius.spl.parser.expressions.SingleArgumentExpression;
import org.apache.imperius.spl.parser.util.ExpressionUtility;
import org.apache.imperius.util.Messages;
import org.apache.imperius.util.SPLLogger;


public class MaxInCollection extends SingleArgumentExpression implements
        Expression
{
    
    public String className = MaxInCollection.class.toString();
    
    private static Logger logger = SPLLogger.getSPLLogger().getLogger();
    private static final String sourceClass="MaxInCollection";
    
    
    
//    public boolean isArray()
//    {
//        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");
//
//        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");
//     
//        
//        return _dataType.getIsArray();
//    }
    
    public MaxInCollection(List exprList, boolean evaluateExpression)
            throws SPLException
    {
        super(exprList);
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "MaxInCollection");

       //System.out.println("MaxInCollection");
        if (evaluateExpression)
        {
           //System.out.println("evaluateExpression " + evaluateExpression);
            if (!validate())
            {
                logger.severe(
                        "validation error: wrong data type passed in "
                        + this._dataType);
                throw new SPLException(Messages.getString(
    					"SPL_VALIDATION_ERROR_MSG", new Object[] { className }));
            }
        }
        
        this._dataType.copy(this._exp.getType());
        this._dataType.setIsArray(false);
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "MaxInCollection");
        
        
    }
    
    public Object evaluate() throws SPLException
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");

       //System.out.println("MaxInCollection:evaluate");
        // int type=this.dataType;
       //System.out.println("this.exp class " + this.exp.getClass() + " type " + this.exp.getType());
       // Expression expression = (Expression) this._exp;
        // Double minNumeric=null;
        // String minString=null;
        // DateTime MinInterval=null;
        Object max = null;
        Object expResult = this._exp.evaluate();
        if (!(expResult instanceof java.util.List))
        {
            logger.severe(Thread.currentThread().getName()+" "+"result of expression is not of type List");
            throw new SPLException(Messages.getString(
					"SPL_PASSED_EXPRESSION_TYPE_ERROR_MSG", new Object[] {
							"RHS", "is not", "List" }));
        }
        ArrayList resultArray = (ArrayList) expResult;
       //System.out.println("resultArray size " + resultArray.size()+ " to string " + resultArray.toString());
        ////System.out.println("resultArray is of type "+expression.getType());
        if ((resultArray != null) && (!resultArray.isEmpty()))
        {
            Iterator resultIt = resultArray.iterator();
            while (resultIt.hasNext())
            {
                Object resultObject = resultIt.next();
               //System.out.println("resultObject,class " + resultObject + " "+ resultObject.getClass());
                if (max == null)
                {
                    max = resultObject;
                }
                
                if (ExpressionUtility.compare(max, resultObject) < 0)
                {
                   //System.out.println("resultObject " + resultObject+ " is greater than max " + max+ "   so resetting max");
                    max = resultObject;
                }
                else
                {
                   //System.out.println("resultObject " + resultObject+ " max not reset" + max);
                    
                }
                
            }
           //System.out.println(max);
            logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");
            
            return max;
        }
        else
        {
            logger.severe(Thread.currentThread().getName()+" "+"result Array is empty");
            throw new SPLException(Messages.getString(
					"SPL_BLANK_EXPRESSION_PASSED_ERROR_MSG",
					new Object[] { expResult }));
        }
        
    }
    
    /*
     * public int getType() {
     * 
     * return this.dataType; }
     */

    public boolean validate() throws SPLException
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "validate");

       //System.out.println("MaxInCollection : validate ");
       //System.out.println("this.exp.getClass() " + this.exp.getClass() + " type " + this.exp.getType());
        
        if (!this._exp.isArray())
        {
           //System.out.println("expression is not a valid BasicCollectExpression");
            logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "validate");
            
            return false;
        }
        else
        {
           //System.out.println("expression is a valid BasicCollectExpression");
            logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "validate");
            
            return true;
        }
        
    }
    
    public String toString()
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "toString");

        String str = "MaxInCollection("+this._exp.toString()+")";
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "toString");
       
        return str;
    }
    
}
