/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */
//

/**
 * @author Prashant Baliga <prabalig@in.ibm.com>
 *
 */
package org.apache.imperius.spl.parser.expressions.impl;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.logging.Logger;

import org.apache.imperius.spl.core.Expression;
import org.apache.imperius.spl.core.TypeConstants;
import org.apache.imperius.spl.parser.exceptions.SPLException;
import org.apache.imperius.spl.parser.expressions.SingleArgumentExpression;
import org.apache.imperius.spl.parser.util.ExpressionUtility;
import org.apache.imperius.util.Messages;
import org.apache.imperius.util.SPLLogger;


public class MedianInCollection extends SingleArgumentExpression implements
        Expression
{
    
    public String className = MedianInCollection.class.toString();
    
    private static Logger logger = SPLLogger.getSPLLogger().getLogger();
    private static final String sourceClass="MedianInCollection";
    
    
    
    public MedianInCollection(List exprList, boolean evaluateExpression)
            throws SPLException
    {
        super(exprList);
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "MedianInCollection");

       //System.out.println("MedianInCollection");
        if (evaluateExpression)
        {
           //System.out.println("evaluateExpression " + evaluateExpression);
            if (!validate())
            {
                logger.severe(Thread.currentThread().getName()+" "+"validation error: wrong data type passed in");
                throw new SPLException(Messages.getString(
    					"SPL_VALIDATION_ERROR_MSG", new Object[] { className }));
            }
        }
        
        this._dataType.copy(this._exp.getType());
        this._dataType.setIsArray(false);
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "MedianInCollection");
        
        
    }
    
    public Object evaluate() throws SPLException
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");

       //System.out.println("MedianInCollection:evaluate");
       //System.out.println("this.exp class " + this.exp.getClass() + " type " + this.exp.getType());
        
        Object expResult = this._exp.evaluate();
        if (!(expResult instanceof java.util.List))
        {
            logger.severe(Thread.currentThread().getName()+" "+"result of expression is not of type List");
            throw new SPLException(Messages.getString(
					"SPL_PASSED_EXPRESSION_TYPE_ERROR_MSG", new Object[] {
							"expResult", "is not", "List" }));
        }
        ArrayList resultArray = (ArrayList) expResult;
        Integer count = new Integer(resultArray.size());
       //System.out.println("resultArray size " + resultArray.size()+ " to string " + resultArray.toString());
        ////System.out.println("resultArray is of type "+expression.getType());
        if (count.intValue() < 2)
        {
            logger.severe(Thread.currentThread().getName()+" "+"result of expression is of size less than 2 hence median cannot be calculated");

            throw new SPLException(Messages.getString(
					"SPL_NO_OF_ARGUMENTS_PASSED_ERROR_MSG", new Object[] {
							"> 2", count }));
        }
        if ((resultArray != null) && (!resultArray.isEmpty()))
        {
            Collections.sort(resultArray);
            if ((count.intValue() / 2) == (count.floatValue() / 2))
            {
               //System.out.println("size of collection is even " + count);
                int medianElementPosition1 = (count.intValue()) / 2;
                int medianElementPosition2 = medianElementPosition1 - 1;
                Object medianElement1 = resultArray.get(medianElementPosition1);
                Object medianElement2 = resultArray.get(medianElementPosition2);
                Number sum = ExpressionUtility.plus((Number) medianElement1,
                        (Number) medianElement2);
                Number avg = ExpressionUtility.division(sum,
                        (Number) new Integer(2));
               //System.out.println("median is " + avg);
                logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");
                
                return avg;
                
            }
            else if (((count.intValue() / 2) != (count.floatValue() / 2)))
            {
               //System.out.println("size of collection is odd " + count);
                int medianElementPosition = (count.intValue() + 1) / 2;
                medianElementPosition = medianElementPosition - 1;
                Object median = resultArray.get(medianElementPosition);
               //System.out.println("median is " + median);
                logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");
                
                return median;
            }
            
        }
        else
        {
            logger.severe(Thread.currentThread().getName()+" "+"result Array is empty");
            throw new SPLException(Messages.getString(
					"SPL_BLANK_EXPRESSION_PASSED_ERROR_MSG",
					new Object[] { resultArray }));
        }
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");
        
        return null;
        
    }
    
    public boolean validate() throws SPLException
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "validate");

       //System.out.println("MedianInCollection : validate ");
       //System.out.println("this.exp.getClass() " + this.exp.getClass() + " type " + this.exp.getType());
        
        if (!this._exp.isArray())
        {
           //System.out.println("expression is not a valid BasicCollectExpression");
            logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "validate");
            
            return false;
        }
        
        else
        {
           //System.out.println("expression is a valid BasicCollectExpression");
            switch (this._exp.getType().getType())
            {
                case TypeConstants.byteType:
                    logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "validate");
                    
                    return true;
                case TypeConstants.doubleType:
                    logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "validate");
                    
                    return true;
                case TypeConstants.floatType:
                    logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "validate");
                    
                    return true;
                case TypeConstants.intType:
                    logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "validate");
                    
                    return true;
                case TypeConstants.longType:
                    logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "validate");
                    
                    return true;
                case TypeConstants.numericType:
                    logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "validate");
                    
                    return true;
                case TypeConstants.shortType:
                    logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "validate");
                    
                    return true;
                default:
                    logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "validate");
                
                    return false;
            }
            
        }
        
    }
    
    public String toString()
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "toString");

        String str = "MedianInCollection("+this._exp.toString()+")";
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "toString");
       
        return str;
    }
    
}
