/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */
//

/**
 * @author Neeraj Joshi
 *
 */
package org.apache.imperius.spl.parser.expressions.impl;

import java.util.List;
import java.util.logging.Logger;

import org.apache.imperius.spl.core.TypeConstants;
import org.apache.imperius.spl.core.TypeInfo;
import org.apache.imperius.spl.parser.exceptions.SPLException;
import org.apache.imperius.spl.parser.expressions.BooleanExpression;
import org.apache.imperius.spl.parser.expressions.DoubleArgumentExpression;
import org.apache.imperius.spl.parser.util.ExpressionUtility;
import org.apache.imperius.spl.parser.util.TypeResolver;
import org.apache.imperius.util.Messages;
import org.apache.imperius.util.SPLLogger;


public class NotEqual extends DoubleArgumentExpression implements
        BooleanExpression
{
    
    public static final String className = NotEqual.class.getName();
    
    private static Logger logger = SPLLogger.getSPLLogger().getLogger();
    private static final String sourceClass="NotEqual";
    
    
    
    public NotEqual(List exprList, boolean validateExpression)
            throws SPLException
    {
        super(exprList);
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "NotEqual");

        if (validateExpression)
        {
            if (!validate())
            {
                logger.severe(Thread.currentThread().getName()+" "+"validation error: " + className
                        + " has wrong data type passed in.");
                
                throw new SPLException(Messages.getString(
    					"SPL_VALIDATION_ERROR_MSG", new Object[] { className }));
            }
        }
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "NotEqual");
        
    }
    
    public Object evaluate() throws SPLException
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");

        try
        {
            Object lhsValue = _lhsExp.evaluate();
            Object rhsValue = _rhsExp.evaluate();
            
            if (ExpressionUtility.compare(lhsValue, rhsValue) != 0)
            {
                logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");
                
                return Boolean.TRUE;
            }
            else
            {
                logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");
                
                return Boolean.FALSE;
            }
        }
        catch (Exception e)
        {
            logger.severe(Thread.currentThread().getName()+" "+"evaluation error: " + e.toString());
            
            throw new SPLException(Messages.getString(
					"SPL_EVALUATION_ERROR_MSG", new Object[] { e
							.getLocalizedMessage() }));

        }
    }
    
    public boolean validate() throws SPLException
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "validate");

        TypeInfo lType = _lhsExp.getType();
        TypeInfo rType = _rhsExp.getType();
       //System.out.println("lType "+lType.getType());
       //System.out.println("rType "+rType.getType());
        
        if(!lType.getIsArray() && !rType.getIsArray())
        {	
        	//System.out.println("rType and ltype are not arrays");
            
	        if (TypeResolver.isTypeAssignableForEquality(lType, rType))
	        {
	        	//System.out.println("TypeResolver.isTypeAssignableForEquality true");
	            
	            _dataType.setType(TypeConstants.booleanType);
	            logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "validate");
	            
	            return true;
	        }
	        else 
	        {
	        	//System.out.println("TypeResolver.isTypeAssignableForEquality false");
	            
	        }
	        if (TypeResolver.isCalendar(lType) && TypeResolver.isCalendar(rType))
	        {
	        	//System.out.println("TypeResolver.isCalendar true");
	            
	            _dataType.setType(TypeConstants.booleanType);
	            logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "validate");
	            
	            return true;
	        }
	        else 
	        {
	        	//System.out.println("TypeResolver.isCalendar false");
	            
	        }
        }
        

        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "validate");
     
        return false;
    }
    
    public String toString()
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "toString");
        
        String str=this._lhsExp.toString() + " != " + this._rhsExp.toString();
        
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "toString");
       
        return str;
    }
    
}
