/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */
//

/**
 * @author Xiping Wang
 * @modified Neeraj Joshi
 *
 */
package org.apache.imperius.spl.parser.expressions.impl;

import java.util.List;
import java.util.logging.Logger;

import org.apache.imperius.spl.core.TypeConstants;
import org.apache.imperius.spl.core.TypeInfo;
import org.apache.imperius.spl.parser.exceptions.SPLException;
import org.apache.imperius.spl.parser.expressions.DoubleArgumentExpression;
import org.apache.imperius.spl.parser.expressions.NumericExpression;
import org.apache.imperius.spl.parser.util.TypeResolver;
import org.apache.imperius.util.Messages;
import org.apache.imperius.util.SPLLogger;


public class Power extends DoubleArgumentExpression implements
        NumericExpression
{
    
    public static final String className = Power.class.getName();
    
    private static Logger logger = SPLLogger.getSPLLogger().getLogger();
    private static final String sourceClass="Power";
    
    
    
    public Power(List exprList, boolean validateExpression)
            throws SPLException
    {
        super(exprList);
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "Power");

        if (validateExpression)
        {
            if (!validate())
            {
                logger.severe(Thread.currentThread().getName()+" "+"validation error: " + className
                        + " has wrong data type passed in.");
                
                throw new SPLException(Messages.getString(
    					"SPL_VALIDATION_ERROR_MSG", new Object[] { className }));
            }
        }
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "Power");
        
    }
    
    public Object evaluate() throws SPLException
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");
  try
        {
            Number lhs = (Number) _lhsExp.evaluate();
            Number rhs = (Number) _rhsExp.evaluate();

            logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");
         
            Number res = _pow(lhs, rhs); 
            return res;
        }
        catch (Exception e)
        {
            logger.severe(Thread.currentThread().getName()+" "+"evaluation errro: " + e.toString());
            
            throw new SPLException(Messages.getString(
					"SPL_EVALUATION_ERROR_MSG", new Object[] { e
							.getLocalizedMessage() }));
        }
    }
    
    private Number _pow(Number base, Number exponent)
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "pow");
        
        Number result = new Double(Math.pow(base.doubleValue(), exponent.doubleValue()));
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "pow");
     
        return result;
    }
    
    public boolean validate() throws SPLException
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "validate");

        TypeInfo lType = _lhsExp.getType();
        TypeInfo rType = _rhsExp.getType();
        
        // dataType = TypeConstants.numericType;
        if (!lType.getIsArray() && !rType.getIsArray() &&
        		TypeResolver.isNumeric(lType) && TypeResolver.isNumeric(rType))
        {
            _dataType.setType(TypeConstants.doubleType);
            logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "validate");
            
            return true;
        }

        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "validate");
     
        return false;
    }
    
    public String toString()
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "toString");
        
        String str="Power("+this._lhsExp.toString()+","+ this._rhsExp.toString() + ")";
        
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "toString");
       
        return str;
    }
    
}
