/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */
//

/**
 * @author Neeraj Joshi
 *
 */
package org.apache.imperius.spl.parser.expressions.impl;

import java.util.List;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.apache.imperius.spl.core.TypeConstants;
import org.apache.imperius.spl.core.TypeInfo;
import org.apache.imperius.spl.parser.exceptions.SPLException;
import org.apache.imperius.spl.parser.expressions.DoubleArgumentExpression;
import org.apache.imperius.spl.parser.expressions.NumericExpression;
import org.apache.imperius.spl.parser.util.ExpressionUtility;
import org.apache.imperius.spl.parser.util.TypeResolver;
import org.apache.imperius.util.Messages;
import org.apache.imperius.util.SPLLogger;


public class Product extends DoubleArgumentExpression implements
NumericExpression
{
    
    public static final String className = Product.class.getName();
    
    private static Logger logger = SPLLogger.getSPLLogger().getLogger();
    private static final String sourceClass="Product";
    
    
    
    public Product(List exprList, boolean validateExpression)
    throws SPLException
    {
        super(exprList);
        logger.entering(sourceClass, "Product");
        
        if (validateExpression)
        {
            if (!validate())
            {
                logger.severe(Thread.currentThread().getName()+" "+"validation error: " + className
                        + " has wrong data type passed in.");
                
                throw new SPLException(Messages.getString(
    					"SPL_VALIDATION_ERROR_MSG", new Object[] { className }));
            }
        }
        logger.exiting(sourceClass, "Product");
        
    }
    
    public Object evaluate() throws SPLException
    {
        logger.entering(sourceClass, "evaluate");
        
        try
        {
            if(logger.isLoggable(Level.FINE))
                logger.fine("LHS*RHS "+_lhsExp.toString()+" * "+_rhsExp.toString());
            
            Number lhs = (Number) _lhsExp.evaluate();
            Number rhs = (Number) _rhsExp.evaluate();
            
            
            
            if(logger.isLoggable(Level.FINE))
                logger.fine("LHS,RHS::"+lhs+","+rhs);
            
            Number result = ExpressionUtility.multiplication(lhs, rhs);
            
            if(logger.isLoggable(Level.FINE))
                logger.fine("product result ::"+lhs+" * "+rhs+"="+result);
            logger.exiting(sourceClass, "evaluate");
            
            return result;
        }
        catch (Exception e)
        {
            logger.severe(Thread.currentThread().getName()+" "+"evaluation error: " + e.toString());
            
            throw new SPLException(Messages.getString(
					"SPL_EVALUATION_ERROR_MSG", new Object[] { e
							.getLocalizedMessage() }));

        }
    }
    
//    private Number _product(Number o1, Number o2) throws SPLException
//    {
//        logger.entering(sourceClass, "evaluate");
//        
//        Number[] o = new Number[2];
//        o[0] = o1;
//        o[1] = o2;
//        
//        if((o1!=null)&&(o2!=null))
//        {
//            
//            //if (resolvedType == TypeConstants.doubleType)
//            //{
//            
//            
//            Double result=new Double(o1.doubleValue() * o2.doubleValue());
//            if(logger.isLoggable(Level.FINE))
//                logger.fine("result "+result);
//            logger.exiting(sourceClass, "evaluate");
//            
//            return result;
//            /* }
//            else if (resolvedType == TypeConstants.floatType)
//            {
//                logger.exiting(sourceClass, "evaluate");
//                
//                Float result=new Float(o1.floatValue() * o2.floatValue());
//                if(logger.isLoggable(Level.FINE))
//                    logger.fine("result "+result);
//                
//                return result;
//            }
//            else if (resolvedType == TypeConstants.longType)
//            {
//                logger.exiting(sourceClass, "evaluate");
//                
//                Long result=new Long(o1.longValue() * o2.longValue());
//                if(logger.isLoggable(Level.FINE))
//                    logger.fine("result "+result);
//                
//                return result;
//            }
//            else if (resolvedType == TypeConstants.intType)
//            {
//                logger.exiting(sourceClass, "evaluate");
//                
//                Long result=new Long(o1.intValue() * o2.intValue());
//                if(logger.isLoggable(Level.FINE))
//                    logger.fine("result "+result);
//                
//                return result;
//            }
//            else
//            {
//                logger.exiting(sourceClass, "evaluate");
//                
//                Integer result=new Integer(o1.byteValue() * o2.byteValue());
//                if(logger.isLoggable(Level.FINE))
//                    logger.fine("result "+result);
//                
//                return result;
//            }*/
//        }
//        else
//        {
//            logger.severe(Thread.currentThread().getName()+" "+"operand is null  LHS="+o1+" RHS="+o2);
//            
//            throw new SPLException("operand is null  LHS="+o1+" RHS="+o2);
//        }
//    }

    public static Number product(Vector o) {
    	Number[] operandArray = (Number[]) o.toArray(new Number[0]);
    	return product(operandArray);
    }

    public static Number product(Number[] o)
    {
        logger.entering(sourceClass, "product(Number[])");
        
        Number result = null;
        for (int i = 0; i < o.length; i++) {
        	if (i == 0) {
        		result = o[i];
        	} else {
        		result = new Double(result.doubleValue() * o[i].doubleValue());
        	}
        }

        logger.exiting(sourceClass, "product(Number[])");

        return result;
    }

    public static Number product(Number o1, Number o2) throws SPLException
    {
        logger.entering(sourceClass, "evaluate");
        
        Number[] o = new Number[2];
        o[0] = o1;
        o[1] = o2;
        
        if((o1!=null)&&(o2!=null))
        {
            
            //if (resolvedType == TypeConstants.doubleType)
            //{
            
            
            Double result=new Double(o1.doubleValue() * o2.doubleValue());
            if(logger.isLoggable(Level.FINE))
                logger.fine("result "+result);
            logger.exiting(sourceClass, "evaluate");
            
            return result;
            /* }
            else if (resolvedType == TypeConstants.floatType)
            {
                logger.exiting(sourceClass, "evaluate");
                
                Float result=new Float(o1.floatValue() * o2.floatValue());
                if(logger.isLoggable(Level.FINE))
                    logger.fine("result "+result);
                
                return result;
            }
            else if (resolvedType == TypeConstants.longType)
            {
                logger.exiting(sourceClass, "evaluate");
                
                Long result=new Long(o1.longValue() * o2.longValue());
                if(logger.isLoggable(Level.FINE))
                    logger.fine("result "+result);
                
                return result;
            }
            else if (resolvedType == TypeConstants.intType)
            {
                logger.exiting(sourceClass, "evaluate");
                
                Long result=new Long(o1.intValue() * o2.intValue());
                if(logger.isLoggable(Level.FINE))
                    logger.fine("result "+result);
                
                return result;
            }
            else
            {
                logger.exiting(sourceClass, "evaluate");
                
                Integer result=new Integer(o1.byteValue() * o2.byteValue());
                if(logger.isLoggable(Level.FINE))
                    logger.fine("result "+result);
                
                return result;
            }*/
        }
        else
        {
            logger.severe(Thread.currentThread().getName()+" "+"operand is null  LHS="+o1+" RHS="+o2);
            
            throw new SPLException("operand is null  LHS="+o1+" RHS="+o2);
        }
    }
    
    public boolean validate() throws SPLException
    {
        logger.entering(sourceClass, "validate");
        
        TypeInfo lType = _lhsExp.getType();
        TypeInfo rType = _rhsExp.getType();
        
        // dataType = TypeConstants.numericType;
        if (!lType.getIsArray() && !rType.getIsArray() &&
        		TypeResolver.isNumeric(lType) && TypeResolver.isNumeric(rType))
        {
            //dataType = TypeResolver
            // .binaryNumericPromotionResolver(lType, rType);
            _dataType.setType(TypeConstants.doubleType);
            logger.exiting(sourceClass, "validate");
            
            return true;
        }
        
        logger.exiting(sourceClass, "validate");
        
        return false;
    }
    
    public String toString()
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "toString");
        
        String str=this._lhsExp.toString() + "*" + this._rhsExp.toString();
        
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "toString");
       
        return str;
    }
    
}
