/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */
//

/**
 * @author Xiping Wang
 *
 */

package org.apache.imperius.spl.parser.expressions.impl;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.logging.Logger;

import org.apache.imperius.spl.core.Expression;
import org.apache.imperius.spl.evaluator.internal.impl.ReturnObjectStore;
import org.apache.imperius.spl.parser.exceptions.SPLException;
import org.apache.imperius.spl.parser.expressions.MultipleArgumentExpression;
import org.apache.imperius.util.SPLLogger;


public class ReturnValues extends MultipleArgumentExpression  
{

    private static Logger logger = SPLLogger.getSPLLogger().getLogger();
    private static final String sourceClass="ReturnValues";

	public ReturnValues(List exprList, boolean validateExpression) 
	{
		super(exprList);
		
	}

	public Object evaluate() throws SPLException 
	{
		List resultList = new ArrayList();
		Iterator expIt = _expressions.iterator();
		while (expIt.hasNext())
		{
			Expression exp = (Expression)expIt.next();
			Object result = exp.evaluate();
			resultList.add(result);
		}
		
		
		ReturnObjectStore.addReturnValues(resultList);
		return new Integer(0);
	}

	public boolean validate() throws SPLException 
	{
		
		return true;
	}
	
    public String toString()
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "toString");
        
        Iterator exprIter = this._expressions.iterator();
        String str = "[";
        while(exprIter.hasNext()) {
        	str = str + ((Expression)exprIter.next()).toString() + (exprIter.hasNext()?", ":"]");
        }
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "toString");
       
        return str;
    }

}
