/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */
//

/**
 * @author Prashant Baliga <prabalig@in.ibm.com>
 *
 */
package org.apache.imperius.spl.parser.expressions.impl;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.logging.Logger;

import org.apache.imperius.spl.core.Expression;
import org.apache.imperius.spl.core.TypeConstants;
import org.apache.imperius.spl.parser.exceptions.SPLException;
import org.apache.imperius.spl.parser.expressions.SingleArgumentExpression;
import org.apache.imperius.spl.parser.util.ExpressionUtility;
import org.apache.imperius.util.Messages;
import org.apache.imperius.util.SPLLogger;


public class SdInCollection extends SingleArgumentExpression implements
        Expression
{
    
    public String className = SdInCollection.class.toString();
    
    private static Logger logger = SPLLogger.getSPLLogger().getLogger();
    private static final String sourceClass="SdInCollection";
    
    
    
    public SdInCollection(List exprList, boolean evaluateExpression)
            throws SPLException
    {
        super(exprList);
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "SdInCollection");

       //System.out.println("SdInCollection");
        if (evaluateExpression)
        {
           //System.out.println("evaluateExpression " + evaluateExpression);
            if (!validate())
            {

                logger.severe(
                        "validation error: wrong data type passed in "
                        + this._dataType);
             
                throw new SPLException(Messages.getString(
    					"SPL_VALIDATION_ERROR_MSG", new Object[] { className }));
            }
        }
        
        this._dataType.copy(this._exp.getType());
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "SdInCollection");
        
        
    }
    
    public Object evaluate() throws SPLException
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");

       //System.out.println("SdInCollection:evaluate");
       //System.out.println("this.exp class " + this.exp.getClass() + " type "+ this.exp.getType());
        Object squareSum = null;
        Object sum = null;
        Object expResult = this._exp.evaluate();
        if (!(expResult instanceof java.util.List))
        {
            logger.severe(Thread.currentThread().getName()+" "+"result of expression is not of type List");
            throw new SPLException(Messages.getString(
					"SPL_NOT_REQUIRED_EXP_EXCEPTION_MSG",
					new Object[] { "List" }));
        }
        ArrayList resultArray = (ArrayList) expResult;
        Integer count = new Integer(resultArray.size());
       //System.out.println("resultArray size " + resultArray.size() + " to string " + resultArray.toString());
        ////System.out.println("resultArray is of type "+expression.getType());
        if ((resultArray != null) && (!resultArray.isEmpty()))
        {
            Iterator resultIt = resultArray.iterator();
            while (resultIt.hasNext())
            {
                Object resultObject = resultIt.next();
               //System.out.println("resultObject,class " + resultObject + " " + resultObject.getClass());
                if (sum == null)
                {
                    sum = resultObject;
                    
                }
                else
                {
                   //System.out .println("Adding resultObject to SdInCollection is " + resultObject + "  " + sum);
                    sum = ExpressionUtility.plus((Number) sum,
                            (Number) resultObject);
                }
                Object square = ExpressionUtility.multiplication(
                        (Number) resultObject, (Number) resultObject);
                if (squareSum == null)
                {
                    squareSum = square;
                    
                }
                else
                {
                   //System.out .println("Adding resultObject to SdInCollection is "+ resultObject + "  " + sum);
                    squareSum = ExpressionUtility.plus((Number) square,
                            (Number) squareSum);
                }
                
            }
            double mean = ((Number) sum).doubleValue() / resultArray.size();
            Number squareSumByCount = ExpressionUtility.division(
                    (Number) squareSum, (Number) count);
            Double meanSquare = new Double(mean * mean);
            Number subtractionResult = ExpressionUtility.minus(
                    squareSumByCount, (Number) meanSquare);
           //System.out.println("current SdInCollection is "+ Math.sqrt(subtractionResult.doubleValue()));
            Number result = new Double(Math.sqrt(subtractionResult.doubleValue())); 
            logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "evaluate" + result);
            
            return result;
            
        }
        else
        {

            logger.severe(Thread.currentThread().getName()+" "+"result Array is empty");
            throw new SPLException(Messages.getString(
					"SPL_BLANK_EXPRESSION_PASSED_ERROR_MSG",
					new Object[] { "expression" }));
        }
        
    }
    
    public boolean validate() throws SPLException
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "validate");

       //System.out.println("SdInCollection : validate ");
       //System.out.println("this.exp.getClass() " + this.exp.getClass() + " type " + this.exp.getType());
        
        if (!this._exp.isArray())
        {
           //System.out.println("expression is not a valid BasicCollectExpression");
            logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "validate");
            
            return false;
        }
        
        else
        {
           //System.out.println("expression is a valid BasicCollectExpression");
            switch (this._exp.getType().getType())
            {
                case TypeConstants.byteType:
                    logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "validate");
                    
                    return true;
                case TypeConstants.doubleType:
                    logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "validate");
                    
                    return true;
                case TypeConstants.floatType:
                    logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "validate");
                    
                    return true;
                case TypeConstants.intType:
                    logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "validate");
                    
                    return true;
                case TypeConstants.longType:
                    logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "validate");
                    
                    return true;
                case TypeConstants.numericType:
                    logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "validate");
                    
                    return true;
                case TypeConstants.shortType:
                    logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "validate");
                    
                    return true;
                default:
                    logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "validate");
                
                    return false;
            }
            
        }
        
    }
    
    public String toString()
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "toString");
        
        String str="SdInCollection("+this._exp.toString()+")";
        
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "toString");
       
        return str;
    }
    
}
