/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */
//

/**
 * @author Prashant Baliga <prabalig@in.ibm.com>
 *
 */
package org.apache.imperius.spl.parser.expressions.impl;

import java.util.logging.Logger;

import org.apache.imperius.spl.core.TypeConstants;
import org.apache.imperius.spl.core.TypeInfo;
import org.apache.imperius.spl.parser.exceptions.SPLException;
import org.apache.imperius.spl.parser.expressions.ConstantExpression;
import org.apache.imperius.spl.parser.expressions.StringExpression;
import org.apache.imperius.spl.parser.util.TypeInfoImpl;
import org.apache.imperius.util.Messages;
import org.apache.imperius.util.SPLLogger;

//Xiping 06/13/08
//public class StringConstant extends ConstantExpression
public class StringConstant extends ConstantExpression implements StringExpression
{
    
    private String stringValue;
    
    public TypeInfo _dataType = new TypeInfoImpl();
    
    private static Logger logger = SPLLogger.getSPLLogger().getLogger();
    private static final String sourceClass="StringConstant";
    
    
    
    public boolean isArray()
    {

        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "isArray");

        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "isArray");
     
        return false;
    }
    
    public StringConstant(String s)
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "StringConstant");

        
        s = s.substring(1, s.length() - 1);
        this.stringValue = s;
        _dataType.setType(TypeConstants.stringType);
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "StringConstant");
        
        
    }
    
    public Object evaluate() throws SPLException
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");

        try
        {
            logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");
            
            return new String(stringValue);
        }
        catch (Exception e)
        {
            logger.severe(Thread.currentThread().getName()+" "+"evaluation error: " + e.toString());
            
            throw new SPLException(Messages.getString(
					"SPL_EVALUATION_ERROR_MSG", new Object[] { e
							.getLocalizedMessage() }));

        }
    }
    
    public TypeInfo getType()
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "getType");

        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "getType");
     
        return _dataType;
    }
    
    
    public String toString()
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "toString");
        
        //xiping 03/18/2009
        //String str=this.stringValue;
        String str = "\"" + this.stringValue + "\"";
        
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "toString");
       
        return str;
    }

	public String getReferenceTypeName() throws SPLException {
		// TODO Auto-generated method stub
		return null;
	}
}
