/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */
//

/**
 * @author Prashant Baliga <prabalig@in.ibm.com>
 *
 */
package org.apache.imperius.spl.parser.expressions.impl;

import java.util.Iterator;
import java.util.List;
import java.util.logging.Logger;

import org.apache.imperius.spl.core.Expression;
import org.apache.imperius.spl.core.TypeConstants;
import org.apache.imperius.spl.parser.exceptions.SPLException;
import org.apache.imperius.spl.parser.expressions.BooleanExpression;
import org.apache.imperius.spl.parser.expressions.DoubleArgumentExpression;
import org.apache.imperius.spl.parser.util.TypeResolver;
import org.apache.imperius.util.Messages;
import org.apache.imperius.util.SPLLogger;


//public class SubCollection extends DoubleArgumentExpression implements
//        Expression

// Xiping 09/15/2009
// must implements BooleanExpression
public class SubCollection extends DoubleArgumentExpression implements
	BooleanExpression
{
    
    // private BasicCollectionExpression basicCollection=null;
    public String className = SubCollection.class.toString();
    
    private static Logger logger = SPLLogger.getSPLLogger().getLogger();
    private static final String sourceClass="SubCollection";
    
    
    
    public SubCollection(List exprList, boolean evaluateExpression)
            throws SPLException
    {
        super(exprList);
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "SubCollection");

       //System.out.println("SubCollection");
        // this.basicCollection=new BasicCollectionExpression(exprList,new
        // SPLSymbolTable());
        
        if (evaluateExpression)
        {
           //System.out.println("evaluateExpression " + evaluateExpression);
            if (!validate())
            {
                logger.severe(Thread.currentThread().getName()+" "+"validation error: wrong data type passed in");
                throw new SPLException(Messages.getString(
    					"SPL_VALIDATION_ERROR_MSG", new Object[] { className }));
            }
            
        }
        this._dataType.setType(TypeConstants.booleanType);
       
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "SubCollection");
        
    }
    
    public Object evaluate() throws SPLException
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");

       //System.out.println("SubCollection:evaluate");
        Object lhsResult = this._lhsExp.evaluate();
        Object rhsResult = this._rhsExp.evaluate();
        if (!(lhsResult instanceof java.util.List))
        {
            logger.severe(Thread.currentThread().getName()+" "+"lhsResult is not of type List");
            throw new SPLException(Messages.getString(
					"SPL_PASSED_EXPRESSION_TYPE_ERROR_MSG",
					new Object[] { "LHS","is not", "List" }));
        }
        else
        {
           //System.out.println("lhsResult is of type List");
        }
        if (!(rhsResult instanceof java.util.List))
        {
            logger.severe(Thread.currentThread().getName()+" "+"rhsResult is not of type List");
            throw new SPLException(Messages.getString(
					"SPL_PASSED_EXPRESSION_TYPE_ERROR_MSG",
					new Object[] { "RHS","is not", "List" }));

        }
        else
        {
           //System.out.println("rhsResult is of type List");
        }
        
        List lhsResultarray = (List) lhsResult;
        List rhsResultarray = (List) rhsResult;
        Boolean result = Boolean.FALSE;
        if (lhsResultarray.size() > rhsResultarray.size())
        {
            result = Boolean.FALSE;
            logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");
            
            return result;
        }
        Iterator lhsResultarrayIt = lhsResultarray.iterator();
        while (lhsResultarrayIt.hasNext())
        {
            Object lhsResultarrayElement = lhsResultarrayIt.next();
           //System.out.println("element from Lhs ResultArray " + lhsResultarrayElement + "  " + lhsResultarrayElement.getClass());
            if (!(rhsResultarray.contains(lhsResultarrayElement)))
            {
               //System.out.println("element does not exists in RHS List");
                result = Boolean.FALSE;
                logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");
                
                return result;
            }
            else
            {
               //System.out.println("element exists in RHS List");
            }
            
        }

        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");
     
        return Boolean.TRUE;
        
    }
    
    public boolean validate() throws SPLException
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "validate");

       //System.out.println("SubCollection : validate ");
        Expression lhsExpression = (Expression) this._lhsExp;
        Expression rhsExpression = (Expression) this._rhsExp;
        if ((lhsExpression == null) || (rhsExpression == null))
        {
           //System.out.println("SubCollection : One or more expressions are null ");
            logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "validate");
            
            return false;
        }
        if (!TypeResolver.areTypesEqual(lhsExpression.getType(),rhsExpression.getType()))
        {
           //System.out.println("SubCollection : lhs Type !=rhs Type " + lhsExpression.getType() + ":" + rhsExpression.getType());
            logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "validate");
            
            return false;
        }
        if (lhsExpression.isArray())
        {
           //System.out.println("lhsExpression.getType() ==TypeConstants.basicCollectionType");
            if (rhsExpression.isArray())
            {
               //System.out.println("rhsExpression.getType() ==TypeConstants.basicCollectionType");
                logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "validate");
                
                return true;
                
            }
            else
            {
               //System.out.println("rhsExpression.getType() !=TypeConstants.basicCollectionType");
                logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "validate");
                
                return false;
            }
        }
        else
        {
           //System.out.println("lhsExpression.getType() !=TypeConstants.basicCollectionType");
            logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "validate");
            
            return false;
        }
        
    }
    
    public String toString()
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "toString");
        String retStr= "SubCollection("+this._lhsExp.toString() + ","+ this._rhsExp.toString()+")";
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "toString");
       
        return retStr;
    }
    
}
