/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */
//

/**
 * @author Neeraj Joshi
 *
 */
package org.apache.imperius.spl.parser.expressions.impl;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.logging.Logger;

import org.apache.imperius.spl.core.Expression;
import org.apache.imperius.spl.parser.exceptions.SPLException;
import org.apache.imperius.spl.parser.expressions.SingleArgumentExpression;
import org.apache.imperius.spl.parser.util.ExpressionUtility;
import org.apache.imperius.spl.parser.util.TypeResolver;
import org.apache.imperius.util.Messages;
import org.apache.imperius.util.SPLLogger;


public class Sum extends SingleArgumentExpression implements Expression
{
    
    public String className = Sum.class.toString();
    
    private static Logger logger = SPLLogger.getSPLLogger().getLogger();
    private static final String sourceClass="Sum";
    
    
    
    public Sum(List exprList, boolean evaluateExpression)
            throws SPLException
    {
        super(exprList);
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "Sum");

        //System.out.println("Sum");
        if (evaluateExpression)
        {
            //System.out.println("evaluateExpression " + evaluateExpression);
            if (!validate())
            {
                logger.severe(
                        "validation error: wrong data type passed in "
                        + this._dataType);
                throw new SPLException(Messages.getString(
    					"SPL_VALIDATION_ERROR_MSG", new Object[] {sourceClass }));
            }
        }
        
        this._dataType.copy(_exp.getType());
        this._dataType.setIsArray(false);
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "Sum");
        
        
    }
    
    public Object evaluate() throws SPLException
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");

        //System.out.println("Sum:evaluate");
        //System.out.println("this.exp class " + this.exp.getClass() + " type "+ this.exp.getType());
        Object sum = null;
        Object expResult = this._exp.evaluate();
        if (!(expResult instanceof java.util.List))
        {
            logger.severe(Thread.currentThread().getName()+" "+"result of expression is not of type List");
            throw new SPLException(Messages.getString(
					"SPL_NOT_REQUIRED_EXP_EXCEPTION_MSG",
					new Object[] { "List" }));
        }
        ArrayList resultArray = (ArrayList) expResult;
        //System.out.println("resultArray size " + resultArray.size()+ " to string " + resultArray.toString());
        ////System.out.println("resultArray is of type "+expression.getType());
        if ((resultArray != null) && (!resultArray.isEmpty()))
        {
            Iterator resultIt = resultArray.iterator();
            while (resultIt.hasNext())
            {
                Object resultObject = resultIt.next();
                //System.out.println("resultObject,class " + resultObject + " " + resultObject.getClass());
                if (sum == null)
                {
                    sum = resultObject;
                }
                else
                {
                    //System.out.println("Adding resultObject to sum is "+ resultObject + "  " + sum);
                    sum = ExpressionUtility.plus((Number) sum,
                            (Number) resultObject);
                }
                //System.out.println("current sum is " + sum);
            }
            //System.out.println("sum " + sum);
            logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "evaluate "+sum);
            
            return sum;
        }
        else
        {

            logger.severe(Thread.currentThread().getName()+" "+"result Array is empty");
         
            throw new SPLException(Messages.getString(
					"SPL_BLANK_EXPRESSION_PASSED_ERROR_MSG",
					new Object[] { "" }));
        }
        
    }
    
    public boolean validate() throws SPLException
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "validate");

        //System.out.println("Sum : validate ");
       ////System.out.println("this.exp.getClass() " + this.exp.getClass()+ " type " + this.exp.getType());
        
        if (!this._exp.isArray())
        {
            //System.out.println("expression is not a valid BasicCollectExpression");
            logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "validate");
            
            return false;
        }
        
        else
        {
           ////System.out.println("expression is a valid BasicCollectExpression");
        	boolean result = TypeResolver.isNumeric(_exp.getType());
        	return result;
            /*switch (this._exp.getType())
            {
            	TypeResol
                case TypeConstants.byteType:
                    logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "validate");
                    
                    return true;
                case TypeConstants.doubleType:
                    logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "validate");
                    
                    return true;
                case TypeConstants.floatType:
                    logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "validate");
                    
                    return true;
                case TypeConstants.intType:
                    logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "validate");
                    
                    return true;
                case TypeConstants.longType:
                    logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "validate");
                    
                    return true;
                case TypeConstants.numericType:
                    logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "validate");
                    
                    return true;
                case TypeConstants.shortType:
                    logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "validate");
                    
                    return true;
                default:
                    logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "validate");
                
                    return false;
            }*/
            
        }
        
    }
    
    public String toString()
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "toString");
        String retStr= "SUM("+this._exp.toString() + ")";
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "toString");
       
        return retStr;
    }
    
    
}
