/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */
//

/**
 * @author Prashant Baliga <prabalig@in.ibm.com>
 *
 */
package org.apache.imperius.spl.parser.expressions.impl;

import java.util.List;
import java.util.logging.Logger;

import org.apache.imperius.spl.core.TypeConstants;
import org.apache.imperius.spl.core.TypeInfo;
import org.apache.imperius.spl.parser.exceptions.SPLException;
import org.apache.imperius.spl.parser.expressions.NumericExpression;
import org.apache.imperius.spl.parser.expressions.SingleArgumentExpression;
import org.apache.imperius.spl.parser.util.TypeResolver;
import org.apache.imperius.util.Messages;
import org.apache.imperius.util.SPLLogger;


public class ToSINT32 extends SingleArgumentExpression implements
        NumericExpression
{
    
    public static final String className = ToSINT32.class.getName();
    
    private static Logger logger = SPLLogger.getSPLLogger().getLogger();
    private static final String sourceClass="ToSINT32";
    
    
    
    public ToSINT32(List exprList, boolean validateExpression)
            throws SPLException
    {
        super(exprList);
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "ToSINT32");

        if (validateExpression)
        {
            if (!validate())
            {
                logger.severe(Thread.currentThread().getName()+" "+"validation error: " + className
                        + " has wrong data type passed in.");
                
                throw new SPLException(Messages.getString(
    					"SPL_VALIDATION_ERROR_MSG", new Object[] { sourceClass }));
            }
        }
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "ToSINT32");
        
    }
    
    public Object evaluate() throws SPLException
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");

        try
        {
            Object obj = _exp.evaluate();
            Integer result = null;
            if (obj instanceof Number)
            {
            	result = new Integer(((Number) obj).intValue()); 
                logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");
                
                return result;
            }
            else
            {
            	result = new Integer((String) obj); 
                logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");
                
                // must be string type
                return result;
            }
        }
        catch (Exception e)
        {
            logger.severe(Thread.currentThread().getName()+" "+"evaluation error: " + e.toString());
            
            throw new SPLException(Messages.getString(
					"SPL_EVALUATION_ERROR_MSG", new Object[] { e
							.getLocalizedMessage() }));

        }
    }
    
    public boolean validate() throws SPLException
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "validate");

        TypeInfo eType = _exp.getType();
        
        // dataType = TypeConstants.numericType;
        if (TypeResolver.isNumeric(eType) || TypeResolver.isString(eType))
        {
            _dataType.setType(TypeConstants.intType);
            logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "validate");
            
            return true;
        }

        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "validate");
     
        return false;
    }
    
    public String toString()
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "toString");
        String retStr= "toSINT32("+this._exp.toString() + ")";
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "toString");
       
        return retStr;
    }
    
}
