/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */
//

/**
 * @author Neeraj Joshi <jneeraj@us.ibm.com>
 *
 */
package org.apache.imperius.spl.parser.expressions.impl;

import java.util.List;
import java.util.logging.Logger;

import org.apache.imperius.spl.core.TypeConstants;
import org.apache.imperius.spl.parser.exceptions.SPLException;
import org.apache.imperius.spl.parser.expressions.NumericExpression;
import org.apache.imperius.spl.parser.expressions.SingleArgumentExpression;
import org.apache.imperius.spl.parser.util.TypeResolver;
import org.apache.imperius.util.Messages;
import org.apache.imperius.util.SPLLogger;


public class UnaryMinusExpression extends SingleArgumentExpression implements
        NumericExpression
{
    private static Logger logger = SPLLogger.getSPLLogger().getLogger();
    private static final String sourceClass="UnaryMinusExpression";
    
    
    
    public UnaryMinusExpression(List exprList, boolean validateExpression)
            throws SPLException
    {
        super(exprList);
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "UnaryMinusExpression");

        if (validateExpression)
        {
            if (!validate())
            {
                logger.severe(
                "validation error: wrong data type passed in.");
                
                throw new SPLException(Messages.getString(
    					"SPL_VALIDATION_ERROR_MSG", new Object[] { sourceClass }));
            }
        }
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "UnaryMinusExpression");
        
    }
    
    // @Override
    public Object evaluate() throws SPLException
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");

        // TODO Auto-generated method stub
        Object n = _exp.evaluate();
        Number result = null;
        if (_dataType.getType() == TypeConstants.longType)
        {
            long s = ((Long) n).shortValue();
            result = new Long(-s);

           
        }
        else if (_dataType.getType() == TypeConstants.floatType)
        {
            float f = ((Float) n).floatValue();
            result = new Float(-f);
            
        }
        else if (_dataType.getType() == TypeConstants.doubleType)
        {
            double f = ((Double) n).doubleValue();
            result = new Double(-f);
          
        }
        else
        {
            int i = ((Integer) n).intValue();
            result = new Integer(-i);
          
        }
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "evaluate" + result);
        
        return result;
        
        // n = 0 - n;
        // return null;
    }
    
    // @Override
    public boolean validate() throws SPLException
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "validate");

        if (TypeResolver.isNumeric(_exp.getType()))
        {
            _dataType.copy(_exp.getType());
            logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "validate");
            
            return true;
        }
        // TODO Auto-generated method stub
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "validate");
        
        return false;
    }
    
    public String toString()
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "toString");
        String retStr= this._exp.toString() + "(...--...)";
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "toString");
       
        return retStr;
    }
    
}
