/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */
//
/**
 * @author Neeraj Joshi <jneeraj@us.ibm.com>
 *
 */
package org.apache.imperius.spl.parser.factory;

import java.lang.reflect.Constructor;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Logger;

import org.apache.imperius.spl.core.Expression;
import org.apache.imperius.spl.parser.compiler.symboltable.MacroSymbol;
import org.apache.imperius.spl.parser.compiler.symboltable.MethodSymbol;
import org.apache.imperius.spl.parser.compiler.symboltable.SPLSymbolTable;
import org.apache.imperius.spl.parser.compiler.symboltable.Symbol;
import org.apache.imperius.spl.parser.exceptions.SPLException;
import org.apache.imperius.spl.parser.expression.primary.MacroExpression;
import org.apache.imperius.util.Messages;
import org.apache.imperius.util.SPLLogger;

public class BuitInOperationExpressionFactory 
{

	/*
	 * stringLength toUpper toLower concatenate substring matchesRegExp
	 * leftSubstring rightSubstring middleSubstring replaceSubstring toShort
	 * toInteger toLong toFloat toDouble toDoubleInteger toUnsignDoubleInteger
	 * toLongInteger toUnsignLongInteger toInteger toUnsignInteger toShort
	 * toUnsignShort toBoolean word startsWith endsWith contains
	 * containsOnlyDigits containsOnlyLetters containsOnlyLettersOrDigits
	 * containsOnlyLettersOrDigits("a1b2C3") like min max remainder power abs
	 * toString toDouble toFloat toLong toDoubleInteger toUnsignDoubleInteger
	 * toLongInteger toUnsignLongInteger toInteger toUnsignInteger toShort
	 * toUnsignShort toBoolean round exp log sqrt floor ceiling getMillisecond
	 * getSecond getMinute getHour12 getHour24 getDayOfWeek getDayOfWeekInMonth
	 * getDayOfMonth getDayOfYear getWeekOfMonth getWeekOfYear getMonth getYear
	 * isWithin toMilliseconds
	 * 
	 */

	private static Logger logger = SPLLogger.getSPLLogger().getLogger();

	private static final String sourceClass = "MethodExpressionFactory";

	public static final String stringLength = "stringLength";

	public static final String toUpper = "toUpper";

	public static final String toLower = "toLower";

	public static final String concatenate = "concatenate";

	public static final String substring = "substring";

	public static final String matchesRegExp = "matchesRegExp";

	public static final String leftSubstring = "leftSubstring";

	public static final String rightSubstring = "rightSubstring";

	public static final String middleSubstring = "middleSubstring";

	public static final String replaceSubstring = "replaceSubstring";

	public static final String word = "word";

	public static final String startsWith = "startsWith";

	public static final String endsWith = "endsWith";

	public static final String contains = "contains";

	public static final String containsOnlyDigits = "containsOnlyDigits";

	public static final String containsOnlyLetters = "containsOnlyLetters";

	public static final String containsOnlyLettersOrDigits = "containsOnlyLettersOrDigits";

	public static final String min = "min";

	public static final String max = "max";

	public static final String remainder = "remainder";

	public static final String power = "power";

	public static final String abs = "abs";

	public static final String toString = "toString";

	public static final String toDouble = "toDouble";

	public static final String toFloat = "toFloat";

	public static final String toLong = "toLong";

	public static final String toDoubleInteger = "toDoubleInteger";

	public static final String toUnsignDoubleInteger = "toUnsignDoubleInteger";

	public static final String toLongInteger = "toLongInteger";

	public static final String toUnsignLongInteger = "toUnsignLongInteger";

	public static final String toInteger = "toInteger";

	public static final String toUnsignInteger = "toUnsignInteger";

	public static final String toShort = "toShort";

	public static final String toUnsignShort = "toUnsignShort";

	public static final String toBoolean = "toBoolean";

	public static final String round = "round";

	public static final String exp = "exp";

	public static final String sqrt = "sqrt";

	public static final String floor = "floor";

	public static final String ceiling = "ceiling";

	public static final String getMillisecond = "getMillisecond";

	public static final String getSecond = "getSecond";

	public static final String getMinute = "getMinute";

	public static final String getHour12 = "getHour12";

	public static final String getHour24 = "getHour24";

	public static final String getDayOfWeek = "getDayOfWeek";

	public static final String getDayOfWeekInMonth = "getDayOfWeekInMonth";

	public static final String getDayOfMonth = "getDayOfMonth";

	public static final String getDayOfYear = "getDayOfYear";

	public static final String getWeekOfMonth = "getWeekOfMonth";

	public static final String getWeekOfYear = "getWeekOfYear";

	public static final String getMonth = "getMonth";

	public static final String getYear = "getYear";

	public static final String isWithin = "isWithin";

	public static final String toMilliseconds = "toMilliseconds";

	public static final String minInCollection = "minInCollection";

	public static final String inCollection = "inCollection";

	public static final String union = "union";

	public static final String maxInCollection = "maxInCollection";

	public static final String collectionSize = "collectionSize";

	public static final String sum = "sum";

	public static final String avrgInCollection = "avrgInCollection";

	public static final String eqCollections = "eqCollections";

	public static final String subCollection = "subCollection";

	public static final String sdInCollection = "sdInCollection";

	public static final String medianInCollection = "medianInCollection";

	public static final String allInCollection = "allInCollection";

	public static final String anyInCollection = "anyInCollection";

	public static final String applyToCollection = "applyToCollection";

	public static final String log10 = "log10";

	public static final String ln = "ln";

	public static final String getCurrentTime = "getCurrentTime";

	public static final String invokePolicy = "invokePolicy";

	public static final String returnValue = "ReturnValue";

	public static Expression createExpression(String id1, String id2,
			List pList, SPLSymbolTable symTab) throws SPLException 
	{

		logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "createExpression");

		boolean recurse = true;
		Expression returnExpr = null;
		if (!id1.equals("") && !id2.equals("")) 
		{
			/*
			 * if(symTab.symbolExists(id1 + "." + id2, recurse)) { Symbol symbol =
			 * symTab.getSymbol(id1 + "." + id2); if(symbol instanceof
			 * MethodSymbol) { returnExpr = new
			 * MethodExpression((MethodSymbol)symbol,pList,symTab); } else {
			 * throw new SPLException("undefined method" + id1 + "." + id2); } }
			 */
			logger.severe("id1.id2 type of method calls not allowed");

			throw new SPLException(Messages
					.getString("SPL_EXPRESSION_FACTORY_EXCEPTION_MSG"));
		} 
		else if (!id1.equals("") && id2.equals("")) 
		{

			if (symTab.symbolExists(id1, recurse)) 
			{
//				TODO xiping
//				Collection c = symTab.getSymbol(id1);
				Symbol symbol = (Symbol)symTab.getSymbol(id1);
//				Symbol symbol = null;
				if (symbol instanceof MacroSymbol) 
				{
					returnExpr = new MacroExpression((MacroSymbol) symbol,
							(ArrayList) pList, symTab);

				} 
				else if (symbol instanceof MethodSymbol) 
				{
					logger.severe("id1.id2 type of method calls not allowed");

					throw new SPLException(Messages
							.getString("SPL_EXPRESSION_FACTORY_EXCEPTION_MSG"));
				}
			} 
			else 
			{
				if (ACPLParserMap.isKnownOperatorExpressionType(id1)) 
				{
					//System.out.println("creating inbuilt function expression");
					returnExpr = createBuiltInExpression(id1, pList, symTab);
				} 
				else 
				{
					logger
							.severe(Thread.currentThread().getName()
									+ " "
									+ "the symbol does not exist in the symbol table and is not a known inbuilt function");

					throw new SPLException(Messages.getString(
							"SPL_SYMBOL_DOES_NOT_EXIST_EXCEPTION_MSG",
							new Object[] { "" }));
				}
			}
		}

		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "createExpression");

		return returnExpr;
	}

	public static Expression createBuiltInExpression(String expName,
			List paramList, SPLSymbolTable symTab) throws SPLException 
	{
		logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "createBuiltInExpression");

		boolean isAction = false;
		Expression returnExpression = _createBuiltInType(expName, paramList,
				symTab, isAction);
	
		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "createBuiltInExpression");

		return returnExpression;
	}

	public static Expression createBuiltInAction(String expName,
			List paramList, SPLSymbolTable symTab) throws SPLException 
	{
		logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "createBuiltInAction");

		boolean isAction = true;
		Expression returnExpression = _createBuiltInType(expName, paramList,
				symTab, isAction);

		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "createBuiltInAction");

		return returnExpression;
	}

	private static Expression _createBuiltInType(String expName,
			List paramList, SPLSymbolTable symTab, boolean isAction)
			throws SPLException 
	{
		logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "createACPLExpression");

		Expression returnExpression = null;
		Boolean validateExpression = Boolean.TRUE;
		if (symTab.getSymbolTableType() == SPLSymbolTable.COLLECT) 
		{
			validateExpression = Boolean.FALSE;
		}
		Class handlerClass = null;
		try 
		{
			if (!isAction) 
			{
				handlerClass = ACPLParserMap
						.getClassForOperatorExpressionType(expName);
			} 
			else 
			{
				handlerClass = ACPLParserMap
						.getClassForActionExpressionType(expName);
			}
		} 
		catch (ClassNotFoundException e) 
		{
			//            Object[] args = new Object[]
			//            {
			//                expName.toString()
			//            };
			// {ERROR_EXPRESSION_CREATION_CLASS_PATH,
			// "Autonomic Manager Library cannot create an expression for
			// element {0}
			// because the expression handler class could not be found or
			// loaded."},
			logger.severe(Thread.currentThread().getName() + " "
					+ "No such expression" + expName);

			throw new SPLException(Messages.getString(
					"SPL_CLASS_NOT_FOUND_EXCEPTION_MSG", new Object[] { e
							.getLocalizedMessage() }));
		}

		/*
		 * There are two cases:
		 * 
		 * 1) the "regular" case like And expression, where the element has only
		 * children elements representing known functions. In the regular case,
		 * the children are converted to expression and passed as a Vector to
		 * the appropriate expression handler.
		 * 
		 * 2) a "SpecialCase" where the element has attributes or supporting
		 * elements that are not known functions (like PolicyTimePeriod). In
		 * this case, the whole element is passed to a SpecialCaseElementParser
		 * that does a special conversion to the appropriate expression handler.
		 */
		{
			// Do the normal case when the element just has children elements
			// Do all children

			Class[] argClassArray = { List.class, boolean.class };
			Object[] argObjectArray = { paramList, validateExpression };

			// Try to build the expression now
			/*
			 * Constructor expressionConstructor; try { expressionConstructor =
			 * handlerClass.getConstructor(new Class[] { List.class,
			 * Boolean.class }); return
			 * (Expression)expressionConstructor.newInstance( new Object[] {
			 * parameters,validateExpression }); } catch (Exception e1) {
			 * Object[] elementArg = new Object[] { expName, handlerClass };
			 * 
			 * throw new SPLException("error instantiating"); }
			 */

			Constructor exprConstructor;
			try 
			{
				Constructor[] ctors = handlerClass.getConstructors();
				for (int k = 0; k < ctors.length; k++) 
				{
					//System.out.println("Constructor name: "+ ctors[k].getName());
					Class[] paramTps = ctors[k].getParameterTypes();
					for (int z = 0; z < paramTps.length; z++) 
					{
						//System.out.println("param type " + paramTps[z].getName());
					}
				}
				exprConstructor = handlerClass.getConstructor(argClassArray);
				returnExpression = (Expression) exprConstructor
						.newInstance(argObjectArray);
				//System.out.println("returnExpression "+ returnExpression.toString());

			} 
			catch (Exception e) 
			{
				logger.severe(Thread.currentThread().getName() + " "
						+ "error instantiating" + e.getMessage());
				throw new SPLException(e.getMessage());
			}
		}
		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "createACPLExpression");

		return returnExpression;
	}

}
