/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */
//

/**
 * @author Prashant Baliga <prabalig@in.ibm.com>
 *
 */
package org.apache.imperius.spl.parser.statements.actions.impl;


import java.util.logging.Logger;

import org.apache.imperius.spl.core.Actuator;
import org.apache.imperius.spl.parser.exceptions.SPLException;
import org.apache.imperius.spl.parser.statements.ActionBlock;
import org.apache.imperius.util.SPLLogger;


public class ConcurrentActionBlock extends DoubleArgumentActionBlock
{
    private static Logger logger = SPLLogger.getSPLLogger().getLogger();
    
    private static final String sourceClass="ConcurrentActionBlock";
    
    public ConcurrentActionBlock(ActionBlock a1, ActionBlock a2)
    {
        super(a1, a2);
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "ConcurrentActionBlock");

        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "ConcurrentActionBlock");
 
    }
    
    // @Override
    /**
     * If the block is of the form &lt;action block&gt; || &lt;action block&gt;, both blocks
     * are evaluated concurrently and the whole block succeeds as soon as one of
     * the two internal blocks succeeds.
     * @throws SPLException 
     */
    public boolean execute(Actuator ac) throws SPLException
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "execute");

        
        boolean result1 = _leftActionBlock.execute(ac);
        boolean result2 = _rightActionBlock.execute(ac);
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "execute");
        
        return result1 || result2;
    }
    
}
