/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */
//

/**
 * @author Prashant Baliga <prabalig@in.ibm.com>
 *
 */
package org.apache.imperius.spl.parser.statements.actions.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.logging.Logger;

import org.apache.imperius.spl.core.Actuator;
import org.apache.imperius.spl.core.Expression;
import org.apache.imperius.spl.parser.compiler.symboltable.SPLSymbolTable;
import org.apache.imperius.spl.parser.exceptions.SPLException;
import org.apache.imperius.spl.parser.expression.primary.PrimaryExpression;
import org.apache.imperius.spl.parser.expressions.impl.AssignmentExpression;
import org.apache.imperius.spl.parser.statements.BasicActionBlock;
import org.apache.imperius.util.Messages;
import org.apache.imperius.util.SPLLogger;




public class SetActionBlock implements BasicActionBlock
{
    
    private List _paramList;
    
    private String _className;
    
    private SPLSymbolTable _symTab;
    
    private static Logger logger = SPLLogger.getSPLLogger().getLogger();

    private static final String sourceClass="SetActionBlock";
   
    
    public SetActionBlock(String className, List paramList, SPLSymbolTable symTab)
    {
        logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "SetActionBlock");
		// System.out.println("creating SetActionBlock "+className);

		_className = className;
		_paramList = paramList;
		_symTab = symTab;

		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "SetActionBlock");
    }
    
    public boolean execute(Actuator ac) throws SPLException
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "execute");

       //System.out.println("Executing set action  on "+this.getClassName());
        HashMap modifiedProperties = new HashMap();
        String instanceName = _className;
//        try
//        {
           //System.out.println("size of parameters in set action block :"+ this._paramList.size());
            for (int i = 0; i < _paramList.size(); i++)
            {
                if (_paramList.get(i) != null)
                {
                   //System.out.println("setactionblock::paramList.get(i) "+ this._paramList.get(i));
                    if (_paramList.get(i) instanceof AssignmentExpression)
                    {
                        AssignmentExpression assignmentExpression = 
                        	(AssignmentExpression) _paramList.get(i);
                        PrimaryExpression lhsExpression = 
                        	(PrimaryExpression) assignmentExpression.getLHSExpression();
                        
                        Expression rhsExpression = 
                        	(Expression) assignmentExpression.getRHSExpression();
                        instanceName = lhsExpression.getclassNameOrInstanceVariableName();
                        String keyName = lhsExpression.getPropertyName();
                       /* int indOfDot = keyName.indexOf('.');
                        if (indOfDot != -1)
                        {
                        	instanceName = keyName.substring(0,indOfDot);
                            keyName = keyName.substring(indOfDot + 1);
                        }*/
                        
                        Object keyValue = rhsExpression.evaluate();
                        modifiedProperties.put(keyName, keyValue);
                        
                    }
                    else
                    {

    					logger.severe(Thread.currentThread().getName()+" "+"paramlist in SetActionBlock does not consist of asignment expressions");
    					throw new SPLException(
								Messages
										.getString("SPL_SET_ACTION_BLOCK_EXCEPTION_MSG"));
                    }
                }
                else
                {
					logger.severe(Thread.currentThread().getName() + " "
							+ "the parameter at position " + i + " is null ");
					throw new SPLException(Messages.getString(
							"SPL_BLANK_EXPRESSION_PASSED_ERROR_MSG",
							new Object[] { "" + i }));
				}
            }
            Object instance = _symTab.getInstance(instanceName);
			String qualifier = _symTab.getQualifierForClass(_className);
			ac.modifyInstance(_className, qualifier, instance,
					modifiedProperties);

			logger.exiting(sourceClass, Thread.currentThread().getName() + " "
					+ "execute");
            
            return true;
//        }
//        catch (SPLException e)
//        {
//            logger.severe(e.getMessage());
//
//            logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "execute");
//     
//            return false;
//        }
    }
    
    public String getClassName()
    {
        logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "getClassName");

		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "getClassName");

		return _className;
    }
    
    public void setClassName(String className)
    {
        logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "setClassName");

		_className = className;

		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "setClassName");
    }
    
    public List getParamList()
    {
        logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "getParamList");

		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "getParamList");

		return _paramList;
    }
    
    public void setParamList(ArrayList paramList)
    {
        logger.entering(sourceClass, Thread.currentThread().getName() + " "
				+ "setParamList");

		_paramList = paramList;

		logger.exiting(sourceClass, Thread.currentThread().getName() + " "
				+ "setParamList");
    }
}
