/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */
//

/**
 * @author Neeraj Joshi <jneeraj@us.ibm.com>
 *
 */
package org.apache.imperius.spl.parser.statements.impl;
import java.util.logging.Logger;

import org.apache.imperius.spl.core.Argument;
import org.apache.imperius.spl.core.TypeConstants;
import org.apache.imperius.spl.core.TypeInfo;
import org.apache.imperius.spl.parser.exceptions.SPLException;
import org.apache.imperius.spl.parser.util.TypeInfoImpl;
import org.apache.imperius.util.SPLLogger;

public class ArgumentImpl implements Argument 
{
    private TypeInfo _type;
    private String _name;
   // private boolean _isArray;
   // private String _referenceName = "";
    private Object value=null;
    private static Logger logger = SPLLogger.getSPLLogger().getLogger();
    
    
    private static final String sourceClass="Argument";
    
    
    public ArgumentImpl(String type, String name, boolean isArray, String referenceName) throws SPLException
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "Argument");
        int tp = TypeConstants.getActualType(type); 
        _type = new TypeInfoImpl(tp,referenceName,isArray);
        _name = name;
       // _isArray = isArray;
       // _referenceName = referenceName;
        
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "Argument");
        
    }
    
    public ArgumentImpl(int type, String name, boolean isArray, String referenceName) throws SPLException
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "Argument");
        
        _type = new TypeInfoImpl(type,referenceName,isArray);
        _name = name;
     //   _isArray = isArray;
     //   _referenceName = referenceName;
        
        
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "Argument");
        
    }
    
    /* (non-Javadoc)
	 * @see org.apache.imperius.spl.parser.statements.impl.IArgument#getReferenceTypeName()
	 */
    public String getReferenceTypeName() 
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "getReferenceTypeName");
        
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "getReferenceTypeName");
        
        return _type.getReferenceTypeName();
    }
    
    /* (non-Javadoc)
	 * @see org.apache.imperius.spl.parser.statements.impl.IArgument#setReferenceTypeName(java.lang.String)
	 */
    public void setReferenceTypeName(String name) 
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "setReferenceTypeName");
        
        _type.setReferenceTypeName( name);
        
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "setReferenceTypeName");
        
    }
    
    
    
    /* (non-Javadoc)
	 * @see org.apache.imperius.spl.parser.statements.impl.IArgument#getName()
	 */
    public String getName() 
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "getName");
        
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "getName");
        
        return _name;
    }
    
    /* (non-Javadoc)
	 * @see org.apache.imperius.spl.parser.statements.impl.IArgument#setName(java.lang.String)
	 */
    public void setName(String name) 
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "setName");
        
        this._name = name;
        
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "setName");
        
    }
    
    /* (non-Javadoc)
	 * @see org.apache.imperius.spl.parser.statements.impl.IArgument#getType()
	 */
    public TypeInfo getType() 
    {       
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "getType");
        
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "getType");
        
        return _type;
    }
    
    /* (non-Javadoc)
	 * @see org.apache.imperius.spl.parser.statements.impl.IArgument#setType(java.lang.String)
	 */
    public void setType(String type) throws SPLException
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "setType");
        int tp = TypeConstants.getActualType(type);
        
        this._type = new TypeInfoImpl(tp,null,false);
        
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "setType");
    }
    
    /* (non-Javadoc)
	 * @see org.apache.imperius.spl.parser.statements.impl.IArgument#getIsArray()
	 */
    public boolean getIsArray()
    {
    	return _type.getIsArray();
    }
    
    /* (non-Javadoc)
	 * @see org.apache.imperius.spl.parser.statements.impl.IArgument#setIsArray(boolean)
	 */
    public void setIsArray(boolean a)
    {
    	_type.setIsArray(a);
    }

	/* (non-Javadoc)
	 * @see org.apache.imperius.spl.parser.statements.impl.IArgument#getValue()
	 */
	public Object getValue() {
		return value;
	}

	/* (non-Javadoc)
	 * @see org.apache.imperius.spl.parser.statements.impl.IArgument#setValue(java.lang.Object)
	 */
	public void setValue(Object value) {
		this.value = value;
	}
    
}
