/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */
//

/**
 * @author Neeraj Joshi <jneeraj@us.ibm.com>
 *
 */
package org.apache.imperius.spl.parser.statements.impl;

import java.util.ArrayList;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.apache.imperius.spl.core.Expression;
import org.apache.imperius.spl.parser.compiler.symboltable.PropertySymbol;
import org.apache.imperius.spl.parser.compiler.symboltable.SPLSymbolTable;
import org.apache.imperius.spl.parser.exceptions.SPLException;
import org.apache.imperius.util.SPLLogger;


public class DeclarationStatement 
{
	private ArrayList constantList = new ArrayList();
	private ArrayList macroList = new ArrayList();
	private SPLSymbolTable symTab;
	private static Logger logger = SPLLogger.getSPLLogger().getLogger();
    private static final String sourceClass="DeclarationStatement";
    
	
	public DeclarationStatement(ArrayList cList, ArrayList mList, SPLSymbolTable sTab)
	{
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "DeclarationStatement");
        
		constantList = cList;
		macroList = mList;
		this.symTab = sTab;

        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "DeclarationStatement");
   
	}

	public ArrayList getConstantList() {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "getConstantList");
        
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "getConstantList");
   
		return constantList;
	}

	public void setConstantList(ArrayList constantList) {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "setConstantList");
        
		this.constantList = constantList;

        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "setConstantList");
   
	}

	public ArrayList getMacroList() {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "getMacroList");
        
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "getMacroList");
   
		return macroList;
	}

	public void setMacroList(ArrayList macroList) {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "setMacroList");

        this.macroList = macroList;
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "setMacroList");
   
	}

	public SPLSymbolTable getSymTab() {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "getSymTab");
        
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "getSymTab");
   
		return symTab;
	}

	public void setSymTab(SPLSymbolTable symTab) {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "setSymTab");

        this.symTab = symTab;
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "setSymTab");
   
	}
	
	public void evaluate() throws SPLException
	{
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");
        
		for(int i=0; i<constantList.size(); i++)
		{
			ConstantDefinition cd = (ConstantDefinition)constantList.get(i);
			Expression e=cd.getInitExpr();
			if(logger.isLoggable(Level.FINE))
				logger.fine(Thread.currentThread().getName()+ "constant expression "+e.toString());
            Object result = e.evaluate();
			String constantSymbol = cd.getIdent();
			// xiping's modification, 06/23/09
			PropertySymbol constantSym = (PropertySymbol)symTab.getSymbol(constantSymbol);
//			Collection c = symTab.getSymbol(constantSymbol);
//			Iterator it = c.iterator();
//			PropertySymbol constantSym = (PropertySymbol)it.next();
            if(logger.isLoggable(Level.FINE))
                logger.fine(Thread.currentThread().getName()+"constantSym name"+ constantSym.getName()+" current value:"+constantSym.getValue()+" setting value to:"+result.toString());
            
            constantSym.setValue(result);
		}

        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");
   
	}

}
