/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */
//

/**
 * @author Neeraj Joshi <jneeraj@us.ibm.com>
 *
 */
package org.apache.imperius.spl.parser.statements.impl;

import java.util.logging.Level;
import java.util.logging.Logger;

import org.apache.imperius.spl.core.Actuator;
import org.apache.imperius.spl.core.DataCollector;
import org.apache.imperius.spl.core.Expression;
import org.apache.imperius.spl.parser.compiler.symboltable.SPLSymbolTable;
import org.apache.imperius.spl.parser.exceptions.SPLException;
import org.apache.imperius.spl.parser.statements.ActionBlock;
import org.apache.imperius.spl.parser.statements.EvaluationResults;
import org.apache.imperius.spl.parser.statements.PolicyStatement;
import org.apache.imperius.spl.parser.statements.PolicyStatementResult;
import org.apache.imperius.util.SPLLogger;


public class PolicyDefinition implements PolicyStatement 
{
    private DeclarationStatement _declStmt;
    private Expression _condition;
    private ActionBlock _decision;
    int _priority;
    private SPLSymbolTable _symTab;
    private String auditloggerString = "";

    private static Logger logger = SPLLogger.getSPLLogger().getLogger();
    
//    private static Logger auditLogger = SPLLogger.getSPLLogger().getAuditLogger();
    
    private static final String sourceClass="PolicyDefinition";
    
    
    
    public PolicyDefinition(DeclarationStatement d, Expression con, ActionBlock dec, 
            String priority, SPLSymbolTable sTab)
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "PolicyDefinition");
        
        _declStmt = d;
        _condition = con;
        if(con==null){
            if(logger.isLoggable(Level.FINE))
                logger.fine(Thread.currentThread().getName()+"PolicyDefinition condition is null");
        }
        else{
            if(logger.isLoggable(Level.FINE))
                logger.fine(Thread.currentThread().getName()+"PolicyDefinition condition "+con.toString());
        }
        _decision = dec;
        if(dec==null){
            if(logger.isLoggable(Level.FINE))
                logger.fine(Thread.currentThread().getName()+"PolicyDefinition decision is null");
        }
        else{
            if(logger.isLoggable(Level.FINE))
                logger.fine(Thread.currentThread().getName()+"PolicyDefinition decision "+dec.toString());
        }
        
        _priority = Integer.parseInt(priority);
        _symTab = sTab;
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "PolicyDefinition");
        
    }
    
    public Expression getCondition() {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "getCondition");
        
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "getCondition");
        
        return _condition;
    }
    
    public void setCondition(Expression condition) {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "setCondition");
        
        _condition = condition;
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "setCondition");
        
    }
    
    public ActionBlock getDecision() {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "getDecision");
        
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "getDecision");
        
        return _decision;
    }
    
    public void setDecision(ActionBlock decision) {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "setDecision");
        
        _decision = decision;
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "setDecision");
        
    }
    
    public DeclarationStatement getDeclStmt() {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "getDeclStmt");
        
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "getDeclStmt");
        
        return _declStmt;
    }
    
    public void setDeclStmt(DeclarationStatement declStmt) {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "setDeclStmt");
        
        _declStmt = declStmt;
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "setDeclStmt");
        
    }
    
    public int getPriority() {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "getPriority");
        
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "getPriority");
        
        return _priority;
    }
    
    public void setPriority(int priority) {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "setPriority");
        _priority = priority;
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "setPriority");
        
        
    }
    
    public SPLSymbolTable getSymTab() {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "getSymTab");
        
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "getSymTab");
        
        return _symTab;
    }
    
    public void setSymTab(SPLSymbolTable symTab) {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "setSymTab");
        
        _symTab = symTab;
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "setSymTab");
        
    }
    
    public int evaluate(DataCollector dc , Actuator ac) throws SPLException {
    	EvaluationResults er = evaluateForResults(dc,ac);
    	return er.getStatusCode();
    }
    
    
    public EvaluationResults evaluateForResults(DataCollector dc , Actuator ac) throws SPLException
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");
        this.auditloggerString="";
//        EvaluationResult result = new EvaluationResult(EvaluationResult.POLICY_EVALUATED_SUCCESSFULLY);
        int result = SPLPolicy.POLICY_EVALUATED_SUCCESSFULLY;
        Boolean conditionResult = null;
        Boolean decisionResult = null;
        
        if(_declStmt != null)
        {	
            _declStmt.evaluate();
        }    
        if(_condition != null)
        {	
            
            if(logger.isLoggable(Level.FINE))
            {	
                logger.fine(Thread.currentThread().getName()+
                		"evaluating condition of Policy Definition");
            }    
            conditionResult = (Boolean)_condition.evaluate();
//            System.out.println("     "+"policy :"+_condition.toString());
            auditloggerString+="     "+"policy :"+_condition.toString()+"\n";
//            System.out.println("     "+"result :"+conditionResult);
            auditloggerString+="     "+"result :"+conditionResult+"\n";
            
			
//            System.out.println("condition : "+_condition.toString());
//            System.out.println("evaluated to : "+conditionResult);
            
            if(logger.isLoggable(Level.FINE))
            {	
            	logger.fine(Thread.currentThread().getName()+
                		"condition="+_condition.toString());
                logger.fine(Thread.currentThread().getName()+
                		"evaluation of condition complete, return value="+conditionResult);
            }  
//            if(auditLogger.isLoggable(Level.FINE))
//            {	
//                auditLogger.fine(Thread.currentThread().getName()+
//                		"Condition : "+this.getCondition().toString());
//            }    
//            
//            if(auditLogger.isLoggable(Level.FINE))
//            {	
//                auditLogger.fine(Thread.currentThread().getName()+
//                		"Condition Evaluated to : "+conditionResult.toString());
//            }    
            
            if(conditionResult.booleanValue())
            {
                if(logger.isLoggable(Level.FINE))
                    logger.fine(Thread.currentThread().getName()+"Condition is True!");
                //modifyInstance(String className,String namespace,Map instanceKeys, Map properties)
                if(logger.isLoggable(Level.FINE))
                    logger.fine(Thread.currentThread().getName()+"executing Decision of Policy Definition");
                //System.out.println("keys string : "+this.symTab.getInstance().getKeys());
//                if(auditLogger.isLoggable(Level.FINE))
//                    auditLogger.fine(Thread.currentThread().getName()+" Executing Decision ");
                boolean executionResult = _decision.execute(ac);
                decisionResult = Boolean.valueOf(executionResult);
                // System.out.println("     "+"decision executed :"+executionResult);
                auditloggerString+="     "+"decision executed :"+executionResult+"\n";
                
//                if(auditLogger.isLoggable(Level.FINE))
//                    auditLogger.fine(Thread.currentThread().getName()+"PolicyDefinition decision.execute return value="+executionResult);
                
                if(executionResult)
                {
//                    if(auditLogger.isLoggable(Level.FINE))
//                        auditLogger.fine(Thread.currentThread().getName()+" Policy Evaluated Successfully ");
                    
                    result = SPLPolicy.POLICY_EVALUATED_SUCCESSFULLY;
                    // System.out.println("     "+"sub policy decision execution successful");
                    auditloggerString+="     "+"sub policy decision execution successful"+"\n";
                    
                    if(logger.isLoggable(Level.FINE))
                        logger.fine(Thread.currentThread().getName()+"Policy was executed successfully");
                   //System.out.println(" POLICY_EVALUATED_SUCCESSFULLY ");
                    
                    logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");
                    
//                    return result;
                }
                else{
                    
//                    auditLogger.severe(Thread.currentThread().getName()+" "+" Policy Evaluation Failed ");
                    
                    result = SPLPolicy.POLICY_EVALUATION_FAILED;
                    // System.out.println("     "+"sub policy decision execution failed");
                    auditloggerString+="     "+"sub policy decision execution failed"+"\n";
                    
//                    auditLogger.severe(Thread.currentThread().getName()+" "+"PolicyDefinition Evalaute return value="+result);
                   //System.out.println(" POLICY_EVALUATION_FAILED ");
                    
                    logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");
                    
//                    return result;
                }
                
            }
            else
            {
//                if(auditLogger.isLoggable(Level.FINE))
//                    auditLogger.fine(Thread.currentThread().getName()+" Policy was not Evaluated as Condition evaluated to False ");
                
                if(logger.isLoggable(Level.FINE))
                    logger.fine(Thread.currentThread().getName()+"Condition is False!");
                //result = SPLPolicy.POLICY_NOT_EVALUATED;
                if(logger.isLoggable(Level.FINE))
                    logger.fine(Thread.currentThread().getName()+"Policy was not executed");
               //System.out.println(" CONDITION evaluated to FALSE ");
                
                // System.out.println("     "+"sub policy decision not executed");
                auditloggerString+="     "+"sub policy decision not executed"+"\n";
                
                logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");
                if(logger.isLoggable(Level.FINE))
                    logger.fine(Thread.currentThread().getName()+"PolicyDefinition Evalaute return value="+result);
                
//                return result;
            }
        }
        else
        {
        	// System.out.println("     "+"sub policy has no condition");
        	auditloggerString+="     "+"sub policy has no condition"+"\n";
            
            boolean executionResult=_decision.execute(ac);
            decisionResult = Boolean.valueOf(executionResult);
            if(logger.isLoggable(Level.FINE))
                logger.fine(Thread.currentThread().getName()+"PolicyDefinition decision.execute return value="+executionResult);
            System.out.println("     "+"decision executed :"+executionResult);
            auditloggerString+="     "+"decision executed :"+executionResult+"\n";
            
            if(executionResult)
            {
//                if(auditLogger.isLoggable(Level.FINE))
//                    auditLogger.fine(Thread.currentThread().getName()+"Policy without condition, was executed successfully");
                
                result = SPLPolicy.POLICY_EVALUATED_SUCCESSFULLY;
                // System.out.println("     "+"sub policy decision execution successful");
                auditloggerString+="     "+"sub policy decision execution successful"+"\n";
                
                if(logger.isLoggable(Level.FINE))
                    logger.fine(Thread.currentThread().getName()+"Policy without condition, was executed successfully");
               //System.out.println(" POLICY_EVALUATED_SUCCESSFULLY ");
                
                logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");
                if(logger.isLoggable(Level.FINE))
                    logger.fine(Thread.currentThread().getName()+"PolicyDefinition Evalaute return value="+result);
                
//                return result;
            }
            else{
//                if(auditLogger.isLoggable(Level.FINE))
//                    auditLogger.fine(Thread.currentThread().getName()+"Evaluation of Policy without condition Failed");
                
                result = SPLPolicy.POLICY_EVALUATION_FAILED;
                // System.out.println("     "+"sub policy decision execution failed");
                auditloggerString+="     "+"sub policy decision execution failed"+"\n";
                
               //System.out.println(" POLICY_EVALUATION_FAILED ");
                
                logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");
                if(logger.isLoggable(Level.FINE))
                    logger.fine(Thread.currentThread().getName()+"PolicyDefinition Evalaute return value="+result);
                
//                return result;
            }
        }
        
        PolicyStatementResult psr;
        if (conditionResult != null)  {
        	psr = new PolicyStatementResult(result, conditionResult, decisionResult);
        } else {
        	psr = new PolicyStatementResult(result, decisionResult);
        }
        return new EvaluationResults(psr);
    }
    
    
    public String toString(){
    	
    	return this._condition.toString();
    	
    }

	public String getAuditLogString() {
		
		return this.auditloggerString;
	}
    
}
