/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */
//

/**
 * @author Neeraj Joshi <jneeraj@us.ibm.com>
 *
 */
package org.apache.imperius.spl.parser.statements.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.apache.imperius.spl.core.Actuator;
import org.apache.imperius.spl.core.DataCollector;
import org.apache.imperius.spl.core.InstanceInfo;
import org.apache.imperius.spl.parser.compiler.symboltable.SPLSymbolTable;
import org.apache.imperius.spl.parser.exceptions.SPLException;
import org.apache.imperius.spl.parser.statements.EvaluationResults;
import org.apache.imperius.spl.parser.statements.PolicyStatement;
import org.apache.imperius.util.SPLLogger;


public class SPLPolicy 
{
	private List _importStmtList;
	private StrategyStatement _strgyStmt;
	private DeclarationStatement _declaration;
	private ArrayList _policyList;
	private SPLSymbolTable _symTab;
	
	public static final int POLICY_EVALUATED_SUCCESSFULLY = 1;
	public static final int POLICY_NOT_EVALUATED = 0;
	public static final int POLICY_EVALUATION_FAILED = -1;
	
	private static Logger logger = SPLLogger.getSPLLogger().getLogger();
	private String auditloggerString = "";

	private static final String sourceClass="SPLPolicy";



	public SPLPolicy(List importStatementList, StrategyStatement s,DeclarationStatement decl,ArrayList pList, SPLSymbolTable symTab2)
	{
		logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "SPLPolicy");


		this._importStmtList = importStatementList;
		this._strgyStmt = s;
		this._declaration = decl;
		this._policyList = sortPoliciesByPriority(pList);

		this._symTab = symTab2;
		logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "SPLPolicy");

	}

//	private ArrayList sortPoliciesByPriority(ArrayList list) 
//	{
//		logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "sortPoliciesByPriority");
//		
//		for(int i=0; i<list.size(); i++)
//		{
//			for(int j=i+1; j<list.size()-1; j++)
//			{
//				PolicyStatement pStatement = (PolicyStatement)list.get(i);
//				PolicyStatement pStatementNext = (PolicyStatement)list.get(j);
//				if(pStatement.getPriority() < pStatementNext.getPriority())
//				{
//					PolicyStatement temp = pStatement;
//					pStatement = pStatementNext;
//					pStatementNext = temp;
//				}
//			}
//		}
//		logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "sortPoliciesByPriority");
//		return list;
//	}

	//Changed by Xiping to comply with the CIM-SPL specification
	private ArrayList sortPoliciesByPriority(ArrayList al) 
	{
		logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "sortPoliciesByPriority");

		ArrayList list = (ArrayList) al.clone(); 
		ArrayList result = new ArrayList();
		while (list.size() > 0) {
			int index = 0;
			for(int j = 1; j < list.size(); j++) {
				PolicyStatement pStatement = (PolicyStatement)list.get(0);
				PolicyStatement pStatementNext = (PolicyStatement)list.get(j);
				if(pStatement.getPriority() > pStatementNext.getPriority()) {
					index = j;
				}
			}
			result.add(list.remove(index));
		}
		
		logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "sortPoliciesByPriority");
		
		return result;
	}

	

	private EvaluationResults _evaluatePolicies(Map instanceMap, 
			DataCollector dc, 
			Actuator ac) 
	throws SPLException
	{
		logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "evaluatePolicies");

		// Iterate over the List of maps
		EvaluationResults result = new EvaluationResults();
		Iterator instanceMapKeyIt = instanceMap.keySet().iterator();
//		System.out.println("Iterating over all instances of Anchor class");

		while(instanceMapKeyIt.hasNext())
		{

			String  className = (String)instanceMapKeyIt.next();	
			List instancesForClass = (List)instanceMap.get(className);
			Iterator instancesForClassIt = instancesForClass.iterator();
			if(logger.isLoggable(Level.FINE))
				logger.fine(Thread.currentThread().getName()+" iterating over instances ");
//			System.out.println(">>>>>>>>>>>"+"Starting policy Evaluation"+"<<<<<<<<<<<");
//			auditloggerString+=">>>>>>>>>>>"+"Starting policy Evaluation"+"<<<<<<<<<<<"+"\n";
			int instanceCount=1;
			while(instancesForClassIt.hasNext())
			{	
//				System.out.println("+++++++++++"+"Evaluating policy against instance "+instanceCount+"+++++++++++");
				auditloggerString+="+++++++++++"+"Evaluating policy against instance "+instanceCount+"+++++++++++"+"\n";
				
				
				InstanceInfo instanceOfClass = (InstanceInfo)instancesForClassIt.next();
				String instanceName = instanceOfClass.getInstanceName();
				Object instance = instanceOfClass.getInstance();
				Map instanceSymbolMap = instanceOfClass.getInstanceSymbolsMap();
				// populate the symboltable with information from the map
				_symTab.populateInstanceValuesForInstance(className, instanceName,instance, instanceSymbolMap);
			}
		 }	
		//evaluate the declaration to populate the values for the constants

		 if(_declaration != null)
		 {	
			_declaration.evaluate();
		 }    
		// iterate over all the policies
		for(int j=0; j<_policyList.size();j++)
		{

//			System.out.println("***********"+"evaluating sub policy number "+(j+1)+"***********");
			auditloggerString+="***********"+"evaluating sub policy number "+(j+1)+"***********"+"\n";
			PolicyStatement pStatement = (PolicyStatement)_policyList.get(j);
			// evaluate each policy	
			EvaluationResults er1 = pStatement.evaluateForResults(dc,ac);
			result.appendResult(er1);
			int result1 = result.getStatusCode(); // pStatement.evaluate(dc,ac);
			auditloggerString+=pStatement.getAuditLogString();

			if(result1 == POLICY_EVALUATION_FAILED) // report failure even if a single policy fails
			{
//				result = result1;
//				System.out.println("     "+"SPLPolicy evaluation failed ");
				auditloggerString+="     "+"SPLPolicy evaluation failed "+"\n";
							
				logger.severe(Thread.currentThread().getName()+"SPLPolicy evaluation failed return value="+result1);


			}
			else
			{
//				System.out.println("     "+"SPLPolicy evaluation successful ");
				auditloggerString+="     "+"SPLPolicy evaluation successful "+"\n";
			}
			// if strategy is to execute first applicable and if the policy has been evaluated
			// terminate

			if(_strgyStmt.getStrategy().equals("Execute_First_Applicable"))
			{
				if(result1 == POLICY_EVALUATED_SUCCESSFULLY || result1 == POLICY_EVALUATION_FAILED)
				{
//					System.out.println("***********"+"completed evaluation of sub policy number "+(j+1)+"***********");
					auditloggerString+="***********"+"completed evaluation of sub policy number "+(j+1)+"***********"+"\n";
					
					break;
				}
			}

//			System.out.println("***********"+"completed evaluation of sub policy number "+(j+1)+"***********");
			auditloggerString+="***********"+"completed evaluation of sub policy number "+(j+1)+"***********"+"\n";
			
		}

		
		if(logger.isLoggable(Level.FINE))
			logger.fine(Thread.currentThread().getName()+"SPLPolicy EvaluatePolicies return value="+result);

		if(logger.isLoggable(Level.FINE))
			logger.fine(Thread.currentThread().getName()+auditloggerString);

		logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "evaluatePolicies");

		return result;
	}

	

	public DeclarationStatement getDeclarationStatement() {
		logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "getDeclarationStatement");

		logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "getDeclarationStatement");

		return _declaration;
	}

	public void setDeclList(DeclarationStatement decl) {
		logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "setDeclList");

		this._declaration = decl;
		logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "setDeclList");

	}

	public List getImportStmtList() {
		logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "getImportStmt");

		logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "getImportStmt");

		return _importStmtList;
	}

	public void setImportStmtList(List importStmtList) {
		logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "setImportStmt");

		this._importStmtList = importStmtList;
		logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "setImportStmt");

	}

	public ArrayList getPolicyList() {
		logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "getPolicyList");

		logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "getPolicyList");

		return _policyList;
	}

	public void setPolicyList(ArrayList policyList) {
		logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "setPolicyList");

		this._policyList = policyList;
		logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "setPolicyList");

	}

	public StrategyStatement getStrgyStmt() {
		logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "getStrgyStmt");

		logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "getStrgyStmt");

		return _strgyStmt;
	}

	public void setStrgyStmt(StrategyStatement strgyStmt) {
		logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "setStrgyStmt");

		this._strgyStmt = strgyStmt;
		logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "setStrgyStmt");

	}

	public DeclarationStatement getDeclaration() {
		logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "getDeclaration");

		logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "getDeclaration");

		return _declaration;
	}

	public void setDeclaration(DeclarationStatement declaration) {
		logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "setDeclaration");
		this._declaration = declaration;
		logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "setDeclaration");


	}

	public SPLSymbolTable getSymTab() {
		logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "getSymTab");

		logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "getSymTab");

		return _symTab;
	}

	public void setSymTab(SPLSymbolTable symTab) {
		logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "setSymTab");
		this._symTab = symTab;
		logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "setSymTab");


	}

	
	public EvaluationResults evaluateForResults(DataCollector dc, Actuator ac,
			String anchorClassName,String anchornamespace, 
			Object anchorInstance, String assocName, String role1, 
			String role2) throws SPLException 
	{
		logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");
		this.auditloggerString="";
		
		EvaluationResults result = new EvaluationResults();
//		System.out.println("cimObjectName "+anchorClassName+" "+anchornamespace+" "+anchorInstance.toString());
		//Map keys=this.symTab.getInstance().getMapOfKeys();
		ImportStatement imp=(ImportStatement)this._importStmtList.get(0);
		InstanceInfo ii=(InstanceInfo)imp.getImportInstanceInfoList().get(0);
		String targetclassnm = ii.getInstanceName();
		List targetClassInstances = dc.getAssociatedInstanceReferences(anchorInstance, anchornamespace, targetclassnm, assocName, role1, role2);
		
		Iterator it=targetClassInstances.iterator();
//		System.out.println(">>>>>>>>>>>"+"Policy Group Evaluation - number of associated instances :"+targetClassInstances.size()+"<<<<<<<<<<<");
		
		auditloggerString+=">>>>>>>>>>>"+"Policy Group Evaluation - number of associated instances :"+targetClassInstances.size()+"<<<<<<<<<<<"+"\n";
		
		while(it.hasNext()){
			Object inst=it.next();
			//InstanceInfo iinfo=new InstanceInfo(targetclassnm,inst);
//			List instanceInfoList=new ArrayList();
			//instanceInfoList.add(iinfo);

			Map objMap=new HashMap();
			objMap.put(targetclassnm, inst);
			result.appendResult(evaluateForResults(dc,ac,objMap));

		}
//		System.out.println(">>>>>>>>>>>"+"policy evaluation complete, result :"+result+"<<<<<<<<<<<");
//		auditloggerString+=">>>>>>>>>>>"+"policy evaluation complete, result :"+result+"<<<<<<<<<<<"+"\n";
		

		logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "evaluate");

		return result;
	}

	public int evaluate(DataCollector dc, Actuator ac, Map iMap) throws SPLException 
    {
		EvaluationResults er = evaluateForResults(dc,ac,iMap);
		return er.getStatusCode();
    }

	public EvaluationResults evaluateForResults(DataCollector dc, Actuator ac, Map iMap) throws SPLException 
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "evaluate2");
        this.auditloggerString="";
        if (logger.isLoggable(Level.FINE))
            logger.fine(Thread.currentThread().getName()+"evaluating policy against instances cimPolicy");
        Map instanceMap =  _buildInstanceMap(iMap,dc);
        
        
        EvaluationResults result;
        if(!instanceMap.isEmpty())
        {
            if (logger.isLoggable(Level.FINE))
            {   
                logger.fine(Thread.currentThread().getName()+
                        "number of instances passed in for evaluation is "+instanceMap.size());
            }    
//            System.out.println(">>>>>>>>>>>"+"number of instances passed in for evaluation is :"+instanceMap.size()+"<<<<<<<<<<<");
            
            auditloggerString+=">>>>>>>>>>>"+"number of instances passed in for evaluation is :"+instanceMap.size()+"<<<<<<<<<<<"+"\n";
                
            
            result = _evaluatePolicies(instanceMap,dc,ac);
            
//            System.out.println(">>>>>>>>>>>"+"policy evaluation complete, result :"+result+"<<<<<<<<<<<");
            auditloggerString+=">>>>>>>>>>>"+"policy evaluation complete, result :"+result+"<<<<<<<<<<<"+"\n";
            
            
        }
        else
        {
            result = new EvaluationResults();
            logger.warning(Thread.currentThread().getName()+" "+"No valid instances for policy evaluation");
            //throw new SPLException("Out of the Instances that were passed in for Evaluation, none have passed the filter");
        }
        if (logger.isLoggable(Level.FINE))
            logger.fine(Thread.currentThread().getName()+"result of evaluation is "+result);

        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "evaluate2");

        return result;
    }
	
	/**
     * This method filters out instances passed to the evaluate method
     * that are not present in the import statement of the policy and builds
     * the InstanceMap that is used internally
     * @param instanceMap
     * @return
     * @throws ClassNotFoundException 
     */
    
	private Map _buildInstanceMap(Map iMap,DataCollector dc) throws SPLException
    {
	    Map instMap = new HashMap();
       Iterator instNameIt = iMap.keySet().iterator();
        while(instNameIt.hasNext())
        {
            String instanceName = (String)instNameIt.next();
            Object instance = iMap.get(instanceName);
       //     String instanceClassName = instance.getClass().getName();
            Iterator importIt = _importStmtList.iterator();
            while(importIt.hasNext())
            {
                ImportStatement importStmt = (ImportStatement)importIt.next();
                String anchorClassName = importStmt.getAnchorClassName();
                                 
                // check if class of the passed instance has been imported
                if(dc.isInstance(anchorClassName,instance))
                {
                    
                    List instanceList = importStmt.getImportInstanceInfoList();
                    Iterator instanceListIt = instanceList.iterator();
                    // iterate over all the instances of this class to see if 
                    // there is a match
                    while(instanceListIt.hasNext())
                    {
                        InstanceInfo iInfo = (InstanceInfo)instanceListIt.next();
                        if(iInfo.getInstanceName().equals(instanceName))
                        {
                            InstanceInfo instanceInfo = new InstanceInfo(instanceName,instance);
                            
                            String qualifier = _symTab.getQualifierForClass(anchorClassName);
                            Map symbolsForInstance = dc.getSymbolsForInstance(anchorClassName,
                                        qualifier,
                                        instance);
                            instanceInfo.setInstanceSymbolsMap(symbolsForInstance);
                            if(instMap.containsKey(anchorClassName))
                            {
                                List instanceInfoList = (List)instMap.get(anchorClassName);
                                instanceInfoList.add(instanceInfo);
                                
                            }
                            else
                            {
                                List instanceInfoList = new ArrayList(1);	// DAW
                                instanceInfoList.add(instanceInfo);
                                instMap.put(anchorClassName, instanceInfoList);
                                
                            }   
                            
                        }
                    }
                }
                    
            }
        }
                
        return instMap;
        
    }

    
	

    public String getAuditLogString() {
		
		return this.auditloggerString;
	}

	private boolean _instancesPassImportFilter(DataCollector dc,Object instance)
	{
		return true;
		/*logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "instanceFilterByCOP");
        String classnm=this.importStmt.getClassName();
        String namespc=this.importStmt.getNameSpace();

        String copShouldStartWith=this.getSymTab().getAnchor();
       //System.out.println("cop Should contain : "+copShouldStartWith);
       //System.out.println("cop is : "+cop);
        if(cop.toString().indexOf(copShouldStartWith) != -1)
        {
            try
            {
                Map instMap=dc.getInstanceMap(classnm, namespc, cop.toString());
                symTab.populateSymbolTableWithInstanceValues(instMap);
                Expression importFilterExpression = importStmt.getExpr();
                if(importFilterExpression != null)
                {
                    if (logger.isLoggable(Level.FINE))
                        logger.fine(Thread.currentThread().getName()+"importFilterExpression != null");
                    // filter the List of Maps based on the import statement filter expression
                    Boolean filterResult = (Boolean)importFilterExpression.evaluate();
                    if(filterResult.booleanValue())
                    {
                        if (logger.isLoggable(Level.FINE))
                            logger.fine(Thread.currentThread().getName()+"filterResult.booleanValue() = true");
                        return true;
                    }
                    else
                    {
                        if (logger.isLoggable(Level.FINE))
                            logger.fine(Thread.currentThread().getName()+"filterResult.booleanValue() = false");
                        return false;
                    }
                }
                else
                {
                    // no filtering required
                    return true;
                }

            }
            catch(SPLException e)
            {
                logger.severe(e.getMessage());
            }



            //System.out.println("passed the filter");

        }
        //System.out.println("did not passed the filter");
        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "instanceFilterByCOP");

        return false;*/
	}
	
	/*  private List filterInstancesByImportFilter(List instSymbolMaps, Expression importFilterExpression) throws SPLException 
    {
        logger.entering(sourceClass,Thread.currentThread().getName()+" "+ "filterInstancesByImportFilter");

        List filteredList = new ArrayList();
        Iterator listIt = instSymbolMaps.iterator();
        while(listIt.hasNext())
        {
            Map instSymbols = (Map)listIt.next();
            symTab.populateSymbolTableWithInstanceValues(instSymbols);
            Boolean filterResult = (Boolean)importFilterExpression.evaluate();
            if(filterResult.booleanValue())
            {
                filteredList.add(instSymbols);
            }
        }

        logger.exiting(sourceClass,Thread.currentThread().getName()+" "+ "filterInstancesByImportFilter");

        return filteredList;
    }*/

	
	

}
