/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */
//

/**
 * @author Prashant Baliga <prabalig@in.ibm.com>
 *
 */


package org.apache.imperius.spl.parser.util;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Properties;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.apache.imperius.util.SPLLogger;



public class PropertiesLoader 
{
	private static String CLASSNAME = "PropertiesLoader";

	private static Logger logger = Logger.getLogger(PropertiesLoader.class.getName());
	private static Properties props = null;
	private static String PROPERTIES_FILE = "customexpressions.properties";
	
	private void _loadProperties() 
	{
		logger.entering(CLASSNAME,"PropertiesLoader:_loadProperties");
		props = new Properties();
		try
		{
			SPLLogger.SPL_HOME = System.getProperty("user.dir");
			if(logger.isLoggable(Level.FINE))
				logger.fine("Trying to Load " + SPLLogger.SPL_HOME + File.separator + PROPERTIES_FILE);
			File propertiesFile = new File(SPLLogger.SPL_HOME + File.separator + PROPERTIES_FILE);
			if(!propertiesFile.exists())
			{	
				FileWriter fstream = new FileWriter(SPLLogger.SPL_HOME + File.separator + PROPERTIES_FILE);
				BufferedWriter out = new BufferedWriter(fstream);
				out.write("SPL_CUSTOM_EXPRESSIONS");
				out.write("\n");
				out.write("SPL_CUSTOM_ACTIONS");
				out.write("\n");
				    //Close the output stream
				out.close();
				
			}
			FileInputStream inputStream = new FileInputStream(propertiesFile);
			props.load(inputStream);
			if(logger.isLoggable(Level.FINE))
				logger.fine("Loaded " + SPLLogger.SPL_HOME + File.separator + PROPERTIES_FILE);
			inputStream.close();
		}
		catch(FileNotFoundException fnfe)
		{
			logger.severe("File not found " + SPLLogger.SPL_HOME + File.separator + PROPERTIES_FILE);
		}
		catch(IOException ioe)
		{
			logger.severe("IO Exception loading " + SPLLogger.SPL_HOME + File.separator + PROPERTIES_FILE);
		}
		
	}
	
	public Properties getProperties()
	{
		if(props == null)
		{	
			_loadProperties();
		}
		
		return props;
	}
}	
	
	
