/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */
//

/**
 * @author Prashant Baliga <prabalig@in.ibm.com>
 *
 */

package org.apache.imperius.spl.parser.util;

import org.apache.imperius.spl.core.TypeConstants;
import org.apache.imperius.spl.core.TypeInfo;
import org.apache.imperius.spl.parser.exceptions.SPLException;
import org.apache.imperius.util.Messages;


public class TypeInfoImpl implements TypeInfo 
{
	private int _splType = TypeConstants.INVALID;
	private String _referenceTypeName = null;
	private boolean _isArray = false;
	
	
	public TypeInfoImpl()
	{
		
		
	}
	public TypeInfoImpl(int type)
	{
		_splType = type;
		_referenceTypeName = null;
		_isArray = false;
	}
	
	public TypeInfoImpl(TypeInfo type)
	{
		_splType = type.getType();
		_referenceTypeName = type.getReferenceTypeName();
		_isArray = type.getIsArray();
	}
		
	public TypeInfoImpl(int splType, String referenceTypeName, boolean isArr) throws SPLException
	{
		if(TypeResolver.isValidType(splType))
		{	
			_splType = splType;
			_referenceTypeName = referenceTypeName;
			_isArray = isArr;
		}
		else
		{
            throw new SPLException(Messages
					.getString("SPL_WRONG_OPERAND_TYPE_EXCEPTION_MSG"));
		}
	}

	/* (non-Javadoc)
	 * @see org.apache.imperius.spl.parser.util.TypeInfo#getReferenceTypeName()
	 */
	public String getReferenceTypeName() {
		return _referenceTypeName;
	}

	/* (non-Javadoc)
	 * @see org.apache.imperius.spl.parser.util.TypeInfo#setReferenceTypeName(java.lang.String)
	 */
	public void setReferenceTypeName(String typeName) {
		_referenceTypeName = typeName;
	}

	/* (non-Javadoc)
	 * @see org.apache.imperius.spl.parser.util.TypeInfo#getType()
	 */
	public int getType() {
		return _splType;
	}

	/* (non-Javadoc)
	 * @see org.apache.imperius.spl.parser.util.TypeInfo#setType(int)
	 */
	public void setType(int type) {
		_splType = type;
	}
	
	/* (non-Javadoc)
	 * @see org.apache.imperius.spl.parser.util.TypeInfo#getIsArray()
	 */
	public boolean getIsArray() {
		return _isArray;
	}


	/* (non-Javadoc)
	 * @see org.apache.imperius.spl.parser.util.TypeInfo#setIsArray(boolean)
	 */
	public void setIsArray(boolean array) {
		_isArray = array;
	}
	
	public void copy(TypeInfo tp) 
	{
		_isArray = tp.getIsArray();
		_splType = tp.getType();
		_referenceTypeName = tp.getReferenceTypeName();
	}


	

}
