/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */
/**/

/**
 * @author Prashant Baliga <prabalig@in.ibm.com>
 *
 */

package org.apache.imperius.util;

import java.util.logging.Level;
import java.util.logging.Logger;

class SPLLoggerClass extends Logger 
{
	private static SPLLoggerClass singletonObj = null;

	private SPLLoggerClass(String name, String resourceBundleName) 
	{
		super(name, resourceBundleName);
	}

	public static SPLLoggerClass getInstance(String name,
			String resourceBundleName) 
	{
		if (singletonObj == null)
			singletonObj = new SPLLoggerClass(name, resourceBundleName);

		return singletonObj;
	}

	public void entering(String sourceClass, String sourceMethod) 
	{
		if (isLoggable(Level.FINE))
			super.entering(sourceClass, sourceMethod);
	}

	public void exiting(String sourceClass, String sourceMethod) 
	{
		if (isLoggable(Level.FINE))
			super.exiting(sourceClass, sourceMethod);
	}
	
	public void log(Level level, String msg, Object[] params) 
	{
		if (isLoggable(Level.SEVERE) || isLoggable(Level.INFO) || isLoggable(Level.WARNING))
			super.log(level, msg, params);
	}
	
	public void log(Level level, String msg) 
	{
		if (isLoggable(Level.SEVERE) || isLoggable(Level.INFO) || isLoggable(Level.WARNING))
			super.log(level, msg);
	}
	
	public void fine(String msg) 
	{
		if (isLoggable(Level.FINE))
			super.fine(msg);
	}
	
	public void severe(String msg) 
	{
		if (isLoggable(Level.SEVERE))
			super.severe(msg);
	}
}
