/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */
package org.apache.imperius.javaspl.samples.simplepolicies;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.Hashtable;
import java.util.Map;

import org.apache.imperius.javaspl.Java_SPLPolicyRuleProvider;
import org.apache.imperius.spl.parser.exceptions.SPLException;

public class RunSamples {
    
    public static String policyFolder = "resources/samples/simple/policies/";

    /**
     * @param args
     */
    public static void main(String[] args) 
    {
        if(args.length == 0)
        {   
            RunSamples.test();
        }
        else
        {
            RunSamples.testPolicy(args[0]);
        }

    }
    
    public static void _test(String policyName, Map objMap)
    {
        try 
        {
            
            Java_SPLPolicyRuleProvider jspl = Java_SPLPolicyRuleProvider.getInstance();
            String aFile = policyFolder + policyName + ".spl";
            //System.out.println("starting test");
            StringBuffer contents = new StringBuffer();
            BufferedReader input = null;
            try
            {
                input = new BufferedReader( new FileReader(aFile) );
                String line = null; 
             
                while (( line = input.readLine()) != null)
                {
                    contents.append(line);
                    contents.append(System.getProperty("line.separator"));
                }
                input.close();
            }
            catch (FileNotFoundException ex) 
            {
              ex.printStackTrace();
            }
            catch (IOException ex){
              ex.printStackTrace();
            }
            try
            {
                jspl.deletePolicy(policyName);
            }
            catch(Exception e)
            {
                
            }
            
            boolean createReturn=jspl.createPolicy(policyName, contents.toString());
            System.out.println("Policy Created : " + policyName +createReturn);
            System.out.println("");
            Object result = jspl.executePolicy(policyName, objMap);
            System.out.println("Result is " + result);
        }   
        catch (SPLException e) 
        {               // TODO Auto-generated catch block
            e.printStackTrace();
        }  
        
    }
    public static void testPolicy(String policyToExecute)
    {
        TestElement te=new TestElement();
        
        Map objMap = new Hashtable();
        objMap.put("te", te);
                
        _test(policyToExecute,objMap);      
            
                
        
    }
    
    public static void testBasicPolicy() 
    {
    
        String policyToExecute = "BasicPolicy"; 
            
            
        Map objMap = new Hashtable();
        
        SampleClass sc1 = new SampleClass(1,2);
        objMap.put("sc1", sc1);
        
        SampleClass sc2 = new SampleClass(3,4);
        objMap.put("sc2", sc2);
        
        AnotherClass ac1 = new AnotherClass(5,6);
        objMap.put("ac1", ac1);
    
        _test(policyToExecute,objMap);
        
        
    }   
    public static void test()
    {
        String [] policiesToTest = {"SimplePolicy","ComplexPolicy",
                "MultiplePolicySample","InvokeMethodSample","DateTime","MacroSample","SimpleInvokePolicy","InvokePolicySample","AccessPrivateVariablesSample"};

        for (int i=0 ; i < policiesToTest.length; i++)
        {
            String policy = policiesToTest[i];
            testPolicy(policy);
        }
        testBasicPolicy();

        
    }

    public static void test(String pFolder) {
        
        policyFolder = pFolder;
        test();
    
        
    }

}
