/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */


/**
 * @author Prashant Baliga <prabalig@in.ibm.com>
 *
 */

package org.apache.imperius.javaspl.samples.userdefinedmethods;

import java.util.List;
import java.util.Properties;
import java.util.logging.Logger;

import javax.mail.Authenticator;
import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.PasswordAuthentication;
import javax.mail.Session;
import javax.mail.Transport;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;

import org.apache.imperius.spl.core.Expression;
import org.apache.imperius.spl.core.TypeConstants;
import org.apache.imperius.spl.core.TypeInfo;
import org.apache.imperius.spl.parser.exceptions.SPLException;
import org.apache.imperius.spl.parser.util.TypeInfoImpl;
import org.apache.imperius.util.SPLLogger;


public class SendMail implements Expression 
{

	private List inputParams =null;
	private static Logger logger = SPLLogger.getSPLLogger().getLogger();


	public SendMail(List input, boolean boo)
	{
		//System.out.println(" creating sendMail action");
		this.inputParams=input;
	}
	public Object evaluate() throws SPLException 
	{
		
		Expression fromExpression=(Expression)this.inputParams.get(0);
		Object from=fromExpression.evaluate();
		System.out.println("from :"+from);


		Expression toExpression=(Expression)this.inputParams.get(1);
		Object to=toExpression.evaluate();
		System.out.println("to :"+to);


		Expression subjectExpression=(Expression)this.inputParams.get(2);
		Object subject=subjectExpression.evaluate();
		System.out.println("subject :"+subject);


		Expression messageExpression=(Expression)this.inputParams.get(3);
		Object message=messageExpression.evaluate();
		System.out.println("message :"+message);

		try {
			//System.out.println("trying to send mail");

			//System.out.println("trying to create service object ");
	        
			boolean ret=postMail((String)from,(String)to,(String)subject,(String)message);
	        System.out.println("Sent mail, return value='" + ret + "'");
	        System.out.println("");
	      } catch (Exception e) {
	    	  System.out.println("send mail exception block");
	    	  logger.severe(Thread.currentThread().getName()+" "+e.getMessage());
	            
	    	  e.printStackTrace();
	        System.out.println(e.toString());
	      }
		 
		// TODO Auto-generated method stub
		return new Boolean(true);
	}


	public boolean postMail( String from,String recipients, String subject,
			String message) throws MessagingException
			{
		//System.out.println("post mail");
		 String JAVAX_MAIL_PROTOCOL = "smtp";
		   String SMTP_HOST_NAME = "relay.de.ibm.com";
//		   String SMTP_AUTH_USER = "anonymous";
//		   String SMTP_AUTH_PWD  = "anonymous";
		   String SMTP_AUTH_ENABLED  = "false";
//		   int SEVERITY = 50;
//		   int NO_ACTION_SEVERITY = 50;
		  
		boolean debug = false;
//		Set the host smtp address
		Properties props = new Properties();
		props.put( "mail.transport.protocol", JAVAX_MAIL_PROTOCOL);
		props.put("mail.smtp.host", SMTP_HOST_NAME);
		Session session = null;
		//System.out.println("post mail1");
		if(SMTP_AUTH_ENABLED.equals("true"))
		{
			props.put("mail.smtp.auth", "true");
			Authenticator auth = new SMTPAuthenticator();
			session = Session.getDefaultInstance(props, auth);
			//System.out.println("post mail2");
		}
		else 
			session = Session.getDefaultInstance(props);
		//System.out.println("post mail3");
		session.setDebug(debug);

//		create a message
		Message msg = new MimeMessage(session);
		//System.out.println("post mail4");
		msg.setFrom(new InternetAddress(from));
		//System.out.println("post mail5");
		msg.setRecipients(Message.RecipientType.TO, InternetAddress.parse( recipients ));
		//System.out.println("post mail6");

//		Setting the Subject and Content Type
		msg.setSubject(subject);
		msg.setContent(message, "text/plain");
		//System.out.println("post mail7");
		try{
			//System.out.println("post mail8");
			Transport.send(msg);
			//System.out.println("post mail9");
		}catch(Exception e)
		{
			e.printStackTrace();
			
		}
		
		//System.out.println("messsage sent");
		return true;
		}


	public String getReferenceTypeName() throws SPLException {
		// TODO Auto-generated method stub
		return null;
	}

	public TypeInfo getType() throws SPLException {
		// TODO Auto-generated method stub
		return new TypeInfoImpl(TypeConstants.booleanType,null,false);
	}

	public boolean isArray() {
		// TODO Auto-generated method stub
		return false;
	}

	public boolean validate() throws SPLException {
		// TODO Auto-generated method stub
		return true;
	}

	
	
	private class SMTPAuthenticator extends javax.mail.Authenticator
	{
	    public PasswordAuthentication getPasswordAuthentication()
	    {
	    	 String SMTP_AUTH_USER = "anonymous";
	    	  String SMTP_AUTH_PWD  = "anonymous";
	    	  
	        String username = SMTP_AUTH_USER;
	        String password = SMTP_AUTH_PWD;
	        return new PasswordAuthentication(username, password);
	    }
	}
}
