/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.  
 */

/**
 * @author Prashant Baliga <prabalig@in.ibm.com>
 *
 */


package org.apache.imperius.javaspl.samples.windowscomputersystem;
import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;
import java.util.StringTokenizer;


public class WindowsComputerSystem {

	public float freeSpace=0;
	public long totalSpace=0;
	public String osArchitecture="";
	public String osName="";
	public String osVersion="";
	public String osPatchLevel="";
	public String userCountry="";
	public String userLanguage="";
	public String ownerName="";
	public String javaVersion="";
	
	public static long getFreeSpace() throws IOException
	  {
	    long bytesFree = -1;
	 
	    if (System.getProperty("os.name").startsWith("Windows")) {
	      // create a .bat file to run a directory command
	      File script = new File(System.getProperty("java.io.tmpdir"), "script.bat");
	      PrintWriter writer = new PrintWriter(new FileWriter(script, false));
	      writer.println("dir \"" + "." + "\"");
	      writer.close();
	 
	      // get the output from running the .bat file
	      Process p = Runtime.getRuntime().exec(script.getAbsolutePath());
	      InputStream reader = new BufferedInputStream(p.getInputStream());
	      StringBuffer buffer = new StringBuffer();
	      for (; ; ) {
	        int c = reader.read();
	        if (c == -1)
	          break;
	        buffer.append( (char) c);
	      }
	      String outputText = buffer.toString();
	      reader.close();
	      script.delete();
	 
	      // parse the output text for the bytes free info
	      StringTokenizer tokenizer = new StringTokenizer(outputText, "\n");
	      while (tokenizer.hasMoreTokens()) {
	        String line = tokenizer.nextToken().trim();
	        // see if line contains the bytes free information
	        if (line.endsWith("bytes free")) {
	          tokenizer = new StringTokenizer(line, " ");
	          tokenizer.nextToken();
	          tokenizer.nextToken();
	          bytesFree = Long.parseLong(tokenizer.nextToken().replaceAll(",",""));
	        }
	      }
	    }
	 
	    if (bytesFree == -1)
	      throw new UnsupportedOperationException(
	          "The method getFreeSpace(File) has not been implemented for this operating system.");
	 
	    return bytesFree;
	  }
	
	public WindowsComputerSystem() throws IOException
	{
		freeSpace=getFreeSpace();
		freeSpace = freeSpace/1000000000l;
		//totalSpace=javaFile.getTotalSpace();
		osArchitecture=System.getProperty("os.arch");
		osName=System.getProperty("os.name");
		osPatchLevel=System.getProperty("sun.os.patch.level");
		osVersion=System.getProperty("os.version");
		ownerName=System.getProperty("user.name");	
		userCountry=System.getProperty("user.country");
		userLanguage=System.getProperty("user.language");
		javaVersion=System.getProperty("java.specification.version");
	}

	public boolean deleteFiles(String flavour) throws IOException{
		Runtime runtime=Runtime.getRuntime();
		if(flavour.equalsIgnoreCase("TemporarySetupFiles")){
			//32
			runtime.exec("CLEANMGR /sagerun:32");
		}
		else if(flavour.equalsIgnoreCase("TemporaryInternetFiles")){
			//64
			runtime.exec("CLEANMGR /sagerun:64");
		}
		else if(flavour.equalsIgnoreCase("TemporaryFiles")){
			//128
			runtime.exec("CLEANMGR /sagerun:128");
		}
		else if(flavour.equalsIgnoreCase("RecycleBin")){
			//256
			runtime.exec("CLEANMGR /sagerun:256");
		}
		
		
		
		return true;
	}
	
	public static void main(String [] args)
	{
		WindowsComputerSystem ws;
		try {
			ws = new WindowsComputerSystem();
			System.out.println("free space" + ws.freeSpace);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
	}
}
